# Copyright 2018-2025 Marek Susčík <markuv.koutek@seznam.cz>
# This add-on is free software, licensed under the terms of the GNU General Public License (version 2). see <https://www.gnu.org/licenses/gpl-2.0.html>.

import globalPluginHandler, scriptHandler, languageHandler, addonHandler
import speech
import core
import tones # We want to hear beeps.
import eventHandler
import controlTypes
from controlTypes import OutputReason
import NVDAObjects
import winsound
from logHandler import log
import os
import os.path
import treeInterceptorHandler
import json
import winUser
import api
import cursorManager
import comtypes.client
from comtypes import COMError
from gui import guiHelper
import ui, gui, wx, re, calendar, math
import config, globalVars
import sys, winsound, config, globalVars
from urllib.request import urlopen
import urllib
import winreg
from contextlib import closing
from configobj import ConfigObj
import textInfos.offsets
from . import keyboard                                      
import collections
from . import radio as Rad
from . import tv
from . import websources
from . import kdd
import re
import inputCore

# Associate object roles to sounds.
sound_files={
controlTypes.Role.CHECKBOX : "checkbox.wav",
controlTypes.Role.RADIOBUTTON : "radiobut.wav",
controlTypes.Role.STATICTEXT : "editabletext.wav",
controlTypes.Role.EDITABLETEXT : "enter.wav",
controlTypes.Role.BUTTON : "buttons.wav",
controlTypes.Role.MENUBAR : "menuitem.wav",
controlTypes.Role.MENUITEM : "menuitem.wav",
controlTypes.Role.MENU : "menuitem.wav",
controlTypes.Role.COMBOBOX : "combobox.wav",
controlTypes.Role.LISTITEM : "lb.wav",
controlTypes.Role.GRAPHIC : "icon.wav",
controlTypes.Role.LINK : "buttons.wav",
controlTypes.Role.TREEVIEWITEM : "treeviewitem.wav",
controlTypes.Role.TAB : "tab.wav",
controlTypes.Role.TABCONTROL : "tab.wav",
controlTypes.Role.SLIDER : "trackb.wav",
controlTypes.Role.DROPDOWNBUTTON : "combobox.wav",
controlTypes.Role.CLOCK: "clock.wav",
controlTypes.Role.ANIMATION : "icon.wav",
controlTypes.Role.ICON : "icon.wav",
controlTypes.Role.IMAGEMAP : "icon.wav",
controlTypes.Role.RADIOMENUITEM : "radiobutton.wav",
controlTypes.Role.RICHEDIT : "editabletext.wav",
controlTypes.Role.SHAPE : "icon.wav",
controlTypes.Role.TEAROFFMENU : "menuitem.wav",
controlTypes.Role.TOGGLEBUTTON : "checkbox.wav",
controlTypes.Role.CHART : "icon.wav",
controlTypes.Role.DIAGRAM : "icon.wav",
controlTypes.Role.DIAL : "slider.wav",
controlTypes.Role.DROPLIST : "combobox.wav",
controlTypes.Role.MENUBUTTON : "button.wav",
controlTypes.Role.DROPDOWNBUTTONGRID : "button.wav",
controlTypes.Role.HOTKEYFIELD : "editabletext.wav",
controlTypes.Role.INDICATOR : "icon.wav",
controlTypes.Role.SPINBUTTON : "slider.wav",
controlTypes.Role.TREEVIEWBUTTON: "button.wav",
controlTypes.Role.DESKTOPICON : "icon.wav",
controlTypes.Role.PASSWORDEDIT : "editabletext.wav",
controlTypes.Role.CHECKMENUITEM : "checkbox.wav",
controlTypes.Role.SPLITBUTTON : "buttons.wav",
controlTypes.Role.DOCUMENT : "enter.wav",
}

sounds = dict() # For holding instances in RAM.
PlayRole = True
SayRole = False
DoAsist = True
SoundEcho = False
WmShortcuts = True
webtextCommand = 0
cityWeather = "Brno"
CHUNK_SEPARATOR = "  "
WmSndPath = ""
CurrentRole = controlTypes.Role.MENUITEM
CurrentWindowHandle = 0
CaretLeft = 0
CaretTop = 0
VoiceHistory = []
oldMessage = ""
prevch = ""
prevcht = 0

_addonDir = os.path.join(os.path.dirname(__file__), "..", "..")
_curAddon = addonHandler.Addon(_addonDir)
_addonSummary = _curAddon.manifest['summary']
_addonVersion = _curAddon.manifest['version']
_addonPage = _curAddon.manifest['url']

CLEANR = re.compile('<.*?>')

def cleanHtml(raw_html):
  cleantext = re.sub(CLEANR, '', raw_html)
  return cleantext

def ProcessGlobalGestures():
  from os.path import expanduser
  home = expanduser("~")
  GestFileDest = os.path.join(home, "AppData", "Roaming", "nvda", "gestures.ini")
  ByAce = False
  #is destination ini authored by ACE?
  if os.path.isfile(GestFileDest):
    with open(GestFileDest, 'r') as f:
      for line in f:
        if ('ace = kb:f13' in line): 
          ByAce = True
    #delete old ACE gestures.ini file
    if ByAce:
      os.remove(GestFileDest)

  if WmShortcuts:
    entries = {
      "globalCommands.GlobalCommands": {
        "dateTime": "kb:control+shift+t",
        "reportStatusLine": "kb:control+s+shift",
        "speechMode": "kb:alt+backspace+control",
        "toggleCurrentAppSleepMode": "kb:backspace+control+shift",
        "revertConfiguration": "kb:alt+control+d+shift",
        "reportClipboardText": "kb:control+f7+shift",
        "toggleVirtualBufferPassThrough": "kb:control+f4+shift",
        "sayAll": "kb:control+shift+space",
        "passNextKeyThrough": "kb:control+delete+shift",
        "say_battery_status": "kb:alt+b+control+shift",
      }
    }
    inputCore.manager.localeGestureMap.update(entries)
    entries = {
      "browseMode.BrowseModeTreeInterceptor": {
        "previousTable": "kb:l+shift",
        "previousList": "kb:s+shift",
        "previousComboBox": "kb:r+shift",
        "previousNotLinkBlock": "kb:shift+t",
        "previousRadioButton": "kb:p+shift",
        "elementsList": "kb:control+o+shift",
        "nextCheckBox": "kb:z",
        "nextNotLinkBlock": "kb:t",
        "nextRadioButton": "kb:p",
        "nextList": "kb:s",
        "nextComboBox": "kb:r",
        "nextTable": "kb:l",
        "nextHeading": "kb:n",
        "previousHeading": "kb:n+shift",
        "previousCheckBox": "kb:shift+z",
      }
    }
    inputCore.manager.localeGestureMap.update(entries)
    entries = {
      "cursorManager.CursorManager": {
        "findPrevious": "kb:f3+shift",
        "find": "kb:control+f",
        "findNext": "kb:f3",
      }
    }
    inputCore.manager.localeGestureMap.update(entries)

class InfoTextFrame(wx.Frame):

  def __init__(self, parent, winTitle, winText):
    super().__init__(
      parent,
      title=winTitle
    )
    self.Maximize()
    self.Bind(wx.EVT_CLOSE, self.onClose)
    mainSizer = wx.BoxSizer(wx.VERTICAL)
    self.outputCtrl = wx.TextCtrl(self, wx.ID_ANY, style=wx.TE_MULTILINE | wx.TE_READONLY|wx.TE_RICH)
    self.outputCtrl.Bind(wx.EVT_KEY_DOWN, self.onOutputKeyDown)
    mainSizer.Add(self.outputCtrl, proportion=1, flag=wx.EXPAND)
    self.SetSizer(mainSizer)
    mainSizer.Fit(self)

    self.outputCtrl.SetValue(winText)

    self.outputCtrl.SetFocus()

  def onClose(self, evt):
    self.Destroy()

  def onOutputKeyDown(self, evt):
    key = evt.GetKeyCode()
    if key == wx.WXK_ESCAPE:
      self.Close()
      return
    evt.Skip()


class OkoSettingsDialog(wx.Dialog):

  _instance = None

  def __new__(cls, *args, **kwargs):
    # Make this a singleton.
    if OkoSettingsDialog._instance is None:
      return super(OkoSettingsDialog, cls).__new__(cls, *args, **kwargs)
    return OkoSettingsDialog._instance

  def __init__(self, parent):
    if OkoSettingsDialog._instance is not None:
      return
    
    OkoSettingsDialog._instance = self
    # The title of a dialog.
    super(OkoSettingsDialog, self).__init__(parent, title=_(u"Nastavení pro OKO"))

    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self,orientation=wx.VERTICAL)

    # The play role sounds checkbox
    self.playRoleSoundCheck = wx.CheckBox(self, -1, _(u"Přehrát &zvuk pro typ"))
    self.playRoleSoundCheck.SetValue (PlayRole)
    sHelper.addItem(self.playRoleSoundCheck)

    # The speak type checkbox
    self.speakTypeCheck = wx.CheckBox(self, -1, _(u"&Ohlásit typ"))
    self.speakTypeCheck.SetValue (SayRole)
    sHelper.addItem(self.speakTypeCheck)

    # The use Winmonitor shortcuts checkbox
    self.useWmShortcutsCheck = wx.CheckBox(self, -1, _(u"Používat klávesové zkratky &Winmonitoru"))
    self.useWmShortcutsCheck.SetValue (WmShortcuts)
    sHelper.addItem(self.useWmShortcutsCheck)

    # The run Asistent shortcut checkbox
    self.runAsistShortcutCheck = wx.CheckBox(self, -1, _(u"Spouštět &Asistenta na Ctrl+Shift+F11"))
    self.runAsistShortcutCheck.SetValue(DoAsist)
    sHelper.addItem(self.runAsistShortcutCheck)

    # The typing echo sounds checkbox
    self.typingEchoSoundsCheck = wx.CheckBox(self, -1, _(u"Zvuk při &psaní"))
    self.typingEchoSoundsCheck.SetValue (SoundEcho)
    sHelper.addItem(self.typingEchoSoundsCheck)

    # The WebText command combobox
    wcChoices = ['Ohlásit aktuální teplotu', 'Zobrazit předpověď počasí']
    self.webtextCommandCombo = sHelper.addLabeledControl(_("Po stisku Ctrl+Shift+X"), wx.Choice, choices=wcChoices)
    self.webtextCommandCombo.SetSelection(webtextCommand)

    # the city editbox
    self.cityWeatherTextEdit = sHelper.addLabeledControl(_("Město pro počasí"), wx.TextCtrl)
    self.cityWeatherTextEdit.SetValue(cityWeather)

    #button row
    sHelper_Buttons = guiHelper.BoxSizerHelper(self, orientation=wx.VERTICAL)

    # The label of an OK button 
    self.okButton = wx.Button(self, label=_(u"&OK"))
    self.okButton.Bind(wx.EVT_BUTTON, self.onOk)
    self.AffirmativeId = self.okButton.Id
    self.okButton.SetDefault()
    sHelper_Buttons.addItem(self.okButton)

    # The label of a Cancel button 
    cancelButton = wx.Button(self, wx.ID_CLOSE, label=_(u"&Zrušit"))
    cancelButton.Bind(wx.EVT_BUTTON, lambda evt: self.Close())
    sHelper_Buttons.addDialogDismissButtons(cancelButton)
    self.Bind(wx.EVT_CLOSE, self.onCancel)
    self.EscapeId = wx.ID_CLOSE

    sHelper.addItem(sHelper_Buttons, flag = wx.CENTER)

    mainSizer.Add(sHelper.sizer, flag=wx.ALL, border=guiHelper.BORDER_FOR_DIALOGS)
    mainSizer.Fit(self)
    self.Sizer = mainSizer
    self.playRoleSoundCheck.SetFocus()
    self.CentreOnScreen()

  def onOk(self, evt):
    # Load settings
    SetFile = os.path.join(os.path.dirname(__file__), "files", "setting.json")
    with open(SetFile, 'r') as f:
      config = json.load(f)
    # Assign values to config
    config['PlayRole'] = self.playRoleSoundCheck.GetValue()
    config['SayRole'] = self.speakTypeCheck.GetValue()
    config['Gestures'] = self.useWmShortcutsCheck.GetValue()
    config['DoAsist'] = self.runAsistShortcutCheck.GetValue()
    config['SoundEcho'] = self.typingEchoSoundsCheck.GetValue()
    config['WebtextCommand'] = self.webtextCommandCombo.GetSelection()
    config['CityWeather'] = self.cityWeatherTextEdit.GetValue()
    # Save the settings
    SetFile = os.path.join(os.path.dirname(__file__), "files", "setting.json")
    with open(SetFile, 'w') as f:
      json.dump(config, f)

    # Apply settings
    NvdaWillRestart = False
    NvdaNeedsRestart = False
    global PlayRole
    PlayRole = config['PlayRole']
    global SayRole
    SayRole = config['SayRole']
    global DoAsist
    DoAsist = config['DoAsist']
    global webtextCommand
    webtextCommand = config['WebtextCommand']
    global cityWeather
    cityWeather = config['CityWeather']
    if self.cityWeatherTextEdit.Value:
      # save the city to file
      FilePath = os.path.join(os.path.dirname(__file__), "files", "city.txt")
      with open(FilePath, 'wb') as f:
        f.write(self.cityWeatherTextEdit.Value.encode('windows-1250'))
    global SoundEcho
    tmpSoundEcho = SoundEcho
    SoundEcho = config['SoundEcho']
    if SoundEcho and not tmpSoundEcho:
      NvdaNeedsRestart = True
    elif not SoundEcho and tmpSoundEcho:
      keyboard.unhook_all()
    global WmShortcuts
    tmpWmShortcuts = WmShortcuts
    WmShortcuts = config['Gestures']
    if tmpWmShortcuts != WmShortcuts:
      ProcessGlobalGestures()
      NvdaNeedsRestart = True
    if NvdaNeedsRestart:
      # NVDA needs to be restarted
      restartMessage = _(
        u"Byla provedena nastavení, která vyžadují restartování NVDA. "
        u"Chcete provést restart nyní?"
      )
      restartTitle = _(u"Restart NVDA")
      result = gui.messageBox(
        message=restartMessage,
        caption=restartTitle,
        style=wx.YES | wx.NO | wx.ICON_WARNING
      )
      if wx.YES == result:
        NvdaWillRestart = True        
    self.Destroy()
    OkoSettingsDialog._instance = None
    if NvdaWillRestart:
      core.restart()

  def onCancel(self, evt):
    self.Destroy()
    OkoSettingsDialog._instance = None

class GlobalPlugin(globalPluginHandler.GlobalPlugin):
  # category for addon commands in input gestures dialog
  scriptCategory = _addonSummary

  def __init__(self, *args, **kwargs):
    super(GlobalPlugin, self).__init__(*args, **kwargs)

    #read settings
    if not os.path.isfile(os.path.join(os.path.dirname(__file__), "files", "setting.json")):
      config = {"PlayRole": True, "SayRole": False, "Gestures": True, "DoAsist": True, "SoundEcho": False, "WebtextCommand": 0, "CityWeather": "Brno"}
      SetFile = os.path.join(os.path.dirname(__file__), "files", "setting.json")
      with open(SetFile, 'w') as f:
        json.dump(config, f)    

    SetFile = os.path.join(os.path.dirname(__file__), "files", "setting.json")
    with open(SetFile, 'r') as f:
      config = json.load(f)
    
    global PlayRole
    PlayRole = config['PlayRole']
    global SayRole
    SayRole = config['SayRole']
    global DoAsist
    DoAsist = config['DoAsist']
    global SoundEcho
    SoundEcho = config['SoundEcho']
    global WmShortcuts
    WmShortcuts = config['Gestures']
    global webtextCommand
    webtextCommand = config['WebtextCommand']
    global cityWeather
    cityWeather = config['CityWeather']

    # Keyboard sound echo
    if SoundEcho:
      keyboard.on_press(self.pressedKeys)

    #global gestures
    ProcessGlobalGestures()
    global WmSndPath
    WmSndPath = os.path.join(os.path.dirname(__file__), "files", "sound")
    
    #function hooks
    self._NVDA_speak = speech.speech.speak
    speech.speech.speak = self._hook_speak
    self._NVDA_getPropertiesSpeech = speech.speech.getPropertiesSpeech
    speech.speech.getPropertiesSpeech = self._hook_getPropertiesSpeech

    self.make_sound_objects()
    
    self.ReadConfig()
    self.createMenu()
    if self.toUpgrade: self.onUpgrade()
        

  def terminate(self):
    self.removeMenu()
    super().terminate()

  def createMenu(self):
    self.OkoMenu = wx.Menu()
    modulesSubmenu = wx.Menu()
    self.OkoMenu.AppendSubMenu(modulesSubmenu, _(u"Rozšíření"), _("Nabídka rozšiřujících modulů"))
    item = modulesSubmenu.Append(wx.ID_ANY, _(u"Knihovny"), _(u"Otevře prostředí pro obsluhu různých knihoven"))
    gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, lambda event: self.script_OpenKdd(None), item)
    item = modulesSubmenu.Append(wx.ID_ANY, _(u"Moje knihovna"), _(u"Otevře Vaši knihovnu s dříve staženými tituly"))
    gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, lambda event: self.script_OpenLibrary(None), item)
    item = modulesSubmenu.Append(wx.ID_ANY, _(u"Poslední zvolené rádio"), _(u"Spustí nebo zastaví poslední zvolenou rádiovou stanici"))
    gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, lambda event: self.script_PlayStopRadio(None), item)
    item = modulesSubmenu.Append(wx.ID_ANY, _(u"Program televizních a rozhlasových stanic"), _(u"Otevře program televizních a rozhlasových stanic"))
    gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onTv, item)
    item = modulesSubmenu.Append(wx.ID_ANY, _(u"Rádio"), _(u"Otevře seznam internetových rádií"))
    gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onRadioGroups, item)
    item = modulesSubmenu.Append(wx.ID_ANY, _(u"Webové zdroje"), _(u"Otevře Webové zdroje"))
    gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onWebSources, item)
    item = self.OkoMenu.Append(wx.ID_ANY, _(u"Nastavení"), _(u"Otevře dialog pro nastavení"))
    gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onSetting, item)
    self.UpgradeAddonItem = self.OkoMenu.Append(wx.ID_ANY, _(u"Kontrola aktualizace"), _("Zkontroluje, zda je dostupná aktualizovaná verze"))
    gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onUpgrade, self.UpgradeAddonItem)
    item = self.OkoMenu.Append(wx.ID_ANY, _(u"Nápověda"), _("Zobrazí nápovědu"))
    gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onHelp, item)
    item = self.OkoMenu.Append(wx.ID_ANY, _(u"O doplňku"), _("Zobrazí informace o doplňku OKO"))
    gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onAbout, item)
    self.menu = gui.mainFrame.sysTrayIcon.menu.GetMenuItems()[0].GetSubMenu()
    self.OkoSubmenu = gui.mainFrame.sysTrayIcon.menu.InsertMenu(2, wx.ID_ANY, u"{summary} {version}".format(summary=_addonSummary, version=_addonVersion), self.OkoMenu)

  def removeMenu(self):
    gui.mainFrame.sysTrayIcon.menu.DestroyItem(self.OkoSubmenu)

  def ReadConfig(self, singleRead = None):
    """.ini"""
    #Default values
    if not singleRead:
      self.toUpgrade = True

  def onAbout(self, evt):
    dlgMessage = _addonSummary + "\nverze " + _addonVersion + "\nCopyright (c) 2018-2025 Marek Susčík"
    dlgTitle = "Informace o doplňku"
    wx.CallAfter(gui.messageBox, dlgMessage, dlgTitle, wx.OK)

  def onSetting(self, evt = None):
    self.showOkoSettings()

  def onHelp(self, evt = None):
    from . import addonDoc
    addonDoc.openDocPath()

  def onUpgrade(self, evt = None):
    if "systemConfig" in _addonDir: return #does not check and return if the wp is run from systemConfig folder

    def NoteDialog(message = "", title = "", newVersion = "", buttons = None):
      sn = str(buttons)
      gui.mainFrame.prePopup()
      d = MyDialog(gui.mainFrame, message, title, newVersion, self.UpgradeAddonItem, buttons)
      d.Show()
      gui.mainFrame.postPopup()

    message, ask = "", False

	#determines the title of the dialog box
    title = _(u'Upozornění')
    if evt: title = evt.GetEventObject().GetLabel(evt.GetId()).rstrip('.')
    
	#reads the version from the page of the addon 
    data = self.GetUrlData('https://suscik.cz/nvda/oko/ver.txt')
    try:
      data = data.decode()
    except: pass
    if not data or data == "no connect":
      if evt:
        NoteDialog(_("Nelze se připojit!"), title)
      return
    try:
      newVersion = data
    except: newVersion = ""
    
    newVersionValue = False
    if (int(newVersion.split(".")[0]) > int(_addonVersion.split(".")[0])):
        newVersionValue = True
    else:
        if (int(newVersion.split(".")[1]) > int(_addonVersion.split(".")[1])):
            newVersionValue = True

	#checks to see if a new version is available
    if newVersion and newVersionValue:

      #go ahead with update
      message = '%s %s %s %s.\n%s' % (
        _(u"Je k dispozici aktualizace"), _addonSummary, _("verze"), newVersion,
        _(u"Chcete ji stáhnout?"))
      ask= view = True
    else:
      if evt: return NoteDialog(_(u"žádná aktualizace není k dispozici."), title)

    if ask:
      return NoteDialog(message, title, newVersion, True)

  def GetUrlData(self, address):
    """Returns the contents of a web page"""
    e = data = ""
    try:
      with closing(urlopen(address)) as response:
        data = response.read()
    except Exception as e:
      if "CERTIFICATE_VERIFY_FAILED" in repr(e):
		#retry using ssl
        import ssl
        data = "no connect"
        gcontext = ssl.SSLContext(ssl.PROTOCOL_TLSv1)
        try:
          e, data = "", None
          with contextlib.closing(urllib2.urlopen(address, context=gcontext)) as response:
            data = response.read()
        except Exception as e: pass
      elif "failed" in repr(e): data = "no connect"

      if e:
        e = str(e)
    return data

  def make_sound_objects(self):
    global WmSndPath
    for key, value in sound_files.items():
      path = os.path.join(WmSndPath, value)
      sounds[key] = path

  def event_gainFocus(self, obj, nextHandler):
    global CurrentRole
    global CurrentWindowHandle
    global WmSndPath
    global PlayRole
    try:
      role = obj.role
    except:
      role = None
    if role in sounds and PlayRole:
      if obj.role == controlTypes.Role.MENUITEM or obj.role == controlTypes.Role.LISTITEM:
        if obj.role != CurrentRole or obj.windowHandle != CurrentWindowHandle:
          fSound = sounds[obj.role]
          if os.path.isfile(fSound): winsound.PlaySound(fSound, winsound.SND_FILENAME)
      
      elif obj.role == controlTypes.Role.CHECKBOX and controlTypes.State.CHECKED in obj.states: 
        self.playSound("checked.wav")
      elif obj.role == controlTypes.Role.TOGGLEBUTTON and controlTypes.State.PRESSED in obj.states: 
        self.playSound("checked.wav")
      
      elif obj.role == controlTypes.Role.DOCUMENT:
        if api.getFocusObject().windowClassName == "RichEditD2DPT":
          self.playSound("enter.wav")
      
      else:
        fSound = sounds[obj.role]
        if os.path.isfile(fSound):
          winsound.PlaySound(fSound, winsound.SND_FILENAME)
      
    #save for listitem sound handling
    CurrentRole = role
    CurrentWindowHandle = obj.windowHandle
    
    try:
      nextHandler()
    except:
      pass
   
  def event_show(self, obj, nextHandler):
    nextHandler()
    
  def _hook_speak(self,speechSequence,symbolLevel=None,priority=None):
    if isinstance(speechSequence, collections.abc.Generator):
      speechSequence = [i for i in speechSequence]
    for item in speechSequence:
      #VoiceHistory
      if isinstance(item, str):
          global VoiceHistory
          #set as top
          VoiceHistory.insert(0,item)
          #trim tail
          if len(VoiceHistory) > 50:
              VoiceHistory.pop(50)
    return self._NVDA_speak(speechSequence,symbolLevel,priority)

  def _hook_getPropertiesSpeech(self, reason=OutputReason.QUERY, *args, **kwargs):
    if (reason == OutputReason.FOCUS):
      role = kwargs.get('role', None)
      if role and SayRole==False:
  		  #NVDA will not announce roles if we put it in as _role.
        kwargs['_role'] = kwargs['role']
        del kwargs['role']
    return self._NVDA_getPropertiesSpeech(reason, *args, **kwargs)

  def pressedKeys(self, e):
    global prevch
    global prevcht
    ch = e.name
    cht = e.time
    curFocus=api.getFocusObject()
    curApp=curFocus.appModule
    if curApp.sleepMode:
      return
    if config.conf["keyboard"]["speakTypedCharacters"]:
      return
    if ch==u"\b":
      prevch = ch
      prevcht = cht
      return
    elif ch=="enter":
      # Enter is send twice, so only first will be played
      t = cht - prevcht
      if t>0.1:
        # second will be played only when it comes after 100 ms
        self.playKey("ENTERKEY.WAV")
    elif ch=="esc":
      self.playKey("ESCAPE.WAV")
    elif ch=="space":
      self.playKey("Mezera.wav")
    elif ch=="decimal":
      self.playKey("CHARS.wav")
    elif len(ch)>1:
      prevch = ch
      prevcht = cht
      return
    elif ch.islower():
      self.playKey("LETTER.WAV")
    elif ch.isupper():
      self.playKey("BIGLET.WAV")
    elif ch.isnumeric():
      self.playKey("NUM" + ch + ".WAV")
    else:
      self.playKey("CHARS.WAV")
    prevch = ch
    prevcht = cht

  @scriptHandler.script(
    description=_(
      "Zobrazí seznam otevřených oken."
    ),
    gesture="kb:control+alt+shift+A"
  )

  def script_ShowWindows(self, gesture):
    WinEnumApp = os.path.join(os.path.dirname(__file__), "files", "enumwnd.exe")
    os.startfile(WinEnumApp)


  @scriptHandler.script(
    description=_(
      "Zobrazí seznam ikon na systémové liště."
    ),
    gesture="kb:control+shift+A"
  )

  def script_ShowTrayIcons(self, gesture):
    WinEnumApp = os.path.join(os.path.dirname(__file__), "files", "enumwnd.exe")
    import subprocess
    subprocess.Popen([WinEnumApp, 'tray'])


  def currentTemp(self):
    self.playSound("buttons.wav")
    global cityWeather
    plainTxt = ""
    weatherPoint = cityWeather
    weatherPoint = weatherPoint.strip()
    weatherPoint = weatherPoint.lower()
    weatherPoint = weatherPoint.replace(' ', '-')
    accent = "áéíóúůěščřžýďťň"
    noaccent = "aeiouuescrzydtn"
    rx = ""
    for x in weatherPoint:
      if x in accent:
        posx = accent.find(x)
        rx = rx + noaccent[posx]
      else:
        rx = rx + x
    weatherPoint = rx
    query_string = urllib.parse.quote_plus(weatherPoint, safe='')
    url = "https://pocasi.seznam.cz/" + query_string
    request = urllib.request.Request(url)
    request.add_header('User-Agent', 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/57.0.2987.133 Safari/537.36')
    request.add_header('Cookie', 'CONSENT=YES+cb.20210418-17-p0.it+FX+917;')
    weatherOk = True
    try:
      response = urllib.request.urlopen(request)
    except:
      weatherOk = False
    if weatherOk:
      content = response.read()
      output = content.decode()
      maxStrLength = 7
      plainTxt = cleanHtml(output)
      startIndex = plainTxt.rfind("Česko")
      if startIndex == -1:
        weatherOk = False
      else:
        startIndex = startIndex + len("Česko") - 1
        endIndex = plainTxt.find("C", startIndex)
        if endIndex == -1:
          weatherOk = False
        else:
          curTempStr = plainTxt[startIndex+1:endIndex]
          if len(curTempStr) > maxStrLength:
            weatherOk = False
    if weatherOk:
      ui.message(curTempStr)
    else:
      for s in re.finditer('Česko', plainTxt):
        startIndex = s.start() + len("Česko") - 1
        endIndex = plainTxt.find("C", startIndex)
        if endIndex == -1:
          break
        else:
          curTempStr = plainTxt[startIndex+1:endIndex]
          if len(curTempStr) > maxStrLength:
            continue
          else:
            weatherOk = True
            break
      if weatherOk:
        ui.message(curTempStr)
      else:
        ui.message("Nepodařilo se získat informaci o aktuální venkovní teplotě.")

  def weatherForecast(self):
    self.playSound("buttons.wav")
    import subprocess
    WinEnumApp = os.path.join(os.path.dirname(__file__), "files", "pocasi.vbs")
    os.startfile(WinEnumApp)


  @scriptHandler.script(
    description=_(
      "Zobrazí informace z webu podle aktuálního  nastavení."
    ),
    gesture="kb:control+shift+X"
  )

  def script_RunWebText(self, gesture):
    if webtextCommand == 1:
      self.weatherForecast()
    else:
      self.currentTemp()


  @scriptHandler.script(
    description=_(
      "Ohlásí detailní informace o aktivním prvku."
    ),
    gesture="kb:control+shift+F3"
  )

  def script_ActiveObjectDetail(self, gesture):
    focusObject=api.getFocusObject()
    vbuf = focusObject.treeInterceptor
    if vbuf is not None:
      focusObject = api.getNavigatorObject()
    if (focusObject.role == controlTypes.Role.LISTITEM):
      Message = self.Details_ListItem(focusObject)
    elif (focusObject.role == controlTypes.Role.TREEVIEWITEM):
      Message = self.Details_TreeItem(focusObject)
    elif (focusObject.role == controlTypes.Role.CHECKBOX):
      Message = self.Details_CheckBox(focusObject)
    elif (focusObject.role == controlTypes.Role.EDITABLETEXT) or (focusObject.role == controlTypes.Role.DOCUMENT and api.getFocusObject().windowClassName == "RichEditD2DPT"):
      Message = self.Details_EditableText(focusObject)
    elif (focusObject.role == controlTypes.Role.BUTTON):
      Message = self.Details_Button(focusObject)
    elif (focusObject.role == controlTypes.Role.LINK):
      Message = self.Details_Link(focusObject)
    else:
      Message = controlTypes.Role(focusObject.role).displayString
      if focusObject.name:
        Message += " " + focusObject.name
    ui.message(Message)

 
  @scriptHandler.script(
    description=_(
      "Ohlásí aktivní okno a prvek."
    ),
    gesture="kb:control+shift+F2"
  )

  def script_ActiveObject(self, gesture):
    #---deploy nvda info---
    windowHandle=winUser.getGUIThreadInfo(None).hwndActive
    title = winUser.getWindowText(windowHandle)
    focusObject = api.getFocusObject()
    vbuf = focusObject.treeInterceptor
    if vbuf is not None:
      focusObject = api.getNavigatorObject()
    srole = controlTypes.Role(focusObject.role).displayString
    if srole == None:
      srole = ""
    if focusObject.name == None:
      sname = ""
    else:
      sname = focusObject.name
    ui.message(sname + " " + srole + u' v okně ' + title)

  def playKey(self, fSoundName):
    global WmSndPath
    fSound = os.path.join(WmSndPath, fSoundName)
    if os.path.isfile(fSound): winsound.PlaySound(fSound, winsound.SND_ASYNC)

  def playSound(self, fSoundName):
    global WmSndPath
    fSound = os.path.join(WmSndPath, fSoundName)
    if os.path.isfile(fSound): winsound.PlaySound(fSound, winsound.SND_FILENAME)

  def Details_Link(self,obj):
    #get address
    address = obj.value
    mess = address
    #save to clipboard
    api.copyToClip(address)
    return mess

  def Details_ListItem(self,obj):
    mess = ""
    className = obj.windowClassName
    currentModule = obj.appModule
    currentAppName = currentModule.appName
    #explorer, w7
    if (className == "DirectUIHWND"):
      #multiple selection?
      objParent = obj.parent
      if (objParent.role == controlTypes.Role.LIST):
        cntSel = 0
        cntItems = 0
        for child in objParent.children:
          cntItems += 1
          if controlTypes.State.SELECTED in child.states: cntSel += 1
        if (cntSel>1): mess += u" vybráno " + str(cntSel) + u" položek" 
      
      #get LV subfields
      for child in obj.children:
        if (child.role == controlTypes.Role.EDITABLETEXT):
          valueString = child.value
          if (valueString == None): valueString = "" 
          if (len(valueString)>0):
            # scan, messy chars are asccii 8206, 8207, ...
            outStr = ""
            for c in valueString:
              if (ord(c)<1000): outStr += c
            mess += " " + outStr
    else:
      #multiple selection?
      objParent = obj.parent
      if (objParent.role == controlTypes.Role.LIST):
        cntSel = 0
        cntSelIndex = 0
        cntItems = 0
        
        for child in objParent.children:
          cntItems += 1
          if controlTypes.State.SELECTED in child.states: 
              cntSel += 1
              
          if controlTypes.State.FOCUSED in child.states: 
              cntSelIndex = cntItems
        if (cntSel>1): mess += u" vybráno " + str(cntSel) + u" položek" 
      if (mess == ""):
          mess = obj.name + u" položka " + str(cntSelIndex) + u" celkem " + str(cntItems)
 

    return mess

  def Details_TreeItem(self,obj):
    mess = ""
    if controlTypes.State.EXPANDED in obj.states: mess += " rozbaleno"
    if controlTypes.State.COLLAPSED in obj.states: mess += " sbaleno"
    objParent = obj.parent
    nestingLevel = 0
    while objParent and (objParent.role == controlTypes.Role.TREEVIEWITEM):
      objParent = objParent.parent
      nestingLevel += 1
    if (nestingLevel>1): mess += u" úroveň " + str(nestingLevel) 
    return mess

  def Details_CheckBox(self,obj):
    mess = ""
    if controlTypes.State.CHECKED in obj.states: 
      mess += u" zaškrtnuto"
    else:
      mess += u" nezaškrtnuto"
    return mess

  def Details_Button(self,obj):
    mess = ""
    key = obj.keyboardShortcut
    if (key != None) and (len(key)): 
      mess = u" klávesa " + key
    else:
      mess = controlTypes.Role(obj.role).displayString + " " + obj.name 
    return mess

  def Details_EditableText(self, obj):
    mess = "kurzor"

    try:
      info=obj.makeTextInfo(textInfos.POSITION_SELECTION)
      if not info.isCollapsed:
        # if there is selected text, then there is a value and we do not report placeholder
        mess = u"vybráno " + str(len(info.text)) + u" znaků, " + info.text
      else:
        #get caret position, see textinfos.init
        InfoLine=obj.makeTextInfo(textInfos.POSITION_CARET)
        InfoCaret=obj.makeTextInfo(textInfos.POSITION_CARET)
        InfoLine.expand(textInfos.UNIT_LINE)
        InfoCaret.setEndPoint(InfoLine,"startToStart")

        CharsOnLine = len(InfoLine.text)
        PosOnLine = len(InfoCaret.text) + 1
        if (PosOnLine > CharsOnLine):
            mess = u"konec řádku, znaků: " + str(CharsOnLine) + ", "
        else:
            mess = u"znak: " + str(len(InfoCaret.text)+1) + " / " + str(CharsOnLine) + ", "

        #get all lines
        if obj.role == controlTypes.Role.DOCUMENT:
          newlineStr = "\r"
        else:
          newlineStr = "\n"

        allLinesInfo = obj.makeTextInfo(textInfos.POSITION_ALL)
        lines = allLinesInfo.text.split(newlineStr)
        allLinesCount = len(lines)

        #get current line, range from start to current line
        allLinesInfo.setEndPoint(InfoLine,"endToStart")
        linesToCaret = allLinesInfo.text.split(newlineStr)
        currentLineCount = len(linesToCaret)
        mess += u" řádek: " + str(currentLineCount) + " / " + str(allLinesCount) 
    except Exception as e:
      newInfo=obj.makeTextInfo(textInfos.POSITION_ALL)
      mess = u"chyba " + newInfo.text
    return mess

  @scriptHandler.script(
    description=_(
      "Zopakuje naposledy ohlášenou informaci. "
      "Po dvojitém stisku bude poslední hlášení vyhláskováno, po trojím stisku bude zkopírováno do schránky."
    ),
    gesture="kb:control+shift+H"
  )

  def script_RepeatLastMessage(self, gesture):
    global VoiceHistory
    global oldMessage
    if scriptHandler.getLastScriptRepeatCount()==1:
      #two presses: spell last message
      speech.speakSpelling(VoiceHistory[0])
    elif scriptHandler.getLastScriptRepeatCount()==2:
      #threee presses: copy last message to clipboard
      api.copyToClip(oldMessage)
      ui.message(u'Do schránky ' + oldMessage)
    else:
      #one press: repeat last message
      ui.message(VoiceHistory[0])
      oldMessage = VoiceHistory[0]


  def onInfoText(self, evt, title, mess):
    gui.mainFrame.prePopup()
    d = InfoTextFrame(gui.mainFrame, title, mess)
    d.Show()
    gui.mainFrame.postPopup()

  @scriptHandler.script(
    description=_(
      "Otevře text obsahující 50 posledních hlášení."
    ),
    gesture="kb:control+shift+F8"
  )

  def script_VoiceList(self, gesture):
    global VoiceHistory
    voiceMessage = ""
    for item in VoiceHistory:
      sOut = item
      voiceMessage = voiceMessage + sOut + "\n"
    wx.CallAfter(self.onInfoText, None, "Historie hlasu", voiceMessage)


  @scriptHandler.script(
    description=_(
      "Spustí aplikaci Asistent, je-li nainstalována."
    ),
    gesture="kb:control+shift+F11"
  )

  def script_LaunchAsist(self, gesture):
    #check settings
    global DoAsist
    if (DoAsist == True):
      #play
      self.playSound("buttons.wav")
      hKey = winreg.OpenKey(winreg.HKEY_CURRENT_USER,"Software\\Elvos\\Common")
      result = winreg.QueryValueEx(hKey, "Path")
      aspath = result[0]
      AsistApp = os.path.join(aspath, "asist.exe")
      os.startfile(AsistApp)
    else:
      gesture.send()


  @scriptHandler.script(
    description=_(
      "Přidá vybraný text do schránky."
    ),
    gesture="kb:control+shift+C"
  )

  def script_AddToClipBoard(self, gesture):     
    obj=api.getFocusObject()
    treeInterceptor=obj.treeInterceptor
    if isinstance(treeInterceptor,treeInterceptorHandler.DocumentTreeInterceptor) and not treeInterceptor.passThrough:
      obj=treeInterceptor
    try:
      info=obj.makeTextInfo(textInfos.POSITION_SELECTION)
    except (RuntimeError, NotImplementedError):
      info=None
    if not info or info.isCollapsed:
      ui.message(u'není nic vybráno')
    else:
      clipstring = api.getClipData() + "\r\n" + info.text
      api.copyToClip(clipstring)
      ui.message(u'přidáno do schránky: ' + info.text)
    

  @scriptHandler.script(
    description=_(
      "Ohlásí obsah schránky. Po dvojitém stisku bude  obsah schránky zobrazen v textovém poli."
    ),
    gesture="kb:control+shift+F7"
  )

  def script_AnnounceClipboard(self, gesture):
    try:
      clipstring = api.getClipData()
    except:
      clipstring = ""
    if not clipstring or not isinstance(clipstring,str):
      WinEnumApp = os.path.join(os.path.dirname(__file__), "files", "enumwnd.exe")
      import subprocess
      subprocess.Popen([WinEnumApp, 'clip_data'])
      return
    if scriptHandler.getLastScriptRepeatCount()==1:
      #two presses: open clipboard content in text window
      sOut = clipstring
      wx.CallAfter(self.onInfoText, None, "Schránka", sOut)
    else:
      #one press: say clipboard content
      if (len(clipstring) == 0):
        ui.message(u"Schránka je prázdná.")
      elif (len(clipstring) == 1):
        speech.speakMessage(u've schránce je: ')
        speech.speakSpelling(clipstring)
      else:
        ui.message(u've schránce je: ' + clipstring)


  def onWebSources(self, evt):
    gui.mainFrame.prePopup()
    d = websources.WebSourcesDialog(gui.mainFrame)
    d.Show()
    gui.mainFrame.postPopup()

  @scriptHandler.script(
    description=_(
      "Spustí funkci Webové zdroje umožňující získávání informací z nejrůznějších webových zdrojů."
    ),
    gesture="kb:control+shift+Z"
  )

  def script_WebSources(self, gesture):
    wx.CallAfter(self.onWebSources, None)


  def onRadioGroups(self, evt):
    gui.mainFrame.prePopup()
    d = Rad.RadioGroupsDialog(gui.mainFrame)
    d.Show()
    gui.mainFrame.postPopup()

  @scriptHandler.script(
    description=_(
      "Spustí funkci Internetová rádia umožňující přehrávání internetových stanic."
    ),
    gesture="kb:control+shift+R"
  )

  def script_OpenRadio(self, gesture):
    if not Rad.Radio.isAllowed:
      ui.message("Rádio nelze spustit, protože nebyl nalezen přehrávač VLC.")
    else:
      wx.CallAfter(self.onRadioGroups, None)


  @scriptHandler.script(
    description=_(
      "Zastaví či znovu spustí dříve zvolenou stanici internetového rádia."
    ),
    gesture="kb:control+alt+R"
  )

  def script_PlayStopRadio(self, gesture):
    if not Rad.Radio.isAllowed:
      ui.message("Rádio nelze spustit, protože nebyl nalezen přehrávač VLC.")
      return
    if Rad.Radio.isPlaying(self):
      Rad.Radio.stop(self)
    else:
      Rad.Radio.play(self)


  def onTv(self, evt):
    gui.mainFrame.prePopup()
    d = tv.StationsDialog(gui.mainFrame)
    d.Show()
    gui.mainFrame.postPopup()

  @scriptHandler.script(
    description=_(
      "Otevře program televizních a rozhlasových stanic."
    ),
    gesture="kb:control+alt+P"
  )

  def script_OpenTv(self, gesture):
    wx.CallAfter(self.onTv, None)


  def onKdd(self, evt, startLibrary):
    gui.mainFrame.prePopup()
    d = kdd.KddMainDialog(gui.mainFrame, startLibrary)
    d.Show()
    gui.mainFrame.postPopup()

  @scriptHandler.script(
    description=_(
      "Spustí modul pro práci s Knihovnami."
    ),
    gesture="kb:control+alt+K"
  )

  def script_OpenKdd(self, gesture):
    wx.CallAfter(self.onKdd, None, False)

  @scriptHandler.script(
    description=_(
      "Otevře vaši knihovnu s uloženými tituly z Knihovny digitálních dokumentů."
    ),
    gesture="kb:control+alt+shift+K"
  )

  def script_OpenLibrary(self, gesture):
    wx.CallAfter(self.onKdd, None, True)

  def onOkoSettings(self, evt):
    gui.mainFrame.prePopup()
    d = OkoSettingsDialog(gui.mainFrame)
    d.Show()
    gui.mainFrame.postPopup()

  def showOkoSettings(self):
    wx.CallAfter(self.onOkoSettings, None)

  @scriptHandler.script(
    description=_(
      "Otevře okno s nastavením pro OKO."
    ),
    gesture="kb:control+shift+F12"
  )

  def script_OkoSettings(self, gesture):
    self.showOkoSettings()

  
  @scriptHandler.script(
    description=_(
      "Přečte obsah okna. Po dvojitém stisku bude obsah okna zobrazen v textovém poli."
    ),
    gesture="kb:control+shift+F9"
  )

  def script_ReadTextInWindow(self, gesture):
    self.playSound("buttons.wav")
    # Find the first focus ancestor that have any display text, according to the display model
    # This must be the root application window, or something close to that.
    # In case of universal apps, traverse child elements.
    text = None
    obj = api.getFocusObject()
    if obj.windowClassName.startswith("Windows.UI.Core"):
      foreground = api.getForegroundObject()
      desktop = api.getDesktopObject()
      uwpTextList = [foreground.name]
      curObject=foreground.firstChild
      while curObject:
        if curObject.name is not None: uwpTextList.append(curObject.name)
        if curObject.simpleFirstChild:
          curObject=curObject.simpleFirstChild
          continue
        if curObject.simpleNext:
          curObject=curObject.simpleNext
          continue
        if curObject.simpleParent:
          parent=curObject.simpleParent
          # As long as one is on current foreground object...
          # Stay within the current top-level window.
          if parent.simpleParent == desktop:
            break
          while parent and not parent.simpleNext:
            parent=parent.simpleParent
          # But sometimes, the top-level window has no sibling at all (such is the case in Windows 10 Start menu).
          try:
            curObject=parent.simpleNext
          except AttributeError:
            continue
      text = "\n".join(uwpTextList)
    else:
      root = None
      for ancestor in api.getFocusAncestors():
        if ancestor.appModule and ancestor.displayText:
          root = ancestor
          break
      if root:
        info = root.makeTextInfo(textInfos.POSITION_FIRST)
        info.move(textInfos.UNIT_LINE, sys.maxsize, endPoint="end")
        text = info.clipboardText.replace("\0", " ")
      if obj.windowClassName == u'ConsoleWindowClass':
        info = obj.makeTextInfo(textInfos.POSITION_FIRST)
        info.expand(textInfos.UNIT_STORY)
        text = info.clipboardText.rstrip()
    if text:
      name = api.getForegroundObject().name
      if name in (None, ""):
        name = _("Beze jména")
      if scriptHandler.getLastScriptRepeatCount()==1:
        # two presses: display text in window
        wx.CallAfter(self.onInfoText, None, "Texty okna", text)
      else:
        # one press: say text in window
        ui.message(text)
    else:
      # Message shown when no text can be virtualized.
      ui.message(_("Bez textu"))


class MyDialog(wx.Dialog):

  def __init__(self, parent, message = "", title = "", zipCodesList = None, newVersion = "", setZipCodeItem = None, setTempZipCodeItem = None, UpgradeAddonItem = None, buttons = None):
    super(MyDialog, self).__init__(parent, title = title)
    
    sn1 = str(zipCodesList) 
    sn2 = str(newVersion)
    sn3 = str(setZipCodeItem)
    sn4 = str(buttons)
    """
    final query before update
    message, title, newVersion, True
    """
    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = gui.guiHelper.BoxSizerHelper(self, orientation=wx.VERTICAL)

    self.zipCodesList = zipCodesList
    self.newVersion = newVersion
    self.setZipCodeItem = setZipCodeItem
    self.setTempZipCodeItem = setTempZipCodeItem
    self.UpgradeAddonItem = UpgradeAddonItem
    sHelper.addItem(wx.StaticText(self, label=message))
    bHelper = sHelper.addDialogDismissButtons(gui.guiHelper.ButtonHelper(wx.HORIZONTAL))

    if buttons:
      confirmButton = bHelper.addButton(self, id=wx.ID_YES)
      cancelButton = bHelper.addButton(self, id=wx.ID_NO)
      cancelButton.Bind(wx.EVT_BUTTON, self.OnCancel)
    
    elif (len(sn1)>0): 
      confirmButton = bHelper.addButton(self, id=wx.ID_YES)
      cancelButton = bHelper.addButton(self, id=wx.ID_NO)
      cancelButton.Bind(wx.EVT_BUTTON, self.OnCancel)
      
    else:
      confirmButton = bHelper.addButton(self, id=wx.ID_OK)
  
    self.buttons = buttons
    confirmButton.SetDefault()
    confirmButton.Bind(wx.EVT_BUTTON, self.OnConfirm)
    mainSizer.Add(sHelper.sizer, border=gui.guiHelper.BORDER_FOR_DIALOGS, flag=wx.ALL)
    self.Sizer = mainSizer
    mainSizer.Fit(self)
    self.SetSizer(mainSizer)
    self.Center(wx.BOTH|wx.Center)


  def OnConfirm(self, evt):
    if (len(self.zipCodesList)>0):
      self.Hide()
      url = 'https://suscik.cz/nvda/oko/oko.nvda-addon'
      #save to downloads
      from os.path import expanduser
      home = expanduser("~")
      target = os.path.join(home, "downloads", "oko.nvda-addon")

      message = _(u"Počkejte prosím...")
      title = _(u"Probíhá ukládání")
      result = self.Download_file(url, target, title, message)
      message = _(u"Stahování zrušeno.")
      title = _addonSummary
      if result is True: message = _(u"Stahování dokončeno. Po stisku tlačítka OK bude spuštěna instalace.")
      elif os.path.isfile(target):
        os.remove(target); view = False

      wx.MessageBox(message, title)

      #Start the update of the addon
      try:
        os.startfile(target)
      except WindowsError: pass


    if self.UpgradeAddonItem: self.EnableMenu(True)
    self.Destroy()


  def EnableMenu(self, flag):
    self.UpgradeAddonItem.Enable(flag)

  def OnCancel(self, evt = None):
    if self.UpgradeAddonItem: self.EnableMenu(True)
    self.Destroy()

  def Download_file(self, url, target, title, message):
    max = 100
    dlg = wx.ProgressDialog(title,
    message,
    maximum = max,
    style = 0
    | wx.PD_CAN_ABORT
    | wx.PD_APP_MODAL
    | wx.PD_ELAPSED_TIME
    | wx.PD_ESTIMATED_TIME
    )
    dlg.Update(0, message)
    try:

      fURL = urlopen(url, timeout=5)
      header = fURL.info()
      size = None
      outFile = open(target, 'wb')
      keepGoing = True
      if "Content-Length" in header:
        size = int(header["Content-Length"])
        kBytes = size/1024
        downloadBytes = int(size/max)
        count = 0
        while keepGoing:
          count += 1
          if count >= max: count = 99
          wx.MilliSleep(200)
          wx.Yield()
          (keepGoing, skip) = dlg.Update(count,
					'%s %s %s %s %s' %(
					_(u"Staženo"), str(count*downloadBytes/1024),
					_("z"), str(kBytes), "KB"))
          b = fURL.read(downloadBytes)
          if b:
            outFile.write(b)
          else:
            break
      else:
        while keepGoing:
          (keepGoing, skip) = dlg.UpdatePulse()
          b = fURL.read(1024*8)
          if b:
            outFile.write(b)
          else:
            break
      outFile.close()
      fURL.close()
      dlg.Update(99, '%s %s %s' % (
			_(u"Staženo"), str(os.path.getsize(target)/1024), "KB"))
      dlg.Destroy()
      return keepGoing
    except Exception as e:
      try:
        outFile.close()
        fURL.close()
      except: pass
      if not "failed" in str(e) and not "Not Found" in str(e) and not "unknown url type" in str(e):
        wx.MessageBox(message, title)
      dlg.Destroy()
      return "Error"
