# Copyright 2018-2025 Marek Susčík <markuv.koutek@seznam.cz>
# This add-on is free software, licensed under the terms of the GNU General Public License (version 2). see <https://www.gnu.org/licenses/gpl-2.0.html>.

import ui
from urllib.request import urlopen
import urllib.request
from logHandler import log
import wx
from gui import guiHelper
import gui
import json
import os
import datetime
import time
import wx.adv
from .xml.etree import ElementTree as ET
import locale
import re
import ssl

locale.setlocale(locale.LC_ALL, "czech")
CLEANR = re.compile('<.*?>')

class StationsDialog(wx.Dialog):

  tree = ET.Element('')
  stations = dict()
  allowEnter = False
  appMode = "TV"
  passKey = False
  _instance = None

  def __new__(cls, *args, **kwargs):
    # Make this a singleton.
    if StationsDialog._instance is None:
      return super(StationsDialog, cls).__new__(cls, *args, **kwargs)
    return StationsDialog._instance

  def __init__(self, parent):
    if StationsDialog._instance is not None:
      return
    StationsDialog._instance = self

    # The title of a dialog
    super(StationsDialog, self).__init__(parent, title="Televizní stanice")

    self.Bind(wx.EVT_CLOSE, self.onClose)
    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self,orientation=wx.VERTICAL)

    # listbox with TV stations
    stationsListSizer = wx.StaticBoxSizer(wx.StaticBox(self), wx.HORIZONTAL)
    stationsListContents = wx.BoxSizer(wx.HORIZONTAL)
    changeStationsSizer = wx.BoxSizer(wx.VERTICAL)

    self.loadStations(self.appMode)
    self.stationsList.Bind(wx.EVT_LISTBOX_DCLICK, self.onstationsListChoice)
    self.stationsList.Bind(wx.EVT_KEY_UP, self.onstationsListKeyUp)
    changeStationsSizer.Add(self.stationsList, proportion=1)
    changeStationsSizer.AddSpacer(guiHelper.SPACE_BETWEEN_ASSOCIATED_CONTROL_HORIZONTAL)

    # datepicker
    self.dpc = wx.adv.DatePickerCtrl(self, wx.ID_ANY, wx.DefaultDateTime)
    self.dpc.Bind(wx.EVT_KEY_UP, self.ondpcKeyUp)
    changeStationsSizer.Add(self.dpc)
    changeStationsSizer.AddSpacer(guiHelper.SPACE_BETWEEN_ASSOCIATED_CONTROL_HORIZONTAL)

    # the combo for switching between TV and radio stations
    wcChoices = ['Televize', 'Rozhlas']
    self.modeSwitchCombo = wx.Choice(self, choices=wcChoices)
    self.modeSwitchCombo.Bind(wx.EVT_CHOICE, self.onmodeSwitchComboChoice)
    self.modeSwitchCombo.Bind(wx.EVT_KEY_UP, self.onmodeSwitchComboKeyUp)
    self.modeSwitchCombo.SetSelection(0)
    changeStationsSizer.Add(self.modeSwitchCombo)

    stationsListContents.Add(changeStationsSizer, flag = wx.EXPAND)

    stationsListSizer.Add(stationsListContents, border=guiHelper.BORDER_FOR_DIALOGS, flag=wx.ALL)
    sHelper.addItem(stationsListSizer)

    mainSizer.Add(sHelper.sizer, flag=wx.ALL, border=guiHelper.BORDER_FOR_DIALOGS)
    mainSizer.Fit(self)
    self.Sizer = mainSizer
    self.stationsList.SetFocus()
    self.CentreOnScreen()
    self.allowEnter = False
    # the timer which don't allow react to Enter key shortly after running this module
    self.timer = wx.Timer(self)
    self.timer.StartOnce(100)
    self.Bind(wx.EVT_TIMER, self.enoughRuntime)

  def loadStations(self, mode):
    # load stations from file
    self.tree.clear()
    if mode == "TV":
      xmlUrl = "https://suscik.cz/nvda/oko/tvstations.xml"
      self.SetTitle("Televizní stanice")
    else:
      xmlUrl = "https://suscik.cz/nvda/oko/radiostations.xml"
      self.SetTitle("Rozhlasové stanice")
    offlineStations = False
    try:
      response = urlopen(xmlUrl).read()
    except:
      offlineStations = True
    if not offlineStations:
      self.tree = ET.fromstring(response)
    else:
      if mode == "TV":
        xmlFile = os.path.join(os.path.dirname(__file__), "files", "tv", "stations.xml")
      else:
        xmlFile = os.path.join(os.path.dirname(__file__), "files", "tv", "stationsradio.xml")

      treexml = ET.parse(xmlFile)
      self.tree = treexml.getroot()
      if mode == "TV":
        self.SetTitle("Televizní stanice (místní databáze)")
      else:
        self.SetTitle("Rozhlasové stanice (místní databáze)")

    # fill listbox with stations
    self.choices = []
    self.choices = [x.attrib["name"] for x in self.tree]
    try:
      self.stationsList.Set(self.choices)
    except:
      self.stationsList = wx.ListBox(self, choices=self.choices)
    self.stationsList.Selection = 0

  def enoughRuntime(self, evt):
    self.allowEnter = True

  def __del__(self):
    StationsDialog._instance = None

  def onmodeSwitchComboChoice(self, evt):
    sel = self.modeSwitchCombo.GetSelection()
    if sel == 0:
      self.appMode = "TV"
    else:
      self.appMode = "radio"
    self.stationsList.Clear()
    self.loadStations(self.appMode)

  def onmodeSwitchComboKeyUp(self, evt):
    key = evt.GetKeyCode()
    if key == wx.WXK_ESCAPE:
      self.onClose(self)
    evt.Skip()

  def ondpcKeyUp(self, evt):
    key = evt.GetKeyCode()
    if key == wx.WXK_RETURN or key == wx.WXK_NUMPAD_ENTER:
      self.onstationsListChoice(self)
    elif key == wx.WXK_ESCAPE:
      self.onClose(self)
    evt.Skip()


  def onstationsListChoice(self, evt):
    self.Disable()
    try:
      TvDialog(self).Show()
    except Exception as e:
      self.Enable()
      raise e

  def onstationsListKeyUp(self, evt):
    key = evt.GetKeyCode()
    if self.passKey:
      self.passKey = False
      return
    if key == wx.WXK_RETURN or key == wx.WXK_NUMPAD_ENTER:
      if not self.allowEnter:
        self.allowEnter = True
        return
      else:
        self.onstationsListChoice(self)
    elif key == wx.WXK_ESCAPE:
      self.onClose(self)
    evt.Skip()

  def onClose(self, evt):
    self.Destroy()
    StationsDialog._instance = None


class TvDialog(wx.Frame):

  allTitles = []
  curDate = wx.DateTime
  stNumber = 0
  stName = ""
  stSource = ""
  stAddress = ""
  stChannel = ""
  titleTvDialog = "Pořady"
  weekDays = ["pondělí", "úterý", "středa", "čtvrtek", "pátek", "sobota", "neděle"]
  treeCt = ET.Element('')
  # fix ssl.SSLError: [SSL: DH_KEY_TOO_SMALL] dh key too small
  ctx = ssl.SSLContext(ssl.PROTOCOL_TLSv1_2)
  ctx.set_ciphers('HIGH:!DH:!aNULL')

  def __init__(self, parent):

    # The title of a dialog
    super(TvDialog, self).__init__(parent, title=self.titleTvDialog)

    self.Bind(wx.EVT_CLOSE, self.onClose)
    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self,orientation=wx.VERTICAL)

    # the menu bar
    menuBar = wx.MenuBar()
    menu = wx.Menu()
    
    # menu items
    item = menu.Append(wx.ID_ANY, "Zobrazit &informace o pořadu\tEnter")
    self.Bind(wx.EVT_MENU, self.ontvListChoice, item)
    item = menu.Append(wx.ID_ANY, "&Ohlásit informace o pořadu\tF2")
    self.Bind(wx.EVT_MENU, self.sayDetailsItem, item)
    item = menu.Append(wx.ID_ANY, "&Stránka pořadu\tCtrl+Enter")
    self.Bind(wx.EVT_MENU, self.titlePageItem, item)
    item = menu.Append(wx.ID_ANY, "Pořad v &iVysílání\tCtrl+Shift+Enter")
    self.Bind(wx.EVT_MENU, self.ivysilaniPageItem, item)
    item = menu.Append(wx.ID_ANY, "Přejít na &následující den\tCtrl+Space")
    self.Bind(wx.EVT_MENU, self.nextDayItem, item)
    item = menu.Append(wx.ID_ANY, "Přejít na &předchozí den\tCtrl+Backspace")
    self.Bind(wx.EVT_MENU, self.previousDayItem, item)
    item = menu.Append(wx.ID_ANY, "&Zpět na seznam stanic\tBackspace")
    self.Bind(wx.EVT_MENU, self.backtoStationsItem, item)
    item = menu.Append(wx.ID_ANY, "Nápo&věda")
    self.Bind(wx.EVT_MENU, self.helpItem, item)
    menu.AppendSeparator()
    item = menu.Append(wx.ID_EXIT, "U&končit")
    self.Bind(wx.EVT_MENU, self.onClose, item)
    
    # the menu title
    menuBar.Append(menu, "Pořady")
    self.SetMenuBar(menuBar)

    # listbox with TV program titles
    tvListSizer = wx.StaticBoxSizer(wx.StaticBox(self), wx.HORIZONTAL)
    tvListContents = wx.BoxSizer(wx.HORIZONTAL)
    changeTvSizer = wx.BoxSizer(wx.VERTICAL)

    self.stName = parent.stationsList.StringSelection
    self.curDate = parent.dpc.GetValue()
    treeItem = parent.tree.findall(f".//station[@name='{parent.stationsList.StringSelection}']")
    treeStation = treeItem[0]
    self.stSourceList = []
    self.stSourceList = [x for x in treeStation.iter("source")]
    displayed = False
    if len(self.stSourceList) > 0:
      for item in self.stSourceList:
        treeSource = item
        self.stSource = treeSource.attrib["id"]
        if self.stSource == "sms":
          self.stAddress = treeSource.find("address").text
        elif self.stSource == "ct":
          self.stChannel = treeSource.find("channel").text
        if self.loadTvTitles():
          displayed = True
          break
    if not displayed:
      errorMessage = _(u"Pořady nebylo možné zobrazit.")
      errorTitle = _(u"Chyba")
      wx.MessageBox(errorMessage, errorTitle)
      wx.MilliSleep(200)
      self.Parent.Enable()
      self.Destroy()
      return

    self.tvList.Bind(wx.EVT_LISTBOX_DCLICK, self.ontvListChoice)
    self.tvList.Bind(wx.EVT_KEY_UP, self.ontvListKeyUp)
    changeTvSizer.Add(self.tvList, proportion=1)
    tvListContents.Add(changeTvSizer, flag = wx.EXPAND)
    tvListSizer.Add(tvListContents, border=guiHelper.BORDER_FOR_DIALOGS, flag=wx.ALL)
    sHelper.addItem(tvListSizer)
    mainSizer.Add(sHelper.sizer, flag=wx.ALL, border=guiHelper.BORDER_FOR_DIALOGS)
    mainSizer.Fit(self)
    self.Sizer = mainSizer
    self.tvList.SetFocus()
    self.CentreOnScreen()

  def helpItem(self, evt):
    from . import addonDoc
    os.startfile(addonDoc.getDocPath("tv.html"))

  def backtoStationsItem(self, evt):
    self.Parent.Enable()
    self.Parent.passKey = True
    self.Parent.stationsList.SetFocus()
    self.Destroy()

  def nextDayItem(self, evt):
    self.curDate = self.curDate + wx.DateSpan(days=1)
    ui.message(self.weekDays[datetime.date(self.curDate.year,self.curDate.month+1,self.curDate.day).weekday()])
    self.tvList.Clear()
    self.loadTvTitles()
    self.tvList.SetFocus()

  def previousDayItem(self, evt):
    self.curDate = self.curDate - wx.DateSpan(days=1)
    ui.message(self.weekDays[datetime.date(self.curDate.year,self.curDate.month+1,self.curDate.day).weekday()])
    self.tvList.Clear()
    self.loadTvTitles()
    self.tvList.SetFocus()

  def getTitleIndex(self):
    # find index of currently broadcasted title
    curIndex = 0
    curTime = datetime.datetime.now()
    if curTime.year == self.curDate.year and curTime.month == self.curDate.month+1 and curTime.day == self.curDate.day:
      titleTimeLast = None
      indexFound = False
      for x in range(len(self.allTitles)):
        timeArr = self.allTitles[x].time.split(':')
        titleTime = curTime.replace(hour=int(timeArr[0]), minute=int(timeArr[1]), second=0, microsecond=0)
        if titleTimeLast is not None and titleTime < curTime and titleTimeLast > titleTime:
          titleTime += datetime.timedelta(days=1)
        if titleTime > curTime:
          curIndex = x -1
          indexFound = True
          if curIndex < 0:
            curIndex = 0
          break;
        titleTimeLast = titleTime
      if not indexFound:
        curIndex = len(self.allTitles)-1
    return curIndex

  def cuttoMidnight(self):
    curTime = datetime.datetime.now()
    titleTimeLast = None
    onemoretitle = 0
    for x in range(len(self.allTitles)):
      timeArr = self.allTitles[x].time.split(':')
      titleTime = curTime.replace(hour=int(timeArr[0]), minute=int(timeArr[1]), second=0, microsecond=0)
      if titleTimeLast is not None and titleTimeLast > titleTime:
        if titleTime > curTime:
          onemoretitle = 1
        curIndex = x -1 -onemoretitle
        if curIndex < 0:
          curIndex = 0
        break;
      titleTimeLast = titleTime
    del self.allTitles[0:x-onemoretitle]

  def loadFromCentrum(self):
    # nested function
    def parseCentrum(prDate, clearTitles = True):
      tmpTitles = []
      channelsUrl = "https://tvprogram.centrum.cz/api/channels"
      try:
        request = urllib.request.Request(channelsUrl)
        request.add_header('User-Agent', 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/57.0.2987.133 Safari/537.36')
        response = urlopen(request).read()
        channelDict = json.loads(response)
      except: return False
      channelId = ""
      for key in channelDict.keys():
        if channelDict[key]["name"] == self.stName:
          channelId = channelDict[key]["id"]
          break
      if not channelId:
        return False
      programUrl = "https://tvprogram.centrum.cz/api/broadcasting/" + str(prDate.year) + "-" + str(prDate.month) + "-" + str(prDate.day) + "?channels[]=" + channelId
      try:
        request = urllib.request.Request(programUrl)
        request.add_header('User-Agent', 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/57.0.2987.133 Safari/537.36')
        response = urlopen(request).read()
        programDict = json.loads(response)
      except: return False
      if clearTitles:
        self.allTitles.clear()
      else:
        tmpTitles.clear()
      for item in programDict[channelId]:
        porTitle = item["title"]
        porTime = item["start"]
        porTime = porTime.split("T")[1]
        porTime = porTime.rsplit(":", 1)[0]
        porHour = porTime.split(":")[0]
        porMin = porTime.split(":")[1]
        porHourShift = int(porHour) + 1
        if time.localtime().tm_isdst > 0:
          porHourShift = porHourShift + 1
        porHourShift = porHourShift % 24
        porTime = str(porHourShift).zfill(2) + ":" + porMin
        if clearTitles:
          self.allTitles.append(TvTitle(porTitle, porTime, "", item))
        else:
          tmpTitles.append(TvTitle(porTitle, porTime, "", item))
      self.allTitles = tmpTitles + self.allTitles
      if self.allTitles:
        return True
      else:
        return False

    # main function
    ttDate = datetime.datetime(self.curDate.year, self.curDate.month+1, self.curDate.day)
    if not parseCentrum(ttDate):
      return -1
    tcList = self.allTitles[0].time.split(":")
    ccTime = datetime.datetime.now().time()
    tcTime = datetime.time(int(tcList[0]), int(tcList[1]), 0)
    # when first title begins later than current time, the previous day is also loaded
    if tcTime > ccTime:
      if not parseCentrum(ttDate - datetime.timedelta(days=1), False):
        return -1
      self.cuttoMidnight()
    return self.getTitleIndex()

  def loadFromSms(self):
    # nested function
    def parseSms(prDate, clearTitles = True):
      tvUrl = "http://m.tv.sms.cz/?cas=0&den=" + str(prDate.year) + "-" + str(prDate.month).zfill(2) + "-" + str(prDate.day).zfill(2) + "&stanice=" + str(self.stAddress)
      tmpTitles = []
      try:
        response = urlopen(tvUrl, context=self.ctx).read()
      except:
        response = ""
      try:
        output = response.decode('windows-1250')
      except:
        output = ""
      if output == "" or output.find("<div class='porady'><div class='porad'") == -1:
        return False
      htmlArr = GetSubstrings(output, "<div class='porady'>", "<div class='predchozi_porady'")
      htmlStr = htmlArr[0]
      startTimes = GetSubstrings(htmlStr, "<div class='cas'", "</span>")
      startTimes = [':'.join(x.rsplit('.', 1)) for x in startTimes]
      startTimes = [x.rsplit('>', 1)[1] for x in startTimes]
      htmlArr = GetSubstrings(htmlStr, "class='nazev'", "<div class='ztr'>")
      titles = [x.rsplit('>', 1)[1] for x in htmlArr]
      detailsAddr = []
      detailsAddr.clear()
      for item in htmlArr:
        tempArr = GetSubstrings(item, "href='https:", "'>")
        detailsAddr.append(tempArr[0])
      if clearTitles:
        self.allTitles.clear()
      else:
        tmpTitles.clear()
      for x in range(len(titles)):
        curTitle = titles[x].strip()
        if clearTitles:
          self.allTitles.append(TvTitle(curTitle, startTimes[x], 'https:' + detailsAddr[x]))
        else:
          tmpTitles.append(TvTitle(curTitle, startTimes[x], 'https:' + detailsAddr[x]))
      self.allTitles = tmpTitles + self.allTitles
      if self.allTitles:
        return True
      else:
        return False

    # main function
    ttDate = datetime.datetime(self.curDate.year, self.curDate.month+1, self.curDate.day)
    if not parseSms(ttDate):
      return -1
    tcList = self.allTitles[0].time.split(":")
    ccTime = datetime.datetime.now().time()
    tcTime = datetime.time(int(tcList[0]), int(tcList[1]), 0)
    # when first title begins later than current time, the previous day is also loaded
    if tcTime > ccTime:
      if not parseSms(ttDate - datetime.timedelta(days=1), False):
        return -1
      self.cuttoMidnight()
    return self.getTitleIndex()

  def loadFromCt(self):
    # nested function
    def parseCt(prDate, clearTitles = True):
      tvUrl = "https://www.ceskatelevize.cz/services-old/programme/xml/schedule.php?user=oko&date=" + str(prDate.day).zfill(2) + "." + str(prDate.month).zfill(2) + "." + str(prDate.year) + "&channel=" + str(self.stChannel)
      tmpTitles = []
      try:
        response = urlopen(tvUrl).read()
      except: return False
      self.treeCt = ET.fromstring(response)
      if clearTitles:
        self.allTitles.clear()
      else:
        tmpTitles.clear()
      for por in self.treeCt.iter("porad"):
        tit = por.find("nazvy")
        try:
          porTitle = tit.find("nazev").text
        except: pass
        if porTitle is not None:
          porDil = por.find("dil").text
          if porDil is not None:
            porTitle = porTitle + " (" + porDil + ")"
          porNazevCasti = tit.find("nazev_casti").text
          if porNazevCasti is not None:
            porTitle = porTitle + " - " + porNazevCasti
          porTime = por.find("cas").text
          if porTime is not None:
            if clearTitles:
              self.allTitles.append(TvTitle(porTitle, porTime, "", por))
            else:
              tmpTitles.append(TvTitle(porTitle, porTime, "", por))
      self.allTitles = tmpTitles + self.allTitles
      if self.allTitles:
        return True
      else:
        return False

    # main function
    ttDate = datetime.datetime(self.curDate.year, self.curDate.month+1, self.curDate.day)
    if not parseCt(ttDate):
      return -1
    tcList = self.allTitles[0].time.split(":")
    ccTime = datetime.datetime.now().time()
    tcTime = datetime.time(int(tcList[0]), int(tcList[1]), 0)
    # when first title begins later than current time, the previous day is also loaded
    if tcTime > ccTime:
      if not parseCt(ttDate - datetime.timedelta(days=1), False):
        return -1
      self.cuttoMidnight()
    return self.getTitleIndex()

  def loadTvTitles(self):
    self.SetTitle(self.titleTvDialog + " - " + str(self.curDate.day) + ". " + str(self.curDate.month+1) + ". " + self.stName)
    if self.stSource == "sms":
      curListIndex = self.loadFromSms()
    elif self.stSource == "centrum":
      curListIndex = self.loadFromCentrum()
    elif self.stSource == "ct":
      curListIndex = self.loadFromCt()
    if curListIndex < 0:
      return False
    self.choices = []
    addItem = ""
    for x in self.allTitles:
      addItem = x.time + ' - ' + x.title
      if self.stSource == "ct" and x.tData is not None:
        ikonky = x.tData.find("ikonky")
        porAd = ikonky.find("ad").text
        if porAd is None:
          porAd = ""
        if porAd == "1":
          addItem = "*" + addItem
      self.choices.append(addItem)
    try:
      self.tvList.Set(self.choices)
    except:
      self.tvList = wx.ListBox(self, choices=self.choices)
    self.tvList.Selection = curListIndex
    return True

  def __del__(self):
    TvDialog._instance = None

  def ontvListChoice(self, evt):
    if self.tvList.GetCount() == 0:
      return
    self.selectedTitle = self.allTitles[self.tvList.Selection]
    self.Disable()
    try:
      DetailsDialog(self).Show()
    except Exception as e:
      self.Enable()
      raise e

  def ontvListKeyUp(self, evt):
    key = evt.GetKeyCode()
    if key == key == wx.WXK_NUMPAD_ENTER:
      self.ontvListChoice(self)
    elif key == wx.WXK_ESCAPE:
      self.onClose(self)
    evt.Skip()

  def onClose(self, evt):
    self.Parent.Destroy()
    self.Destroy()
    StationsDialog._instance = None

  def sayDetailsItem(self, evt):
    sayTitle = self.allTitles[self.tvList.Selection]
    src = self.stSource
    txtDetail = DetailsDialog.getDetails(self, sayTitle, src)
    ui.message(txtDetail)

  def titlePageItem(self, evt):
    curTitle = self.allTitles[self.tvList.Selection]
    src = self.stSource
    if src != "ct":
      return
    try:
      porlinky = curTitle.tData.find("linky")
      openlink = porlinky.find("program").text
    except:
      openlink = ""
    if openlink:
      os.startfile(openlink)
    else:
      dlgMessage = _(u"U pořadu není uvedena jeho webová stránka.")
      dlgTitle = _(u"Upozornění")
      wx.MessageBox(dlgMessage, dlgTitle)

  def ivysilaniPageItem(self, evt):
    curTitle = self.allTitles[self.tvList.Selection]
    src = self.stSource
    if src != "ct":
      return
    try:
      porlinky = curTitle.tData.find("linky")
      openlink = porlinky.find("ivysilani").text
    except:
      openlink = ""
    if openlink:
      os.startfile(openlink)
    else:
      dlgMessage = _(u"U pořadu není uvedena stránka pro iVysílání.")
      dlgTitle = _(u"Upozornění")
      wx.MessageBox(dlgMessage, dlgTitle)


class DetailsDialog(wx.Dialog):

  # fix ssl.SSLError: [SSL: DH_KEY_TOO_SMALL] dh key too small
  ctx = ssl.SSLContext(ssl.PROTOCOL_TLSv1_2)
  ctx.set_ciphers('HIGH:!DH:!aNULL')

  def __init__(self, parent):
    # The title of the details dialog
    super(DetailsDialog, self).__init__(parent, size=(1000, 800), title="Popis pořadu " + parent.selectedTitle.title)
    self.Bind(wx.EVT_CLOSE, self.onClose)
    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self, orientation=wx.VERTICAL)

    # The details edit field
    self.outputCtrl = wx.TextCtrl(self, wx.ID_ANY, style=wx.TE_MULTILINE | wx.TE_READONLY)
    self.outputCtrl.Bind(wx.EVT_KEY_UP, self.onOutputKeyUp)
    sHelper.addItem(self.outputCtrl, proportion=1, flag=wx.EXPAND)
    self.outputCtrl.SetValue(self.getDetails(parent.selectedTitle, parent.stSource))
    self.outputCtrl.SetFocus()

  def getDetails(self, title, source):
    details = []
    detailfinal = "Popis pořadu není k dispozici."
    if source == "centrum":
      cenTit = title. tData
      if cenTit["description"]:
        detailfinal = cenTit["description"]
      if cenTit["durationMin"]:
        if cenTit["description"]:
          detailfinal = detailfinal + "\nDélka: " + str(cenTit["durationMin"]) + " min"
        else:
          detailfinal = "Délka: " + str(cenTit["durationMin"]) + " min"

    elif source == "sms":
      address = title.detailAddress
      try:
        response = urlopen(address, context=self.ctx).read()
      except Exception as e:
        response = ""
        raise e
      output = response.decode('windows-1250')
      details = GetSubstrings(output, 'div class="nadpis_scroll">', "<div class=")
      if len(details) >0 and details[0].lower().find("popis") != -1:
        tempStr = details[0].strip()
        details = tempStr.split("</div>")
        detailfinal = details[1]
      detailfinal = cleanHtml(detailfinal)
      detailfinal = detailfinal.replace("...", "")
    elif source == "ct":
      ctTit = title.tData
      try:
        detailfinal = ctTit.find("noticka").text
      except:
        detailfinal = ""
      if detailfinal:
        detailfinal = detailfinal + "\n\n"
      else:
        detailfinal = "\n\n"
      try:
        detailfinal = detailfinal + "Délka: " + ctTit.find("stopaz").text.lstrip("0") + "\n"
      except: pass
      try:
        ctZanr = ctTit.find("zanr").text
      except:
        ctZanr = ""
      if ctZanr:
        detailfinal = detailfinal + "Žánr: " + ctZanr + "\n"
      ctDostupnost = ctTit.find("dostupnost")
      if ctDostupnost:
        detailfinal = detailfinal + "Dostupnost v iVysílání: " + ctDostupnost.find("stav").text
        titOd = ctDostupnost.find("od").text
        if titOd:
          strDatum = titOd.split(" ")[0]
          strCas = titOd.split(" ")[1]
          strList = strDatum.split("-")
          strList2 = strCas.split(":")
          detailfinal = detailfinal + " od " + strList[2].lstrip("0") + "." + strList[1].lstrip("0") + "." + strList[0] + " " + strList2[0] + ":" + strList2[1]
        titDo = ctDostupnost.find("do").text
        if titDo:
          strDatum = titDo.split(" ")[0]
          strCas = titDo.split(" ")[1]
          strList = strDatum.split("-")
          strList2 = strCas.split(":")
          detailfinal = detailfinal + " do " + strList[2].lstrip("0") + "." + strList[1].lstrip("0") + "." + strList[0] + " " + strList2[0] + ":" + strList2[1]
        detailfinal = detailfinal + "\n"
      
      ctIkonky = ctTit.find("ikonky")
      try:
        ctPremiera = ctIkonky.find("premiera").text
      except:
        ctPremiera = ""
      if ctPremiera:
        detailfinal = detailfinal + "Premiéra: " + numtoAns(ctPremiera) + "\n"
      try:
        titZvuk = ctIkonky.find("zvuk").text
      except:
        titZvuk = ""
      if titZvuk:
        zvukStr = ""
        if titZvuk == "M":
          zvukStr = "mono"
        elif titZvuk == "S":
          zvukStr = "stereo"
        elif titZvuk== "D":
          zvukStr = "Duální"
        elif titZvuk == "E":
          zvukStr = "Dolby"
        elif titZvuk == "B":
          zvukStr = "Dolby Surround"
        detailfinal = detailfinal + "Zvuk: " + zvukStr + "\n"
      try:
        ctZvukPopis = ctIkonky.find("ad").text
      except:
        ctZvukPopis = ""
      if ctZvukPopis:
        detailfinal = detailfinal + "Zvukový popis: " + numtoAns(ctZvukPopis) + "\n"
      try:
        ctSkryteTitulky = ctIkonky.find("skryte_titulky").text
      except:
        ctSkryteTitulky = ""
      if ctSkryteTitulky:
        detailfinal = detailfinal + "Skryté titulky: " + numtoAns(ctSkryteTitulky) + "\n"
      try:
        ctNeslysici = ctIkonky.find("neslysici").text
      except:
        ctNeslysici = ""
      if ctNeslysici:
        detailfinal = detailfinal + "Komentář pro neslyšící: " + numtoAns(ctNeslysici) + "\n"
      try:
        ctPomer = ctIkonky.find("pomer").text
      except:
        ctPomer = ""
      if ctPomer:
        detailfinal = detailfinal + "Poměr stran: " + ctPomer + "\n"
      try:
        ctVps = ctTit.find("vps").text
      except:
        ctVps = ""
      if ctVps:
        detailfinal = detailfinal + "VPS: " + ctVps + "\n"

    detailfinal = detailfinal.replace("&#39;", "'")
    return detailfinal

  def onOutputKeyUp(self, evt):
    key = evt.GetKeyCode()
    if key == wx.WXK_BACK:
      self.Parent.Enable()
      self.Destroy()
    elif key == wx.WXK_ESCAPE:
      self.onClose(self)
    evt.Skip()

  def onClose(self, evt):
    self.Parent.Destroy()
    self.Parent.Parent.Destroy()
    self.Destroy()
    TvDialog._instance = None
    StationsDialog._instance = None


class TvTitle:

  def __init__(self, title, time, detailAddress, tData = None):
    self.title = title
    self.time = time
    self.detailAddress = detailAddress
    self.tData = tData

def GetSubstrings(stringSubject,string1,string2):
  MyList = []
  intstart=0
  strlength=len(stringSubject)
  continueloop = 1

  while(intstart < strlength and continueloop == 1):
    intindex1=stringSubject.find(string1,intstart)
    if(intindex1 != -1): #The substring was found, lets proceed
      intindex1 = intindex1+len(string1)
      intindex2 = stringSubject.find(string2,intindex1)
      if(intindex2 != -1):
        subsequence=stringSubject[intindex1:intindex2]
        MyList.append(subsequence)
        intstart=intindex2+len(string2)
      else:
        continueloop=0
    else:
      continueloop=0
  return MyList

def numtoAns(s):
  return "ano" if s == "1" else "ne"

def cleanHtml(raw_html):
  cleantext = re.sub(CLEANR, '', raw_html)
  return cleantext