# Copyright 2018-2025 Marek Susčík <markuv.koutek@seznam.cz>
# This add-on is free software, licensed under the terms of the GNU General Public License (version 2). see <https://www.gnu.org/licenses/gpl-2.0.html>.

import wx
from gui import guiHelper
import os
import sys
import api
import treeInterceptorHandler
import textInfos.offsets
from logHandler import log
import urllib.request
import re
import gui

CLEANR = re.compile('<.*?>')

def cleanHtml(raw_html):
  cleantext = re.sub(CLEANR, '', raw_html)
  return cleantext

def getSubstr(s, startstr, endstr):
  substr = re.search(startstr + '(.*)' + endstr, s)
  return substr.group(1)

def GetSubstrings(stringSubject,string1,string2):
  MyList = []
  intstart=0
  strlength=len(stringSubject)
  continueloop = 1

  while(intstart < strlength and continueloop == 1):
    intindex1=stringSubject.find(string1,intstart)
    if(intindex1 != -1): #The substring was found, lets proceed
      intindex1 = intindex1+len(string1)
      intindex2 = stringSubject.find(string2,intindex1)
      if(intindex2 != -1):
        subsequence=stringSubject[intindex1:intindex2]
        MyList.append(subsequence)
        intstart=intindex2+len(string2)
      else:
        continueloop=0
    else:
      continueloop=0
  return MyList

class WebSourcesDialog(wx.Dialog):

  _instance = None
  wsList = []

  def __new__(cls, *args, **kwargs):
    # Make this a singleton.
    if WebSourcesDialog._instance is None:
      return super(WebSourcesDialog, cls).__new__(cls, *args, **kwargs)
    return WebSourcesDialog._instance

  def __init__(self, parent):
    
    if WebSourcesDialog._instance is not None:
      return
    
    WebSourcesDialog._instance = self
    # The title of a dialog.
    super(WebSourcesDialog, self).__init__(parent, title=_(u"Webové zdroje"))

    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self,orientation=wx.VERTICAL)

    # The label of a phrase text field
    phraseTextLabel = _(u"&Slovo nebo fráze:")
    self.phraseTextEdit = sHelper.addLabeledControl(phraseTextLabel, wx.TextCtrl)

    # The listbox with sources
    global WebSourcePath
    WebSourcePath = os.path.join(os.path.dirname(__file__), "files\\websources\\")
    ListFile = os.path.join(WebSourcePath, "list.txt")
    self.choices = []
    fs_enc = sys.getfilesystemencoding()
    if os.path.isfile(ListFile):
      f = open(ListFile, "r", encoding='utf-8')
      for line in f:
        line = line.rstrip("\n")
        (key, val) = line.split(";")
        self.wsList.append((key, val))
    for t in self.wsList:
      self.choices.append(t[1])
    self.sourceList = wx.ListBox(self, choices=self.choices)
    # load source list position
    lpos = ""
    FilePath = os.path.join(os.path.dirname(__file__), "files\\websources\WebText\lastitem.txt")
    if os.path.isfile(FilePath):
      with open(FilePath, 'r') as f:
        lpos = f.read()
    if lpos:
      self.sourceList.Selection = int(lpos)
    else:
      self.sourceList.Selection = 0
    self.sourceList.Bind(wx.EVT_LISTBOX, self.onSourceListChoice)
    sHelper.addItem(self.sourceList, flag = wx.EXPAND)

    # The label of a hint text field
    hintTextLabel = _(u"&Nápověda:")
    self.hintTextEdit = sHelper.addLabeledControl(hintTextLabel, wx.TextCtrl)
  
    #button row
    sHelper_Buttons = guiHelper.BoxSizerHelper(self, orientation=wx.VERTICAL)

    # The label of an OK button 
    self.okButton = wx.Button(self, label=_(u"&OK"))
    self.okButton.Bind(wx.EVT_BUTTON, self.onOk)
    self.AffirmativeId = self.okButton.Id
    self.okButton.SetDefault()
    sHelper_Buttons.addItem(self.okButton)

    # The label of a Cancel button 
    cancelButton = wx.Button(self, wx.ID_CLOSE, label=_(u"&Zrušit"))
    cancelButton.Bind(wx.EVT_BUTTON, lambda evt: self.Close())
    sHelper_Buttons.addDialogDismissButtons(cancelButton)
    self.Bind(wx.EVT_CLOSE, self.onCancel)
    self.EscapeId = wx.ID_CLOSE

    sHelper.addItem(sHelper_Buttons, flag = wx.CENTER)

    # get current selection
    obj=api.getFocusObject()
    treeInterceptor=obj.treeInterceptor
    if isinstance(treeInterceptor,treeInterceptorHandler.DocumentTreeInterceptor) and not treeInterceptor.passThrough:
      obj=treeInterceptor
    try:
      info=obj.makeTextInfo(textInfos.POSITION_SELECTION)
    except (RuntimeError, NotImplementedError):
      info=None
    if not info or info.isCollapsed:
      # get current word
      info=api.getReviewPosition().copy()
      info.expand(textInfos.UNIT_WORD)

    # fill current word into phrase edit field
    self.phraseTextEdit.SetValue(info.text)

    self.onSourceListChoice(None)
    mainSizer.Add(sHelper.sizer, flag=wx.ALL, border=guiHelper.BORDER_FOR_DIALOGS)
    mainSizer.Fit(self)
    self.Sizer = mainSizer
    self.phraseTextEdit.SetFocus()
    self.CentreOnScreen()

  def onSourceListChoice(self, evt):
    self.sel = self.sourceList.Selection
    self.stringSel = self.sourceList.StringSelection
    self.wsNameString = ""
    for t in self.wsList:
      if t[1] == self.stringSel:
        self.wsNameString = t[0]
    self.okButton.Enabled = self.sel>= 0
    FilePath = os.path.join(WebSourcePath, "%s.txt" % self.wsNameString)
    if os.path.isfile(FilePath):
      f = open(FilePath, "r", encoding='utf-8')
      self.hintTextEdit.SetValue(f.readline())
    else:
      self.hintTextEdit.SetValue("")

  def onOk(self, evt):
    # save the source list position
    FilePath = os.path.join(os.path.dirname(__file__), "files\\websources\WebText\lastitem.txt")
    with open(FilePath, 'w') as f:
      f.write(str(self.sel))
    if self.phraseTextEdit.Value:
      # save the search phrase
      FilePath = os.path.join(os.path.dirname(__file__), "files\\websources\WebText\searchphrase.txt")
      with open(FilePath, 'wb') as f:
        f.write(self.phraseTextEdit.Value.encode('windows-1250'))
      if self.wsNameString == "jednotky":
        self.jednotkyWebSource()
      else:
        FilePath = os.path.join(WebSourcePath, "%s.vbs" % self.wsNameString)
        os.startfile(FilePath)
    self.wsList.clear()
    self.Destroy()
    WebSourcesDialog._instance = None

  def onCancel(self, evt):
    self.wsList.clear()
    self.Destroy()
    WebSourcesDialog._instance = None

  def jednotkyWebSource(self):
    query_string = urllib.parse.quote_plus(self.phraseTextEdit.Value, safe='')
    url = "https://www.google.com/search?q=" + query_string
    # import requests
    # headers = {'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/110.0.0.0 Safari/537.36'}
    # cookies = {'CONSENT':"YES+cb.20220419-08-p0.cs+FX+111"}
    # result = requests.get(url, headers=headers, cookies=cookies).text
    headers = {
      'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36',
      'CONSENT': "YES+cb.20220419-08-p0.cs+FX+111"
    }
    request = urllib.request.Request(url, None, headers)
    response = urllib.request.urlopen(request)
    content = response.read()
    output = content.decode()
    stripOutputList = GetSubstrings(output, '<div class="vk_gy vk_sh">', '<div class="di8g3 ChOqnd"')
    if len(stripOutputList) == 0:
      stripOutputList = GetSubstrings(output, '<div><span class="DFlfde eNFL1">', '<div class="k0Rg6d hqAUc">')
      if len(stripOutputList) == 0:
        plainTxt = ""
        result = "Výraz " + self.phraseTextEdit.Value + " není možné pro převod jednotek použít."
    if len(stripOutputList) != 0:
      stripOutput = stripOutputList[0]
      plainTxt = cleanHtml(stripOutput)
      result = plainTxt
    wx.CallAfter(self.onInfoText, None, "Převod", result)

  def onInfoText(self, evt, title, mess):
    gui.mainFrame.prePopup()
    d = InfoTextFrame(gui.mainFrame, title, mess)
    d.Show()
    gui.mainFrame.postPopup()


class InfoTextFrame(wx.Frame):

  def __init__(self, parent, winTitle, winText):
    super().__init__(
      parent,
      title=winTitle
    )
    self.Maximize()
    self.Bind(wx.EVT_CLOSE, self.onClose)
    mainSizer = wx.BoxSizer(wx.VERTICAL)
    self.outputCtrl = wx.TextCtrl(self, wx.ID_ANY, style=wx.TE_MULTILINE | wx.TE_READONLY|wx.TE_RICH)
    self.outputCtrl.Bind(wx.EVT_KEY_DOWN, self.onOutputKeyDown)
    mainSizer.Add(self.outputCtrl, proportion=1, flag=wx.EXPAND)
    self.SetSizer(mainSizer)
    mainSizer.Fit(self)

    self.outputCtrl.SetValue(winText)

    self.outputCtrl.SetFocus()

  def onClose(self, evt):
    self.Destroy()

  def onOutputKeyDown(self, evt):
    key = evt.GetKeyCode()
    if key == wx.WXK_ESCAPE:
      self.Close()
      return
    evt.Skip()

