# Copyright 2018-2025 Marek Susčík <markuv.koutek@seznam.cz>
# This add-on is free software, licensed under the terms of the GNU General Public License (version 2). see <https://www.gnu.org/licenses/gpl-2.0.html>.

import ui
from logHandler import log
from .xml.etree import ElementTree as ET
import wx
import gui
from gui import guiHelper
from contextlib import closing
from urllib.request import urlopen
import locale
import os
import urllib.request
import urllib.parse
import http.cookiejar
import re
from ftplib import FTP
import time
import datetime
import json
from speech import sayAll
from .striprtf import striprtf
import base64
import speech
try:
  from . import vlc
except:
  vlcInstalled = False
else:
  vlcInstalled = True
import math
import threading
from . import id3reader
import tones


locale.setlocale(locale.LC_ALL, "czech")

kddUser = ""
kddCredits = 0
kddBonusCredits = 0
kddLastLogin = ""
kddLogged = False
kddConfig = dict()
mathildaConfig = dict()
spvkConfig = dict()
ktnConfig = dict()
zoraConfig = dict()
kddUserLogin = ""
kddUserPassword = ""
kddUpdate = ""
kddUpdateBefore = ""
newKddCatalogDownloaded = False
kddCatSize = 0
mathildaUserLogin = ""
mathildaUserPassword = ""
spvkUserLogin = ""
spvkUserPassword = ""
spvkApiKey = base64.b64decode("NTU2ZGQwODctY2Q3My00MDIzLThkNDAtYTMzNGRiMTkyOWU4").decode("utf-8")
ktnUserLogin = ""
ktnUserPassword = ""
ktnCatalogIndex = ""
newKtnCatalogDownloaded = False
zoraUserLogin = ""
zoraUserPassword = ""
mainTitle = "Katalog knihovny"
libraryDataFolder = os.path.join(os.getenv('APPDATA'), "Marsus", "NVDA", "OKO", "Library")
finalSelection = ""

# initiate the cookie jar (using http.cookiejar and urllib.request)
cookie_jar = http.cookiejar.CookieJar()
opener = urllib.request.build_opener(urllib.request.HTTPCookieProcessor(cookie_jar))
urllib.request.install_opener(opener)

class KddMainDialog(wx.Frame):

  tree = ET.Element('')
  treeMathildaBooks = ET.Element('')
  treeMathildaMovies = ET.Element('')
  treeSpvk = ET.Element('')
  treeKtn = ET.Element('')
  categories = ["Moje knihovna", "Knihy", "Časopisy", "Texty", "Zvukové dokumenty", "Zora", "Knihy Mathilda", "Filmy Mathilda", "Knihy SPVK", "MP3 KTN", "Texty KTN", "Novinky KTN za posledních 60 dnů", "Jeden svět (Mathilda)", "Hledání", "Informace o uživateli", "Nastavení"]
  categoriesNews = []
  categoriesNewsKtn = []
  currentLevel = 0
  browsePath = []
  addLevel = -1
  addPath = []
  resultsTree = ET.Element('root')
  passKey = False
  sNazev = ""
  sAutor = ""
  sAnotace = ""
  sRokVydani = ""
  sNakladatelstvi = ""
  sMistoVydani = ""
  sIsbn = ""
  sNacitatel = ""
  sVse = ""
  kddLibraryFolder = os.path.join("Documents", "Knihovna OKO")
  newsDisplayed = False
  okoMarkFilename = "OkoMark.xml"
  bookmarksFileName = "Bookmarks.xml"
  selCat = ""
  startupMyLibrary = False
  magArchiveList = []
  allowEnter = False
  libraryType = ""
  downloadUrl = ""
  downloadTarget = ""
  downloadDok = None
  downloadList = []
  zoraMp3Page = ""
  zoraMp3PageHtml = ""
  zoraTextyLogged = False
  zoraTextyPage = ""
  zoraTextyPageHtml = ""
  zoraMagPageHtml = ""
  jedenSvetDict = dict()
  catalogKddLoaded = False
  catalogMathildaBookLoaded = False
  catalogMathildaMovieLoaded = False
  catalogSpvkLoaded = False
  catalogKtnLoaded = False
  allCatalogsLoaded = False
  currentIndex = 0

  _instance = None

  def __new__(cls, *args, **kwargs):
    # Make this a singleton.
    if KddMainDialog._instance is None:
      return super(KddMainDialog, cls).__new__(cls, *args, **kwargs)
    return KddMainDialog._instance

  def __init__(self, parent, startMyLibrary):
    if KddMainDialog._instance is not None:
      return
    KddMainDialog._instance = self

    self.startupMyLibrary = startMyLibrary

    # The title of a dialog
    super(KddMainDialog, self).__init__(parent, size=(1000, 800), title=mainTitle)

    self.Bind(wx.EVT_CLOSE, self.onClose)
    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self,orientation=wx.VERTICAL)
    self.choices = []

    # the menu bar
    menuBar = wx.MenuBar()
    menu = wx.Menu()
    
    # menu items
    item = menu.Append(wx.ID_ANY, "&Informace o titulu\tF2")
    self.Bind(wx.EVT_MENU, self.dokumentInfoItem, item)
    item = menu.Append(wx.ID_ANY, "&Ohlásit anotaci\tCtrl+F2")
    self.Bind(wx.EVT_MENU, self.sayDokumentAnotaceItem, item)
    item = menu.Append(wx.ID_ANY, "Kde &jsem\tSpace")
    self.Bind(wx.EVT_MENU, self.whereamiItem, item)
    if not startMyLibrary:
      item = menu.Append(wx.ID_ANY, "&Najít titul\tCtrl+F")
      self.Bind(wx.EVT_MENU, self.kddSearchItem, item)
      item = menu.Append(wx.ID_ANY, "&Moje knihovna\tCtrl+K")
      self.Bind(wx.EVT_MENU, self.myLibraryItem, item)
      item = menu.Append(wx.ID_ANY, "Informace o &uživateli\tCtrl+U")
      self.Bind(wx.EVT_MENU, self.userInfoItem, item)
    item = menu.Append(wx.ID_ANY, "Hlavní &nabídka\tCtrl+Backspace")
    self.Bind(wx.EVT_MENU, self.levelRootItem, item)
    item = menu.Append(wx.ID_ANY, "Nápo&věda")
    self.Bind(wx.EVT_MENU, self.helpItem, item)
    menu.AppendSeparator()
    item = menu.Append(wx.ID_EXIT, "U&končit")
    self.Bind(wx.EVT_MENU, self.onClose, item)
    
    # the menu title
    menuBar.Append(menu, "Knihovna")
    self.SetMenuBar(menuBar)

    # move data folder to new location
    self.moveUserDataToMarsus()

    # if not present, create library data folder
    if not os.path.exists(libraryDataFolder):
      try:
        os.makedirs(libraryDataFolder, exist_ok=True)
      except Exception as e:
        log.error("Nebylo možné vytvořit uživatelskou složku " + libraryDataFolder + ", chyba: " +e)

    # create browsePath list containing empty items
    self.initbrowsePath()

    if not startMyLibrary:
      self.loadConfigs()

      # load all catalogs in separate thread, without blocking main program
      t = threading.Thread(
        name=f"{self.__class__.__module__}.{self.__init__.__qualname__}",
        target=self.loadCatalogs
      )
      t.setDaemon(True)
      t.start()
      
      # assign main categories
      # insert news item if needed
      self.insertNews()
      
      # find index of item Záložky
      bookmarkIndexList = [n for n, l in enumerate(self.categories) if l.startswith('Záložky')]
      if not os.path.isfile(os.path.join(libraryDataFolder, self.bookmarksFileName)):
        # item "záložky" remains on list after closing app, so in this case we delete it
        # if the item is present
        if bookmarkIndexList:
          del self.categories[bookmarkNewsList[0]]
      else:
        # if the item is not present
        if not bookmarkIndexList:
          # insert item Záložky after item Hledání
          self.categories.insert(self.categories.index("Hledání")+1, "Záložky")
      self.choices = self.categories
      self.currentLevel = 0

    self.mainList = wx.ListBox(self, choices=self.choices)
    self.mainList.Bind(wx.EVT_LISTBOX_DCLICK, self.onmainListChoice)
    self.mainList.Bind(wx.EVT_KEY_UP, self.onmainListKeyUp)
    self.mainList.SetMinSize((1000,800));
    sHelper.addItem(self.mainList, flag = wx.EXPAND)

    mainSizer.Add(sHelper.sizer, flag=wx.ALL, border=guiHelper.BORDER_FOR_DIALOGS)
    self.Sizer = mainSizer

    self.mainList.SetFocus()
    self.CentreOnScreen()
    if startMyLibrary:
      self.SetTitle("Moje knihovna")
      if not self.displayMyLibrary():
        self.onClose(self)
        return
      else:
        self.browsePath[0] = "Moje knihovna"
    self.mainList.Selection = 0

    self.allowEnter = False
    # the timer which don't allow react to Enter key shortly after running this module
    self.timer = wx.Timer(self)
    self.timer.StartOnce(100)
    self.Bind(wx.EVT_TIMER, self.enoughRuntime)

  def moveUserDataToMarsus(self):
    okoFolder = os.path.join(os.getenv('APPDATA'), "Marsus", "NVDA", "OKO")
    oldOkoFolder = os.path.join(os.getenv('APPDATA'), "ACE Design", "OKO_NVDA")
    if os.path.exists(okoFolder):
      return
    if os.path.exists(oldOkoFolder):
      import shutil
      try:
        os.makedirs(okoFolder, exist_ok=True)
        for item in os.listdir(oldOkoFolder):
          sourcePath = os.path.join(oldOkoFolder, item)
          destPath = os.path.join(okoFolder, item)
          if os.path.isfile(sourcePath):
            shutil.copy2(sourcePath, destPath)
          elif os.path.isdir(sourcePath):
            shutil.copytree(sourcePath, destPath)  # Kopírování podsložky
        shutil.rmtree(oldOkoFolder)

      except Exception as e:
        log.error(f"Chyba při přenosu uživatelských dat: {e}")

  def enoughRuntime(self, evt):
    self.allowEnter = True

  def initbrowsePath(self):
    if len(self.browsePath) > 0:
      for x in range(6):
        self.browsePath[x] = ""
    else:
      for x in range(6):
        self.browsePath.append("")
    if len(self.addPath) > 0:
      for x in range(6):
        self.addPath[x] = ""
    else:
      for x in range(6):
        self.addPath.append("")

  def __del__(self):
    KddMainDialog._instance = None

  def loadConfigs(self):
    # load KDD configuration
    readcfg()
    # load Mathilda configuration
    readcfgMathilda()
    # load SPVK configuration
    readcfgSpvk()
    # read config for KTN
    readcfgKtn()
    # load Zora configuration
    readcfgZora()

  def loadCatalogs(self):
    # KDD catalog
    self.catalogKddLoaded = self.loadKddCatalog()
    if self.catalogKddLoaded:
      if newKddCatalogDownloaded:
        self.insertNews()
        if self.currentLevel == 0:
          # refresh main menu for news to appear
          curIndex = self.mainList.GetSelection()
          self.levelRoot()
          self.mainList.Selection = curIndex
        speech.speakMessage("Nový katalog KDD byl načten.")
      else:
        speech.speakMessage("Katalog KDD byl načten.")
    else:
      speech.speakMessage("Při načítání katalogu KDD došlo k chybě.")
    # Knihy Mathilda catalog
    self.catalogMathildaBookLoaded = self.loadMathildaBookCatalog()
    if self.catalogMathildaBookLoaded:
      speech.speakMessage("Katalog Knihy Mathilda byl stažen a načten.")
    else:
      speech.speakMessage("Při načítání katalogu Knihy Mathilda došlo k chybě.")
    # Filmy Mathilda catalog
    self.catalogMathildaMovieLoaded = self.loadMathildaMovieCatalog()
    if self.catalogMathildaMovieLoaded:
      speech.speakMessage("Katalog Filmy Mathilda byl stažen a načten.")
    else:
      speech.speakMessage("Při načítání katalogu Filmy Mathilda došlo k chybě.")
    # SPVK catalog
    if self.loadSpvkCatalog():
      self.catalogSpvkLoaded = True
      speech.speakMessage("Katalog SPVK byl stažen a načten.")
    else:
      speech.speakMessage("Nepodařilo se stáhnout katalog SPVK.")
    # KTN catalog
    self.loadKtnCatalog()
    self.catalogKtnLoaded = True
    if newKtnCatalogDownloaded:
      speech.speakMessage("Nový katalog KTN byl načten.")
    else:
      speech.speakMessage("Katalog KTN byl načten.")
    # all catalogs are loaded
    self.allCatalogsLoaded = True
    # login to KDD
    if kddLogin():
      speech.speakMessage("Přihlášen do KDD jako " + kddUser + ".")

  def loadKddCatalog(self):
    # download KDD catalog
    kddCatalog()

    # get XML catalog from KDD XML file
    xmlFile = os.path.join(libraryDataFolder, "dokumenty.xml")
    try:
      treexml = ET.parse(xmlFile)
      self.tree = treexml.getroot()
    except:
      return False
    return True

  def loadMathildaBookCatalog(self):
    # get XML books from Mathilda web
    xmlUrl = "https://www.dkzp.cz/wp-load.php?security_token=dc67ebad0c3ef52c&export_id=1&action=get_data"
    try:
      response = urlopen(xmlUrl).read()
    except:
      speech.speakMessage("Chyba při načítání Mathildy.")
      return False
    try:
      self.treeMathildaBooks = ET.fromstring(response)
    except:
      return False
    return True

  def loadMathildaMovieCatalog(self):
    # get XML movies from Mathilda web
    xmlUrl = "https://www.dkzp.cz/wp-load.php?security_token=5c664a72b849958f&export_id=2&action=get_data"
    try:
      response = urlopen(xmlUrl).read()
    except:
      speech.speakMessage("Chyba při načítání filmů z Mathildy.")
      return False
    try:
      self.treeMathildaMovies = ET.fromstring(response)
    except:
      return False
    return True

  def loadSpvkCatalog(self):
    # download SPVK catalog
    if not spvkCatalog():
      return False
    else:
      # get XML catalog from SPVK XML file
      xmlFile = os.path.join(libraryDataFolder, "catalog.xml")
      treexml = ET.parse(xmlFile)
      self.treeSpvk = treexml.getroot()
      return True

  def loadKtnCatalog(self):
    # download KTN catalog if needed
    ktnCatalog()

    # get XML catalog from KTN XML file
    xmlFile = os.path.join(libraryDataFolder, "ktncat.xml")

    # load catalog from xmlFile
    treexml = ET.parse(xmlFile)
    self.treeKtn = treexml.getroot()

  def helpItem(self, evt):
    self.passKey = True
    from . import addonDoc
    os.startfile(addonDoc.getDocPath("library.html"))

  def insertNews(self):
    if kddUpdateBefore:
      newsDate = timestamptoDateTime(kddUpdateBefore)
      # item "Nové tituly" remains on list after closing app, so in this case we delete it before inserting
      # find index of item Nové tituly
      newsIndexList = [n for n, l in enumerate(self.categories) if l.startswith('Nové tituly')]
      # if the item is present
      if newsIndexList:
        del self.categories[newsIndexList[0]]
      # insert item Nové tituly after item Zora
      self.categories.insert(self.categories.index("Zora")+1, "Nové tituly od " + newsDate.strftime('%d.%m.%Y'))
      self.newsDisplayed = True

  def createTreeDokJedensvet(self, itemstr):
    dok = ET.Element("jedensvet")
    strlist = itemstr.split(",", 1)
    strlist2 = strlist[1].rsplit(" ", 1)
    dokNazev = strlist2[0].strip()
    dokPoradoveCislo = strlist2[1].strip()
    dokRokVydani = strlist[0].strip()
    # add name of the magazine
    magNameLm = ET.Element("nazev")
    if dokNazev:
      magNameLm.text = dokNazev
    dok.append(magNameLm)
    # add number of the magazine
    magNumberLm = ET.Element("cislo")
    if dokPoradoveCislo:
      magNumberLm.text = dokPoradoveCislo
    dok.append(magNumberLm)
    # add year of the magazine
    magYearLm = ET.Element("rok")
    if dokRokVydani:
      magYearLm.text = dokRokVydani
    dok.append(magYearLm)
    return dok

  def createTreeDokZoraMp3(self, itemstr):
    dok = ET.Element("zora")
    strlist = itemstr.split(",", 1)
    strlist2 = strlist[1].rsplit(" ", 1)
    dokNazev = strlist2[0].strip()
    dokPoradoveCislo = strlist2[1].strip()
    dokRokVydani = strlist[0].strip()
    # add name of the magazine
    magNameLm = ET.Element("nazev")
    if dokNazev:
      magNameLm.text = dokNazev
    dok.append(magNameLm)
    # add number of the magazine
    magNumberLm = ET.Element("cislo")
    if dokPoradoveCislo:
      magNumberLm.text = dokPoradoveCislo
    dok.append(magNumberLm)
    # add year of the magazine
    magYearLm = ET.Element("rok")
    if dokRokVydani:
      magYearLm.text = dokRokVydani
    dok.append(magYearLm)
    return dok

  def createTreeDokZoraTexty(self, itemstr):
    dok = ET.Element("zora")
    strlist = itemstr.rsplit(",", 1)
    strlist2 = strlist[0].rsplit(" ", 1)
    dokNazev = strlist2[0].strip()
    dokPoradoveCislo = strlist2[1].strip()
    dokRokVydani = strlist[1].strip()
    # add name of the magazine
    magNameLm = ET.Element("nazev")
    if dokNazev:
      magNameLm.text = dokNazev
    dok.append(magNameLm)
    # add number of the magazine
    magNumberLm = ET.Element("cislo")
    if dokPoradoveCislo:
      magNumberLm.text = dokPoradoveCislo
    dok.append(magNumberLm)
    # add year of the magazine
    magYearLm = ET.Element("rok")
    if dokRokVydani:
      magYearLm.text = dokRokVydani
    dok.append(magYearLm)
    return dok

  def userData(self):
    global kddUser
    try:
      KddUserDataDialog(self).ShowModal()
    except Exception as e:
      self.Enable()
      raise e
    if kddUser:
      self.SetTitle(mainTitle + ", " + kddUser)
    else:
      self.SetTitle(mainTitle)
    
  def userDataMathilda(self):
    try:
      MathildaUserDataDialog(self).ShowModal()
    except Exception as e:
      self.Enable()
      raise e

  def userDataSpvk(self):
    try:
      SpvkUserDataDialog(self).ShowModal()
    except Exception as e:
      self.Enable()
      raise e

  def userDataKtn(self):
    try:
      KtnUserDataDialog(self).ShowModal()
    except Exception as e:
      self.Enable()
      raise e

  def userDataZora(self):
    try:
      ZoraUserDataDialog(self).ShowModal()
    except Exception as e:
      self.Enable()
      raise e

  def userInfo(self):
    infoText = "KDD: \n"
    if kddLogged:
      global kddUser
      global kddCredits
      global kddBonusCredits
      global kddLastLogin
      infoText = infoText + "Přihlášený uživatel: " + kddUser + "\n"
      infoText = infoText + "Počet kreditů: " + str(kddCredits) + "\n"
      infoText = infoText + "Počet bonusových kreditů: " + str(kddBonusCredits)  + "\n"
      infoText = infoText + "Datum posledního přihlášení: " + kddLastLogin + "\n"
    else:
      infoText = infoText + "Není přihlášen žádný uživatel. \n"
    infoText = infoText + "\n"
    infoText = infoText + "Mathilda: \n"
    if mathildaUserLogin:
      infoText = infoText + "uživatelské údaje jsou vyplněny."
    else:
      infoText = infoText + "nejsou zadány uživatelské údaje."
    infoText = infoText + "\n"
    infoText = infoText + "SPVK: \n"
    if spvkUserLogin:
      infoText = infoText + "uživatelské údaje jsou vyplněny."
    else:
      infoText = infoText + "nejsou zadány uživatelské údaje."
    infoText = infoText + "\n"
    infoText = infoText + "KTN: \n"
    if ktnUserLogin:
      infoText = infoText + "uživatelské údaje jsou vyplněny."
    else:
      infoText = infoText + "nejsou zadány uživatelské údaje."
    TextReader.dlgTitle = "Informace o uživateli"
    TextReader.dispText = infoText
    TextReader.isReader = False
    self.displayInfo()

  def afterReadingOrPlayingFinish(self):
    global finalSelection
    if (TextReader.isReader and self.currentLevel < 5) or (finalSelection == "Přehrát"):
      if finalSelection == "Přehrát":
        finalSelection = ""
      flist = self.myLibrary()
      self.fillmainList(flist)
      self.mainList.Selection = self.currentIndex
  
  def displayInfo(self):
    self.Hide()
    self.Disable()
    try:
      TextReader(self).Show()
    except Exception as e:
      self.Enable()
      raise e

  def spvkGetCategories(self, dok):
    categorystr = ""
    for category in dok.iter("nazev_kategorie"):
      if not categorystr:
        categorystr = categorystr + category.text
      else:
        categorystr = categorystr + ", " + category.text
    return categorystr
  
  def ktnGetAuthors(self, dok):
    autorstr = ""
    autorField = dok.find("datafield[@tag='100']")
    if autorField:
      sbField = autorField.find("subfield[@code='a']")
      autorstr = sbField.text
      autorstr = autorstr.replace(",", "")
    return autorstr

  def spvkGetAuthors(self, dok):
    autorstr = ""
    for autor in dok.iter("nazev_autora"):
      if not autorstr:
        autorstr = autorstr + autor.text
      else:
        autorstr = autorstr + ", " + autor.text
    return autorstr
  
  def mathildaGetAuthors(self, dok):
    autorstr = ""
    for autor in dok.iter("author"):
      if not autorstr:
        autorstr = autorstr + autor.text
      else:
        autorstr = autorstr + ", " + autor.text
    return autorstr
  
  def isinLibrary(self, dok):
    titleinLibrary = False
    titList = []
    searchedTitle = ""
    from os.path import expanduser
    home = expanduser("~")
    kddLibraryPath = os.path.join(home, self.kddLibraryFolder)
    if os.path.exists(kddLibraryPath):
      subfoldersList = [f.path for f in os.scandir(kddLibraryPath) if f.is_dir()]
      titList = [os.path.split(x)[1] for x in subfoldersList if os.path.isfile(os.path.join(x, self.okoMarkFilename))]
    if len(titList) > 0:
      if self.libraryType == "JEDENSVET":
        searchedTitle = self.dokumentFolderNameZora(dok)
      elif self.libraryType.startswith("ZORA"):
        searchedTitle = self.dokumentFolderNameZora(dok)
      elif self.libraryType.startswith("KTN"):
        searchedTitle = self.dokumentFolderNameKtn(dok)
      elif self.libraryType == "SPVK":
        searchedTitle = self.dokumentFolderNameSpvk(dok)
      elif self.libraryType == "Mathilda":
        searchedTitle = self.dokumentFolderNameMathilda(dok)
      else:
        searchedTitle = self.dokumentFolderName(dok)
    if searchedTitle in titList:
      titleinLibrary = True
    return titleinLibrary

  def dokumentPrice(self, dok):
    dokKategorieDokumentu = dok.find("kategorie_dokumentu_nazev_cs").text
    dokZdarma = dok.find("zdarma").text
    if dokZdarma == "ano":
      price = 0
    elif dokKategorieDokumentu == "kniha":
      price = 1000
    elif dokKategorieDokumentu == "text":
      price = 1000
    elif dokKategorieDokumentu == "audio_dokument":
      price = 100
    elif dokKategorieDokumentu == "časopis":
      price = 250
    elif dokKategorieDokumentu == "casopis_zora":
      price = 0
    return price

  def dokumentInfo(self, dok):
    infoText = ""
    if not self.libraryType or self.libraryType == "KDD":
      dokId = dok.find("id-dokumentu").text
      dokNazev = dok.find("nazev").text
      dokPodnazev = dok.find("podnazev").text
      try:
        dokPoradoveCislo = dok.find("poradove_cislo").text
      except:
        dokPoradoveCislo = ""
      try:
        dokFormatDokumentu  = dok.find("format_dokumentu").text
      except:
        dokFormatDokumentu = ""
      dokPrekladNazvu = dok.find("preklad_nazvu").text
      dokVydani = dok.find("vydani").text
      dokMistoVydani = dok.find("misto_vydani").text
      dokRokVydani = dok.find("rok_vydani").text
      dokRozsah = dok.find("rozsah").text
      dokFormat = dok.find("format").text
      dokEdice = dok.find("edice_nazev_cs").text
      dokIsbn = dok.find("isbn").text
      dokIssn = dok.find("issn").text
      dokAnotace = dok.find("anotace").text
      dokVelikostSouboru = dok.find("velikost_souboru").text
      dokPoznamka = dok.find("poznamka").text
      dokAutorNeprimy = dok.find("autor_neprimy").text
      dokKlicovaSlova = dok.find("klicova_slova").text
      dokDatumSchvaleni = dok.find("datum_schvaleni").text
      dokZdarma = dok.find("zdarma").text
      dokPoradiEdice = dok.find("poradi_edice").text
      dokPeriodicita = dok.find("periodicita_vydavani").text
      dokSkenovac = dok.find("skenovac").text
      dokAutori = dok.find("autori").text
      dokFormy = dok.find("formy").text
      dokPredmetoveSkupiny = dok.find("predmetove_skupiny").text
      dokTypDokumentu = dok.find("typ_dokumentu_nazev_cs").text
      dokStavZdroje = dok.find("stav_zdroje_nazev_cs").text
      dokKodovani = dok.find("kodovani_nazev_cs").text
      dokStavPoZpracovani = dok.find("stav_po_zpracovani_nazev_cs").text
      dokNakladatelstvi = dok.find("nakladatelstvi_nazev_cs").text
      dokFormatZdroje = dok.find("format_zdroje_nazev_cs").text
      dokFormatZpracovani = dok.find("format_zpracovani_nazev_cs").text
      dokKategorieDokumentu = dok.find("kategorie_dokumentu_nazev_cs").text
      try:
        dokReziseri = dok.find("reziseri").text
      except:
        dokReziseri = ""
      try:
        dokProducent = dok.find("producent").text
      except:
        dokProducent = ""
      try:
        dokZemePuvodu = dok.find("zeme_puvodu").text
      except:
        dokZemePuvodu = ""
      try:
        dokObsazeni = dok.find("obsazeni").text
      except:
        dokObsazeni = ""
      try:
        dokDelka = dok.find("delka").text
      except:
        dokDelka = ""
      try:
        dokProdukceAd = dok.find("produkce_ad").text
      except:
        dokProdukceAd = ""
      try:
        dokNapsalAd = dok.find("napsal_ad").text
      except:
        dokNapsalAd = ""
      try:
        dokNamluvilAd = dok.find("namluvil_ad").text
      except:
        dokNamluvilAd = ""
      try:
        dokRokVyrobyAd = dok.find("rok_vyroby_ad").text
      except:
        dokRokVyrobyAd = ""
      try:
        dokSerie = dok.find("serie").text
      except:
        dokSerie = ""
      try:
        dokCisloEpizody = dok.find("cislo_epizody").text
      except:
        dokCisloEpizody = ""
      try:
        dokPocetEpizod = dok.find("pocet_epizod").text
      except:
        dokPocetEpizod = ""
      try:
        dokCisloSerie = dok.find("cislo_serie").text
      except:
        dokCisloSerie = ""
      try:
        dokNazevEpizody = dok.find("nazev_epizody").text
      except:
        dokNazevEpizody = ""
      try:
        dokOriginalniNazevEpizody = dok.find("originalni_nazev_epizody").text
      except:
        dokOriginalniNazevEpizody = ""
      try:
        dokOriginalniNazevSerialu = dok.find("originalni_nazev_serialu").text
      except:
        dokOriginalniNazevSerialu = ""
      try:
        dokAnotaceEpizody = dok.find("anotace_epizody").text
      except:
        dokAnotaceEpizody = ""
      try:
        dokViceInformaci = dok.find("vice_informaci").text
      except:
        dokViceInformaci = ""
      try:
        dokproDetiVhodne = dok.find("pro_deti_vhodne").text
      except:
        dokproDetiVhodne = ""
      if dokKategorieDokumentu == "kniha" or dokKategorieDokumentu == "časopis" or dokKategorieDokumentu == "text" or dokKategorieDokumentu == "casopis_zora"or dokKategorieDokumentu == "audio_dokument":
        infoText = "Název: " + dokNazev + "\n"
        if dokPodnazev:
          infoText = infoText + "Podnázev: " + dokPodnazev + "\n"
        if dokPoradoveCislo:
          infoText = infoText + "Pořadové číslo: " + dokPoradoveCislo + "\n"
        if dokNazevEpizody:
          infoText = infoText + "Název epizody: " + dokNazevEpizody + "\n"
        if dokCisloEpizody:
          infoText = infoText + "Číslo epizody: " + dokCisloEpizody + "\n"
        if dokFormatDokumentu:
          dokFormatDokumentu = dokFormatDokumentu.replace("|", ", ")
          infoText = infoText + "Formát dokumentu: " + dokFormatDokumentu + "\n"
        if dokPrekladNazvu:
          infoText = infoText + "Překlad názvu: " + dokPrekladNazvu + "\n"
        if dokOriginalniNazevSerialu:
          infoText = infoText + "Originální název seriálu: " + dokOriginalniNazevSerialu + "\n"
        if dokOriginalniNazevEpizody:
          infoText = infoText + "Originální název epizody: " + dokOriginalniNazevEpizody + "\n"
        if dokAutori:
          infoText = infoText + "Autoři: " + dokAutori + "\n"
        if dokAutorNeprimy:
          infoText = infoText + "Autor nepřímý: " + dokAutorNeprimy + "\n"
        if dokEdice:
          infoText = infoText + "Série: " + dokEdice
          if dokPoradiEdice:
            infoText = infoText + ", číslo: " + dokPoradiEdice + "\n"
          else:
            infoText = infoText + "\n"
        if dokAnotace:
          infoText = infoText + "Anotace: " + dokAnotace.strip() + "\n"
        if dokAnotaceEpizody:
          infoText = infoText + "Anotace epizody: " + dokAnotaceEpizody.strip() + "\n"
        if dokSerie:
          infoText = infoText + "Série: " + dokSerie + "\n"
        if dokCisloSerie:
          infoText = infoText + "Číslo série: " + dokCisloSerie + "\n"
        if dokPocetEpizod:
          infoText = infoText + "Počet epizod: " + dokPocetEpizod + "\n"
        if dokVydani:
          infoText = infoText + "Vydání: " + dokVydani + "\n"
        if dokRokVydani:
          infoText = infoText + "Rok vydání: " + dokRokVydani + "\n"
        if dokRozsah:
          infoText = infoText + "Rozsah: " + dokRozsah + "\n"
        if dokFormat:
          infoText = infoText + "Formát: " + dokFormat + "\n"
        if dokIsbn:
          infoText = infoText + "ISBN: " + dokIsbn + "\n"
        if dokIssn:
          infoText = infoText + "ISSN: " + dokIssn + "\n"
        if dokReziseri:
          infoText = infoText + "Režiséři: " + dokReziseri + "\n"
        if dokProducent:
          infoText = infoText + "Producent: " + dokProducent + "\n"
        if dokZemePuvodu:
          infoText = infoText + "Země původu: " + dokZemePuvodu + "\n"
        if dokObsazeni:
          infoText = infoText + "Obsazení: " + dokObsazeni + "\n"
        if dokDelka:
          infoText = infoText + "Délka: " + dokDelka + " minut\n"
        if dokProdukceAd:
          infoText = infoText + "Produkce audiopopisu: " + dokProdukceAd + "\n"
        if dokNapsalAd:
          infoText = infoText + "Autor audiopopisu: " + dokNapsalAd + "\n"
        if dokNamluvilAd:
          infoText = infoText + "Načitatel audiopopisu: " + dokNamluvilAd + "\n"
        if dokRokVyrobyAd:
          infoText = infoText + "Rok výroby audiopopisu: " + dokRokVyrobyAd + "\n"
        if dokViceInformaci:
          infoText = infoText + "Více informací: " + dokViceInformaci + "\n"
        if dokproDetiVhodne:
          infoText = infoText + "Vhodné pro děti: " + dokproDetiVhodne + "\n"
        if dokId:
          infoText = infoText + "ID dokumentu: " + dokId + "\n"
        if dokVelikostSouboru:
          infoText = infoText + "Velikost souboru: " + dokVelikostSouboru + "\n"
        if dokPoznamka:
          infoText = infoText + "Poznámka: " + dokPoznamka + "\n"
        if dokDatumSchvaleni:
          infoText = infoText + "Datum zařazení: " + dokDatumSchvaleni + "\n"
        if dokSkenovac:
          infoText = infoText + "Naskenováno od: " + dokSkenovac + "\n"
        if dokFormy:
          infoText = infoText + "Formy: " + dokFormy + "\n"
        if dokPredmetoveSkupiny:
          infoText = infoText + "Předmětové skupiny: " + dokPredmetoveSkupiny + "\n"
        if dokTypDokumentu:
          infoText = infoText + "Typ dokumentu: " + dokTypDokumentu + "\n"
        if dokStavZdroje:
          infoText = infoText + "Stav zdroje: " + dokStavZdroje + "\n"
        if dokKodovani:
          infoText = infoText + "Kódování: " + dokKodovani + "\n"
        if dokStavPoZpracovani:
          infoText = infoText + "Stav po zpracování: " + dokStavPoZpracovani + "\n"
        if dokNakladatelstvi:
          infoText = infoText + "Nakladatelství: " + dokNakladatelstvi + "\n"
        if dokFormatZdroje:
          infoText = infoText + "Formát zdroje: " + dokFormatZdroje + "\n"
        if dokFormatZpracovani:
          infoText = infoText + "Formát zpracování: " + dokFormatZpracovani + "\n"
      dokPrice = self.dokumentPrice(dok)
      if dokPrice == 0:
        infoText = infoText + "Cena: zdarma" + "\n"
      else:
        infoText = infoText + "Cena: " + str(dokPrice) + " kreditů" + "\n"
    elif self.libraryType == "Mathilda":
      dokId = dok.find("id").text
      dokNazev = dok.find("title").text
      dokAutori = self.mathildaGetAuthors(dok)
      dokAnotace = dok.find("description").text
      dokKategorie = dok.find("category").text
      if dokKategorie:
        dokKategorie = dokKategorie.replace(">", ", ")
      dokDatum = dok.find("date").text
      dokStazeni = dok.find("downloads").text
      if dokNazev:
        infoText = infoText + "Název: " + dokNazev + "\n"
      if dokAnotace:
        infoText = infoText + "Anotace: " + dokAnotace + "\n"
      if dokAutori:
        infoText = infoText + "Autoři: " + dokAutori + "\n"
      if dokKategorie:
        infoText = infoText + "Kategorie: " + dokKategorie + "\n"
      if dokStazeni:
        infoText = infoText + "Počet stažení: " + dokStazeni + "\n"
      if dokDatum:
        infoText = infoText + "Datum zařazení: " + dokDatum + "\n"
      if dokId:
        infoText = infoText + "Id titulu: " + dokId + "\n"
    elif self.libraryType == "SPVK":
      dokId = dok.find("id-dokumentu").text
      dokNazev = dok.find("nazev").text
      dokAutori = self.spvkGetAuthors(dok)
      dokAnotace = dok.find("anotace").text
      dokKategorie = self.spvkGetCategories(dok)
      dokDatum = dok.find("datum_pridani").text
      dokStazeni = dok.find("pocet_stazeni").text
      if dokNazev:
        infoText = infoText + "Název: " + dokNazev + "\n"
      if dokAnotace:
        infoText = infoText + "Anotace: " + dokAnotace + "\n"
      if dokAutori:
        infoText = infoText + "Autoři: " + dokAutori + "\n"
      if dokKategorie:
        infoText = infoText + "Kategorie: " + dokKategorie + "\n"
      if dokStazeni:
        infoText = infoText + "Počet stažení: " + dokStazeni + "\n"
      if dokDatum:
        infoText = infoText + "Datum zařazení: " + dokDatum + "\n"
      if dokId:
        infoText = infoText + "Id titulu: " + dokId + "\n"
    elif self.libraryType.startswith("KTN"):
      dokPodnazev = ""
      dokKategorie = ""
      dokAnotace = ""
      dokIsbn = ""
      dokVydani = ""
      dokPoznamka = ""
      dokVyroba = ""
      dokNacitatel = ""
      dokDisky = ""
      dokCelkovyCas = ""
      dokDatum = ""
      dtField = dok.find("datafield[@tag='245']")
      sbField = dtField.find("subfield[@code='a']")
      dokNazev = sbField.text
      sbField = dtField.find("subfield[@code='b']")
      if sbField is not None and sbField.text:
        dokPodnazev = sbField.text
        dokPodnazev = dokPodnazev.strip("/")
      ctrlField = dok.find("controlfield[@tag='001']")
      dokId = ctrlField.text
      dokAutori = self.ktnGetAuthors(dok)
      ctrlField = dok.find("controlfield[@tag='008']")
      rawDate = ctrlField.text[:6]
      if rawDate:
        tdate = datetime.datetime.now()
        tyear = tdate.year
        kyear = int("20" + rawDate[0:2])
        if tyear < kyear:
          kyear = kyear - 100
        dokDatum = rawDate[4:6] + "." + rawDate[2:4] + "." + str(kyear)
      dtField = dok.find("datafield[@tag='520']")
      if dtField is not None:
        sbField = dtField.find("subfield[@code='a']")
        dokAnotace = sbField.text
      for dtField in dok.findall("datafield[@tag='653']"):
        if dtField.attrib["ind1"] == "1":
          sbField = dtField.findall("subfield[@code='a']")
          for cat in sbField:
            if not dokKategorie:
              dokKategorie = cat.text
            else:
              dokKategorie = dokKategorie + ", " + cat.text
      if dokKategorie:
        dokKategorie = dokKategorie + " - "
      subcat = ""
      for dtField in dok.findall("datafield[@tag='653']"):
        if dtField.attrib["ind1"] == "2":
          sbField = dtField.findall("subfield[@code='a']")
          for cat in sbField:
            if not subcat:
              subcat = cat.text
            else:
              subcat = subcat + ", " + cat.text
      if subcat:
        dokKategorie = dokKategorie + subcat
      dtField = dok.find("datafield[@tag='534']")
      sbField = dtField.find("subfield[@code='c']")
      if sbField.text:
        dokVydani = sbField.text
      sbField = dtField.find("subfield[@code='b']")
      if sbField.text:
        if dokVydani:
          dokVydani = dokVydani + ", " + sbField.text
        else:
          dokVydani = sbField.text
      sbField = dtField.find("subfield[@code='z']")
      if sbField is not None and sbField.text:
        dokIsbn = sbField.text
      dtField = dok.find("datafield[@tag='500']")
      if dtField is not None:
        sbField = dtField.find("subfield[@code='a']")
        dokPoznamka = sbField.text
      dtField = dok.find("datafield[@tag='260']")
      sbField = dtField.find("subfield[@code='a']")
      if sbField is not None and sbField.text:
        dokVyroba = sbField.text
      sbField = dtField.find("subfield[@code='b']")
      if sbField is not None and sbField.text:
        if dokVyroba:
          if dokVyroba[-1] != ",":
            dokVyroba = dokVyroba + ","
          dokVyroba = dokVyroba + " " + sbField.text
        else:
          dokVyroba = sbField.text
      sbField = dtField.find("subfield[@code='c']")
      if sbField is not None:
        if sbField.text:
          if dokVyroba:
            if dokVyroba[-1] != ",":
              dokVyroba = dokVyroba + ","
            dokVyroba = dokVyroba + " " + sbField.text
          else:
            dokVyroba = sbField.text
      if self.libraryType == "KTNMP3":
        dokDisky = str(self.getDisksKtn(dok))
        dtField = dok.find("datafield[@tag='511']")
        if dtField is not None:
          sbField = dtField.find("subfield[@code='a']")
          dokNacitatel = sbField.text
        dtField = dok.find("datafield[@tag='306']")
        if dtField is not None:
          sbField = dtField.find("subfield[@code='a']")
          casstr = sbField.text
          hourstr = casstr[:2]
          minstr = casstr[2:4]
          secstr = casstr[4:6]
          if int(hourstr) > 0:
            dokCelkovyCas = hourstr.lstrip("0")
            if int(hourstr) == 1:
              dokCelkovyCas = dokCelkovyCas + " hodina, "
            elif 2 <= int(hourstr) <= 4:
              dokCelkovyCas = dokCelkovyCas + " hodiny, "
            else:
              dokCelkovyCas = dokCelkovyCas + " hodin, "
          if int(minstr) > 0:
            dokCelkovyCas = dokCelkovyCas + minstr.lstrip("0")
            if int(minstr) == 1:
              dokCelkovyCas = dokCelkovyCas + " minuta, "
            elif 2 <= int(minstr) <= 4:
              dokCelkovyCas = dokCelkovyCas + " minuty, "
            else:
              dokCelkovyCas = dokCelkovyCas + " minut, "
          if int(secstr) > 0:
            dokCelkovyCas = dokCelkovyCas + secstr.lstrip("0")
            if int(secstr) == 1:
              dokCelkovyCas = dokCelkovyCas + " vteřina"
            elif 2 <= int(secstr) <= 4:
              dokCelkovyCas = dokCelkovyCas + " vteřiny"
            else:
              dokCelkovyCas = dokCelkovyCas + " vteřin"
      if dokNazev:
        infoText = infoText + "Název: " + dokNazev + "\n"
      if dokPodnazev:
        infoText = infoText + "Podnázev: " + dokPodnazev + "\n"
      if dokAnotace:
        infoText = infoText + "Anotace: " + dokAnotace + "\n"
      if dokAutori:
        infoText = infoText + "Autoři: " + dokAutori + "\n"
      if dokKategorie:
        infoText = infoText + "Kategorie: " + dokKategorie + "\n"
      if dokIsbn:
        infoText = infoText + "ISBN: " + dokIsbn + "\n"
      if dokVydani:
        infoText = infoText + "Vydáno: " + dokVydani + "\n"
      if dokPoznamka:
        infoText = infoText + "Poznámka: " + dokPoznamka + "\n"
      if dokDisky:
        infoText = infoText + "Počet nosičů: " + dokDisky + "\n"
      if dokVyroba:
        infoText = infoText + "Vyrobeno: " + dokVyroba + "\n"
      if dokNacitatel:
        infoText = infoText + dokNacitatel + "\n"
      if dokCelkovyCas:
        infoText = infoText + "Celkový čas: " + dokCelkovyCas + "\n"
      if dokDatum:
        infoText = infoText + "V katalogu od: " + dokDatum + "\n"
      if dokId:
        infoText = infoText + "Id titulu: " + dokId + "\n"
    elif self.libraryType.startswith("ZORA") or self.libraryType == "JEDENSVET":
      dokNazev = dok.find("nazev").text
      dokPoradoveCislo = dok.find("cislo").text
      dokRokVydani = dok.find("rok").text
      if dokNazev:
        infoText = infoText + "Název: " + dokNazev + "\n"
      if dokPoradoveCislo:
        infoText = infoText + "Pořadové číslo: " + dokPoradoveCislo + "\n"
      if dokRokVydani:
        infoText = infoText + "Rok vydání: " + dokRokVydani + "\n"
    infoText = infoText + "Knihovna: "
    if self.libraryType:
      if self.libraryType.startswith("KTN"):
        dokLibrary = "KTN"
      elif self.libraryType.startswith("ZORA"):
        dokLibrary = "ZORA"
      elif self.libraryType == "JEDENSVET":
        dokLibrary = "Mathilda - Jeden svět"
      else:
        dokLibrary = self.libraryType
      infoText = infoText + dokLibrary + "\n"
    else:
      infoText = infoText + "KDD\n"
    return infoText

  def dokumentMark(self, dok, dokFolder, dokName = ""):
    from os.path import expanduser
    home = expanduser("~")

    # add name of file to read
    addToFile = False
    try:
      rfile = dok.find("filetoread")
    except:
      rfile = None
    if rfile is None:
      rfile = ET.Element("filetoread")
      addToFile = True
    if dokName:
      rfile.text = dokName
    else:
      if self.libraryType == "JEDENSVET":
        rfile.text = self.findFileToReadJedensvet(dok)
      elif self.libraryType.startswith("ZORA"):
        rfile.text = self.findFileToReadZora(dok)
      elif self.libraryType.startswith("KTN"):
        rfile.text = self.findFileToReadKtn(dok)
      elif self.libraryType == "Mathilda":
        rfile.text = self.findFileToReadMathildaMovie(dok)
      else:
        rfile.text = self.findFileToRead(dok)
    if addToFile:
      dok.append(rfile)

    # add file size
    addToFile = False
    try:
      sfile = dok.find("filesize")
    except:
      sfile = None
    if sfile is None:
      sfile = ET.Element("filesize")
      addToFile = True
    sfile.text = str(os.path.getsize(os.path.join(home, self.kddLibraryFolder, dokFolder, rfile.text)))
    if addToFile:
      dok.append(sfile)

    # add reading position
    addToFile = False
    try:
      pos = dok.find("position")
    except:
      pos = None
    if pos is None:
      pos = ET.Element("position")
      addToFile = True
    pos.text = "0"
    if addToFile:
      dok.append(pos)

    # add type of library
    addToFile = False
    try:
      ltype = dok.find("librarytype")
    except:
      ltype = None
    if ltype is None:
      ltype = ET.Element("librarytype")
      addToFile = True
    ltype.text = self.libraryType
    if addToFile:
      dok.append(ltype)

    # save element to XML file
    bookmarkFile = os.path.join(home, self.kddLibraryFolder, dokFolder, self.okoMarkFilename)
    with open(bookmarkFile, 'wb') as f:
      f.write(ET.tostring(dok, encoding='utf-8', method='xml'))

  def findFileToReadJedensvet(self, dok):
    from os.path import expanduser
    home = expanduser("~")
    dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderNameJedensvet(dok))
    filetoRead = ""
    if os.path.exists(dokFolder):
      filetoRead = os.listdir(dokFolder)[0]
      if os.path.splitext(filetoRead)[1] == ".m3u":
        filetoRead = os.listdir(dokFolder)[1]
    return filetoRead

  def findFileToReadZora(self, dok):
    from os.path import expanduser
    home = expanduser("~")
    dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderNameZora(dok))
    filetoRead = ""
    if os.path.exists(dokFolder):
      filetoRead = os.listdir(dokFolder)[0]
      if os.path.splitext(filetoRead)[1] == ".m3u":
        filetoRead = os.listdir(dokFolder)[1]
    return filetoRead

  def findFileToReadKtn(self, dok):
    from os.path import expanduser
    home = expanduser("~")
    dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderNameKtn(dok))
    filetoRead = ""
    if os.path.exists(dokFolder):
      filetoRead = os.listdir(dokFolder)[0]
      if os.path.splitext(filetoRead)[1] == ".m3u":
        filetoRead = os.listdir(dokFolder)[1]
    return filetoRead

  def findFileToReadMathildaMovie(self, dok):
    from os.path import expanduser
    home = expanduser("~")
    dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderNameMathilda(dok))
    filetoRead = ""
    if os.path.exists(dokFolder):
      filetoRead = os.listdir(dokFolder)[0]
      if os.path.splitext(filetoRead)[1].upper() == ".M3U":
        filetoRead = os.listdir(dokFolder)[1]
    return filetoRead

  def findFileToRead(self, dok):
    dokKategorieDokumentu = dok.find("kategorie_dokumentu_nazev_cs").text
    from os.path import expanduser
    home = expanduser("~")
    dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderName(dok))
    filetoRead = ""
    if os.path.exists(dokFolder):
      if dokKategorieDokumentu == "časopis":
        if os.path.exists(os.path.join(dokFolder, "obsah.txt")):
          filetoRead = "obsah.txt"
      if not filetoRead:
        for filename in os.listdir(dokFolder):
          if os.path.splitext(filename)[1] == ".mp3":
            filetoRead = filename
            break
      if not filetoRead:
        for filename in os.listdir(dokFolder):
          if os.path.splitext(filename)[1] == ".txd":
            filetoRead = filename
            break
      if not filetoRead:
        for filename in os.listdir(dokFolder):
          if os.path.splitext(filename)[1] == ".rtf":
            filetoRead = filename
            break
      if not filetoRead:
        for filename in os.listdir(dokFolder):
          if os.path.splitext(filename)[1] == ".html":
            filetoRead = filename
            break
      if not filetoRead:
        for filename in os.listdir(dokFolder):
          if os.path.splitext(filename)[1] == ".txt":
            if filename != "anotace.txt" and filename != "kddinfo.txt":
              filetoRead = filename
              break
    return filetoRead

  def displayMyLibrary(self):
    flist = self.myLibrary()
    if len(flist) == 0:
      dlgMessage = "Ve své knihovně nemáte zatím žádné tituly."
      dlgTitle = "Informace"
      wx.MessageBox(dlgMessage, dlgTitle)
      self.passKey = True
      if not self.startupMyLibrary:
        self.levelRoot()
      return False
    else:
      self.currentLevel = 4
      self.fillmainList(flist)
      return True

  def myLibrary(self):
    titList = []
    from os.path import expanduser
    home = expanduser("~")
    kddLibraryPath = os.path.join(home, self.kddLibraryFolder)
    if os.path.exists(kddLibraryPath):
      subfoldersList = [f.path for f in os.scandir(kddLibraryPath) if f.is_dir()]
      titList = [os.path.split(x)[1] for x in subfoldersList if os.path.isfile(os.path.join(x, self.okoMarkFilename))]
    if len(titList) > 0:
      # mark titles (but not KDD magazines) which users started to read
      for f in titList:
        bFile = os.path.join(kddLibraryPath, f, self.okoMarkFilename)
        bTree = ET.parse(bFile)
        bDokument = bTree.getroot()
        try:
          curDokKat = bDokument.find("kategorie_dokumentu_nazev_cs").text
        except:
          curDokKat = ""
        if curDokKat != "časopis":
          dokCursorPos = float(bDokument.find("position").text)
          if dokCursorPos > 0:
            itemPos = titList.index(f)
            filestr = bDokument.find("filetoread").text.upper()
            if ".MP3" in filestr:
              percentPos = dokCursorPos * 100
              titList[itemPos] = titList[itemPos] + ", přehráno " + str(math.trunc(percentPos)) + "%"
            else:
              fSize = int(bDokument.find("filesize").text)
              percentPos = (dokCursorPos * 100) / fSize
              titList[itemPos] = titList[itemPos] + ", přečteno " + str(math.trunc(percentPos)) + "%"
    return titList

  def markReadArticleJedensvet(self, titList, dok):
    itemPos = -1
    from os.path import expanduser
    home = expanduser("~")
    dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderNameJedensvet(dok))
    bFile = os.path.join(dokFolder, self.okoMarkFilename)
    bTree = ET.parse(bFile)
    bDokument = bTree.getroot()
    dokCursorPos = float(bDokument.find("position").text)
    titList = [x.rsplit(", přehráno", 1)[0] for x in titList]
    if dokCursorPos > 0:
      filestr = bDokument.find("filetoread").text
      itemPos = titList.index(filestr)
      percentPos = dokCursorPos * 100
      titList[itemPos] = titList[itemPos] + ", přehráno " + str(math.trunc(percentPos)) + "%"
    return titList, itemPos

  def markReadArticleKddMp3(self, titList, dok):
    itemPos = -1
    from os.path import expanduser
    home = expanduser("~")
    dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderName(dok) + " MP3")
    bFile = os.path.join(dokFolder, self.okoMarkFilename)
    bTree = ET.parse(bFile)
    bDokument = bTree.getroot()
    dokCursorPos = float(bDokument.find("position").text)
    titList = [x.rsplit(", přehráno", 1)[0] for x in titList]
    if dokCursorPos > 0:
      filestr = bDokument.find("filetoread").text
      itemPos = titList.index(self.getId3Title(os.path.join(dokFolder, filestr)))
      percentPos = dokCursorPos * 100
      titList[itemPos] = titList[itemPos] + ", přehráno " + str(math.trunc(percentPos)) + "%"
    return titList, itemPos

  def getListofArticlesZoraKddMp3(self, dok):
    from os.path import expanduser
    home = expanduser("~")
    dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderName(dok) + " MP3")
    articleList = []
    for x in os.listdir(dokFolder):
      if x.lower().endswith(".mp3"):
        articleList.append(self.getId3Title(os.path.join(dokFolder, x)))
    return articleList
  
  def getListofArticlesJedensvet(self, dok):
    from os.path import expanduser
    home = expanduser("~")
    dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderNameJedensvet(dok))
    articleList = []
    for x in os.listdir(dokFolder):
      if x.lower().endswith(".mp3"):
        articleList.append(x)
    return articleList
  
  def markReadArticleZora(self, titList, dok):
    itemPos = -1
    from os.path import expanduser
    home = expanduser("~")
    dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderNameZora(dok))
    bFile = os.path.join(dokFolder, self.okoMarkFilename)
    bTree = ET.parse(bFile)
    bDokument = bTree.getroot()
    dokCursorPos = float(bDokument.find("position").text)
    titList = [x.rsplit(", přehráno", 1)[0] for x in titList]
    if dokCursorPos > 0:
      filestr = bDokument.find("filetoread").text
      itemPos = titList.index(self.getId3Title(os.path.join(dokFolder, filestr)))
      percentPos = dokCursorPos * 100
      titList[itemPos] = titList[itemPos] + ", přehráno " + str(math.trunc(percentPos)) + "%"
    return titList, itemPos

  def getMagShortNameZora(self, htmlpage, magazine):
    indexShort = htmlpage.find(">" + magazine + "<")
    if indexShort != -1:
      indexShort2 = htmlpage.rfind('="', 0, indexShort)+2
      indexShort = htmlpage.find('"', indexShort2)
      magShortName = htmlpage[indexShort2:indexShort].strip()
    else:
      magShortName = ""
    return magShortName
    
  def getId3Title(self, mpfile):
    idr = id3reader.Reader(mpfile)
    btit = idr.getRawData('TIT2')
    if btit[0] == 1:
      # first bit is 1, set encoding to UTF-8
      utfEncoding = True
    else:
      utfEncoding = False
    tagTitle = idr.getValue('TIT2')
    if not utfEncoding:
      tagTitle = tagTitle.encode('latin1', 'ignore').decode('windows-1250', 'ignore')
    return tagTitle
  
  def getFilenameforId3(self, tag, folder):
    for x in os.listdir(folder):
      if x.lower().endswith(".mp3"):
        if tag == self.getId3Title(os.path.join(folder, x)):
          fname = x
          break
    return fname
  
  def getListofArticlesZoraMp3(self, dok):
    from os.path import expanduser
    home = expanduser("~")
    dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderNameZora(dok))
    articleList = []
    for x in os.listdir(dokFolder):
      if x.lower().endswith(".mp3"):
        articleList.append(self.getId3Title(os.path.join(dokFolder, x)))
    return articleList
  
  def getMagArticleFile(self, dok, artTitle):
    from os.path import expanduser
    home = expanduser("~")
    dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderName(dok))
    contentFile = os.path.join(dokFolder, "obsah.txt")
    contentMag = ""
    if os.path.isfile(contentFile):
      # open content "obsah.txt" file encoded in windows-1250
      with open(contentFile, 'r', encoding='windows-1250') as f:
        contentMag = f.readlines()
    if contentMag:
      artFile = ""
      for line in contentMag:
        if line.startswith("﻿"):
          line = line[1:]
        contentMagList = line.split(".", 1)
        if contentMagList[1].strip() == artTitle:
          artFile = contentMagList[0].strip() + ".txt"
          break
    return artFile

  def getMagazineContent(self, dok):
    articleList = []
    from os.path import expanduser
    home = expanduser("~")
    dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderName(dok))
    contentFile = os.path.join(dokFolder, "obsah.txt")
    contentMag = ""
    if os.path.isfile(contentFile):
      # open content "obsah.txt" file encoded in windows-1250
      with open(contentFile, 'r', encoding='windows-1250') as f:
        contentMag = f.readlines()
    if contentMag:
      for line in contentMag:
        contentMagList = line.split(".", 1)
        articleList.append(contentMagList[1].strip())
    return articleList

  def magArticleRead(self, dok, artFile):
    from os.path import expanduser
    home = expanduser("~")
    dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderName(dok))
    TextReader.dlgTitle = dok.find("nazev").text
    try:
      TextReader.titleEncoding = dok.find("kodovani_nazev_cs").text
    except:
      TextReader.titleEncoding = ""
    TextReader.dispText = os.path.join(dokFolder, artFile)
    TextReader.isReader = True
    TextReader.dokPath = dokFolder
    TextReader.cursorPos = 0
    self.displayInfo()

  def dokumentPlay(self, dok):
    if not vlcInstalled:
      errorMessage = _(u"Titul nelze přehrát, protože nebyl nalezen přehrávač VLC.")
      errorTitle = _(u"Upozornění")
      wx.MessageBox(errorMessage, errorTitle)
      self.passKey = True
      return
    from os.path import expanduser
    home = expanduser("~")
    if self.libraryType == "Mathilda":
      dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderNameMathilda(dok))
      AudioPlayer.dlgTitle = dok.find("title").text
    elif self.libraryType == "KTNMP3":
      dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderNameKtn(dok))
      dtField = dok.find("datafield[@tag='245']")
      sbField = dtField.find("subfield[@code='a']")
      dokNazev = sbField.text
      AudioPlayer.dlgTitle = dokNazev
    elif self.libraryType == "ZORAMP3":
      dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderNameZora(dok))
      AudioPlayer.dlgTitle = dok.find("nazev").text + " " + dok.find("cislo").text + "(" + dok.find("rok").text + ")"
    elif self.libraryType == "JEDENSVET":
      dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderNameJedensvet(dok))
      AudioPlayer.dlgTitle = dok.find("nazev").text + " " + dok.find("cislo").text + "(" + dok.find("rok").text + ")"
    else:
      dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderName(dok))
      dokKategorie = dok.find("kategorie_dokumentu_nazev_cs").text
      if dokKategorie == "casopis_zora":
        dokFolder = dokFolder + " MP3"
        dlgtitle = dok.find("nazev").text + " " + dok.find("poradove_cislo").text + " (" + dok.find("rok_vydani").text
      else:
        dlgtitle = dok.find("nazev").text + ")"
      AudioPlayer.dlgTitle = dlgtitle
    AudioPlayer.source = os.path.join(dokFolder, dok.find("filetoread").text)
    AudioPlayer.dokPath = dokFolder
    AudioPlayer.playingPos = float(dok.find("position").text)
    self.startPlaying()

  def startPlaying(self):
    self.Hide()
    self.Disable()
    try:
      AudioPlayer(self).Show()
    except Exception as e:
      self.Enable()
      raise e

  def dokumentRead(self, dok):
    from os.path import expanduser
    home = expanduser("~")
    if self.libraryType == "ZORATXT":
      dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderNameZora(dok))
      TextReader.dlgTitle = dok.find("nazev").text + " " + dok.find("cislo").text + "(" + dok.find("rok").text + ")"
      TextReader.titleEncoding = "DETECT"
    elif self.libraryType.startswith("KTN"):
      dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderNameKtn(dok))
      dtField = dok.find("datafield[@tag='245']")
      sbField = dtField.find("subfield[@code='a']")
      dokNazev = sbField.text
      TextReader.dlgTitle = dokNazev
      TextReader.titleEncoding = "DETECT"
    elif self.libraryType == "SPVK":
      dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderNameSpvk(dok))
      TextReader.dlgTitle = dok.find("nazev").text
      TextReader.titleEncoding = "DETECT"
    elif self.libraryType == "Mathilda":
      dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderNameMathilda(dok))
      TextReader.dlgTitle = dok.find("title").text
      TextReader.titleEncoding = "UTF-8"
    else:
      dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderName(dok))
      dokKategorie = dok.find("kategorie_dokumentu_nazev_cs").text
      if dokKategorie == "casopis_zora":
        dokFolder = dokFolder + " Text"
      TextReader.dlgTitle = dok.find("nazev").text
      # encoding is left to autodetect
      TextReader.titleEncoding = "DETECT"
      # code bellow obtains encoding from catalog attribute, but this was not reliable, so this code is commented
      # try:
        # TextReader.titleEncoding = dok.find("kodovani_nazev_cs").text
      # except:
        # TextReader.titleEncoding = ""
    TextReader.libType = self.libraryType
    TextReader.dispText = os.path.join(dokFolder, dok.find("filetoread").text)
    TextReader.isReader = True
    TextReader.dokPath = dokFolder
    TextReader.cursorPos = int(dok.find("position").text)
    # in case of HTML document we open it in default browser
    if os.path.splitext(TextReader.dispText)[1] == ".html":
      os.startfile(os.path.join(dokFolder, TextReader.dispText))
    else:
      self.displayInfo()

  def dokumentDelete(self, dok):
    delItem = self.mainList.StringSelection
    if ", přečteno" in delItem or ", přehráno" in delItem:
      delItemList = delItem.rsplit(", ", 1)
      delItem = delItemList[0]
    dlgMessage = "Opravdu chcete odstranit titul " + delItem + " ze své knihovny? Bude trvale odstraněn také z Vašeho počítače."
    dlgTitle = "Upozornění"
    result = gui.messageBox(
      message=dlgMessage,
      caption=dlgTitle,
      style=wx.YES | wx.NO | wx.ICON_WARNING
    )
    if wx.YES == result:
      selItemNumber = self.mainList.GetSelection()
      from os.path import expanduser
      home = expanduser("~")
      if self.libraryType == "JEDENSVET":
        dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderNameJedensvet(dok))
      elif self.libraryType.startswith("ZORA"):
        dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderNameZora(dok))
      elif self.libraryType.startswith("KTN"):
        dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderNameKtn(dok))
      elif self.libraryType == "SPVK":
        dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderNameSpvk(dok))
      elif self.libraryType == "Mathilda":
        dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderNameMathilda(dok))
      else:
        dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderName(dok))
        dokKategorie = dok.find("kategorie_dokumentu_nazev_cs").text
        if dokKategorie == "casopis_zora":
          if delItem.endswith(" MP3"):
            dokFolder = dokFolder + " MP3"
          elif delItem.endswith(" Text"):
            dokFolder = dokFolder + " Text"
      if os.path.exists(dokFolder):
        import shutil
        shutil.rmtree(dokFolder)
        if selItemNumber != -1:
          self.mainList.Delete(selItemNumber)
          if self.mainList.GetCount() == 0:
            self.levelRoot()
          else:
            if selItemNumber == self.mainList.GetCount():
              self.mainList.Selection = selItemNumber-1
            else:
              self.mainList.Selection = selItemNumber
    self.passKey = True
    
  def fixpath(self, s):
    validName = ""
    for i in s:
      if i in "\/:*?<>|\"":
        if i == ":":
          i = " -" 
        elif i == "\"":
          i = "" 
        else:
          i = "-"
      validName = validName + i
    return validName

  def dokumentFolderName(self, dok):
    dokNazev = dok.find("nazev").text
    try:
      dokKategorie = dok.find("kategorie_dokumentu_nazev_cs").text
    except:
      dokKategorie = ""
    try:
      dokAutori = dok.find("autori").text
    except:
      dokAutori = ""
    try:
      dokRokVydani = dok.find("rok_vydani").text
    except:
      dokRokVydani = ""
    try:
      dokPoradoveCislo = dok.find("poradove_cislo").text
    except:
      dokPoradoveCislo = ""
    try:
      dokNazevEpizody = dok.find("nazev_epizody").text
    except:
      dokNazevEpizody = ""
    try:
      dokCisloEpizody = dok.find("cislo_epizody").text
    except:
      dokCisloEpizody = ""
    folderName = dokNazev
    if dokKategorie == "kniha" or dokKategorie == "text" or dokKategorie == "audio_dokument":
      if dokCisloEpizody:
        folderName = folderName + " - " + dokCisloEpizody
      if dokNazevEpizody:
        if not dokCisloEpizody:
          folderName = folderName + " -"
        folderName = folderName + " " + dokNazevEpizody
      if dokAutori:
        folderName = folderName + " - " + dokAutori
    elif dokKategorie == "časopis":
      if dokRokVydani:
        folderName = folderName + "(" + dokRokVydani + ")"
    elif dokKategorie == "casopis_zora":
      if dokPoradoveCislo:
        folderName = folderName + " " + dokPoradoveCislo
      if dokRokVydani:
        folderName = folderName + "(" + dokRokVydani + ")"
      if finalSelection.endswith("táhnout titul jako zvuk"):
        folderName = folderName + " MP3"
      elif finalSelection.endswith("táhnout titul jako text"):
        folderName = folderName + " Text"
    # replace not allowed symbols in foldername
    folderName = self.fixpath(folderName)
    return folderName

  def dokumentFolderNameMathilda(self, dok):
    dokNazev = dok.find("title").text
    dokAutori = self.mathildaGetAuthors(dok)
    folderName = dokNazev
    if dokAutori:
      folderName = folderName + " - " + dokAutori
    # replace not allowed symbols in foldername
    folderName = self.fixpath(folderName)
    return folderName

  def dokumentFolderNameSpvk(self, dok):
    dokNazev = dok.find("nazev").text
    dokAutori = self.spvkGetAuthors(dok)
    folderName = dokNazev
    if dokAutori:
      folderName = folderName + " - " + dokAutori
    folderName = self.fixpath(folderName)
    return folderName

  def dokumentFolderNameKtn(self, dok):
    dtField = dok.find("datafield[@tag='245']")
    sbField = dtField.find("subfield[@code='a']")
    dokNazev = sbField.text
    dokAutori = self.ktnGetAuthors(dok)
    folderName = dokNazev
    if dokAutori:
      folderName = folderName + " - " + dokAutori
    folderName = self.fixpath(folderName)
    return folderName

  def dokumentFolderNameZora(self, dok):
    dokNazev = dok.find("nazev").text
    dokPoradoveCislo = dok.find("cislo").text
    dokRokVydani = dok.find("rok").text
    try:
      dokLibrary = dok.find("librarytype").text
    except:
      dokLibrary = ""
    if dokLibrary:
      if dokLibrary == "ZORAMP3":
        dokLibrary = "MP3"
      else:
        dokLibrary = "Text"
    else:
      if self.browsePath[0] == "Zora MP3":
        dokLibrary = "MP3"
      elif self.browsePath[0] == "Zora Texty":
        dokLibrary = "Text"
    folderName = dokNazev + " " + dokPoradoveCislo + "(" + dokRokVydani + ") " + dokLibrary
    folderName = self.fixpath(folderName)
    return folderName

  def dokumentFolderNameJedensvet(self, dok):
    dokNazev = dok.find("nazev").text
    dokPoradoveCislo = dok.find("cislo").text
    dokRokVydani = dok.find("rok").text
    folderName = dokNazev + " " + dokPoradoveCislo + "(" + dokRokVydani + ")"
    folderName = self.fixpath(folderName)
    return folderName

  def dokumentUnzip(self, dok, fTarget = ""):
    if self.libraryType == "JEDENSVET":
      unzipTitle = self.dokumentFolderNameJedensvet(dok)
    elif self.libraryType.startswith("ZORA"):
      unzipTitle = self.dokumentFolderNameZora(dok)
    elif self.libraryType.startswith("KTN"):
      unzipTitle = self.dokumentFolderNameKtn(dok)
      disksKtn = self.getDisksKtn(dok)
      if disksKtn > 1:
        rtarget = fTarget.rsplit("-CD")[0]
        fTarget = rtarget + "-CD1.zip"
    else:
      dokId = dok.find("id-dokumentu").text
      unzipTitle = self.dokumentFolderName(dok)
    unzipDone = True
    from os.path import expanduser
    home = expanduser("~")
    if fTarget:
      target = fTarget
    else:
      target = os.path.join(home, self.kddLibraryFolder, dokId + ".zip")
    unzipFolder = os.path.join(home, self.kddLibraryFolder, unzipTitle)
    if not os.path.exists(unzipFolder):
      os.mkdir(unzipFolder)
    import zipfile
    try:
      with zipfile.ZipFile(target, "r") as z:
        if self.libraryType.startswith("KTN"):
          for zip_info in z.infolist():
            if zip_info.is_dir():
              continue
            zip_info.filename = os.path.basename(zip_info.filename)
            z.extract(zip_info, unzipFolder)
        else:
          z.extractall(unzipFolder)
    except:
      unzipDone = False
    if unzipDone:
      os.remove(target)
      if self.libraryType.startswith("KTN"):
        if "-CD1" in target:
          for x in os.listdir(unzipFolder):
            os.rename(os.path.join(unzipFolder, x), os.path.join(unzipFolder, "CD1-"+x))
        actDisk = 2
        while actDisk <= disksKtn:
          unzipDone = True
          target = target.replace("-CD"+str(actDisk-1), "-CD"+str(actDisk))
          try:
            with zipfile.ZipFile(target, "r") as z:
              for zip_info in z.infolist():
                if zip_info.is_dir():
                  continue
                zip_info.filename = os.path.basename(zip_info.filename)
                z.extract(zip_info, unzipFolder)
          except:
            unzipDone = False
          if unzipDone:
            os.remove(target)
            if "-CD"+str(actDisk) in target:
              for x in os.listdir(unzipFolder):
                if x.startswith("CD"):
                  continue
                os.rename(os.path.join(unzipFolder, x), os.path.join(unzipFolder, "CD"+str(actDisk)+"-"+x))
          actDisk += 1
      self.dokumentMark(dok, unzipTitle)

  def dokumentAccessKtn(self):
    canBeAccessed = True
    global ktnUserLogin
    if not ktnUserLogin:
      canBeAccessed = False
      errorMessage = _(u"Titul nelze stáhnout, protože nejsou vyplněny přihlašovací údaje do KTN.")
      errorTitle = _(u"Upozornění")
      wx.MessageBox(errorMessage, errorTitle)
      self.passKey = True
    return canBeAccessed

  def dokumentAccessSpvk(self):
    canBeAccessed = True
    global spvkUserLogin
    if not spvkUserLogin:
      canBeAccessed = False
      errorMessage = _(u"Titul nelze stáhnout, protože nejsou vyplněny přihlašovací údaje do SPVK.")
      errorTitle = _(u"Upozornění")
      wx.MessageBox(errorMessage, errorTitle)
      self.passKey = True
    return canBeAccessed

  def dokumentAccessMathilda(self):
    canBeAccessed = True
    global mathildaUserLogin
    if not mathildaUserLogin:
      canBeAccessed = False
      errorMessage = _(u"Titul nelze stáhnout, protože nejsou vyplněny přihlašovací údaje do Mathildy.")
      errorTitle = _(u"Upozornění")
      wx.MessageBox(errorMessage, errorTitle)
      self.passKey = True
    return canBeAccessed

  def dokumentAccess(self, dok):
    canBeAccessed = True
    dokPrice = self.dokumentPrice(dok)
    if dokPrice > 0:
      if not kddLogged:
        canBeAccessed = False
        errorMessage = _(u"Titul nelze stáhnout, protože nejste přihlášeni do KDD")
        errorTitle = _(u"Upozornění")
        wx.MessageBox(errorMessage, errorTitle)
        self.passKey = True
      elif kddLogged and int(kddCredits) - dokPrice < 0:
        canBeAccessed = False
        errorMessage = _(u"Pro stažení tohoto titulu nemáte v KDD dostatečný počet kreditů.")
        errorTitle = _(u"Upozornění")
        wx.MessageBox(errorMessage, errorTitle)
        self.passKey = True
    return canBeAccessed

  def dokumentDownload(self, dok):
    global finalSelection
    dokId = dok.find("id-dokumentu").text
    dokKategorie = dok.find("kategorie_dokumentu_nazev_cs").text
    if dokKategorie == "casopis_zora":
      url = "https://kdd.cz/filezora.php?id=" + dokId
      if finalSelection == "Stáhnout titul jako zvuk":
        url = url + "&au=1"
    elif dokKategorie == "audio_dokument":
      url = "https://kdd.cz/filezvuk.php?id=" + dokId
    else:
      url = "https://kdd.cz/file.php?id=" + dokId
    from os.path import expanduser
    home = expanduser("~")
    destFolder = os.path.join(home, self.kddLibraryFolder)
    if not os.path.exists(destFolder):
      os.mkdir(destFolder)
    target = os.path.join(home, self.kddLibraryFolder, dokId + ".zip")
    ui.message("Stahuji titul, prosím čekejte.")
    result = self.Download_file(url, target)
    return result

  def dokumentDownloadMathilda(self, dok):
    ui.message("Stahuji titul z Mathildy, prosím čekejte.")
    dokLink = dok.find("link").text
    values = {'user': mathildaUserLogin, 'pass': mathildaUserPassword}
    query_string = urllib.parse.urlencode(values)
    data = query_string.encode()
    try:
      response = urllib.request.urlopen(dokLink, data)
      if response.code != 200:
        errorMessage = _(u"Titul nelze stáhnout, možná nejsou zadány správně uživatelské údaje do Mathildy.")
        errorTitle = _(u"Upozornění")
        wx.MessageBox(errorMessage, errorTitle)
        self.passKey = True
        return
    except:
      errorMessage = _(u"Titul nelze stáhnout, možná nejsou zadány správně uživatelské údaje do Mathildy.")
      errorTitle = _(u"Upozornění")
      wx.MessageBox(errorMessage, errorTitle)
      self.passKey = True
      return
    targetFilename = response.info().get_filename()
    from os.path import expanduser
    home = expanduser("~")
    dokDestFolder = self.dokumentFolderNameMathilda(dok)
    destFolder = os.path.join(home, self.kddLibraryFolder)
    if not os.path.exists(destFolder):
      os.mkdir(destFolder)
    destFolder = os.path.join(home, self.kddLibraryFolder, dokDestFolder)
    if not os.path.exists(destFolder):
      os.mkdir(destFolder)
    target = os.path.join(home, self.kddLibraryFolder, dokDestFolder, targetFilename)
    chunkSize = 8192
    toneCounter = 0
    with open(target, "wb") as out_file:
      while True:
        chunk = response.read(chunkSize)
        if not chunk:
          tones.beep(880, 40)
          break
        out_file.write(chunk)
        toneCounter += 1
        if toneCounter == 1000:
          toneCounter = 0
          tones.beep(440, 40)
    # if title is zipped, we have to unzip it first
    if targetFilename.upper().endswith(".ZIP"):
      unzipDone = True
      unzipFolder = os.path.join(home, self.kddLibraryFolder, dokDestFolder)
      import zipfile
      try:
        with zipfile.ZipFile(target, "r") as z:
          z.extractall(unzipFolder)
      except:
        unzipDone = False
      if unzipDone:
        os.remove(target)
      self.dokumentMark(dok, dokDestFolder)
    else:
      self.dokumentMark(dok, dokDestFolder, targetFilename)

  def dokumentDownloadSpvk(self, dok):
    ui.message("Stahuji titul z SPVK, prosím čekejte.")
    dokId = ""
    for item in dok.iter("soubor"):
      try:
        if item.find("typ_souboru").text == "text":
          dokId = item.find("id_souboru").text
          break
      except: pass
    if not dokId:
      for item in dok.iter("soubor"):
        try:
          dokId = item.find("id_souboru").text
          if dokId:
            break
        except: pass
    dokLink = "https://www.spvk.cz/api/v1/file"
    values = {'fileId': dokId, 'email': spvkUserLogin, 'password': spvkUserPassword}
    query_string = urllib.parse.urlencode(values)
    data = query_string.encode()
    request = urllib.request.Request(dokLink, method="POST")
    request.data = data
    request.add_header("X-API-Key", spvkApiKey)
    try:
      response = urllib.request.urlopen(request)
      if response.code != 200:
        errorMessage = _(u"Titul nelze stáhnout, možná nejsou zadány správně uživatelské údaje do SPVK.")
        errorTitle = _(u"Upozornění")
        wx.MessageBox(errorMessage, errorTitle)
        self.passKey = True
        return
    except:
      errorMessage = _(u"Titul nelze stáhnout, možná nejsou zadány správně uživatelské údaje do SPVK.")
      errorTitle = _(u"Upozornění")
      wx.MessageBox(errorMessage, errorTitle)
      self.passKey = True
      return
    content = response.read()
    targetFilename = response.info().get_filename()
    from os.path  import expanduser
    home = expanduser("~")
    dokDestFolder = self.dokumentFolderNameSpvk(dok)
    destFolder = os.path.join(home, self.kddLibraryFolder)
    if not os.path.exists(destFolder):
      os.mkdir(destFolder)
    destFolder = os.path.join(home, self.kddLibraryFolder, dokDestFolder)
    if not os.path.exists(destFolder):
      os.mkdir(destFolder)
    target = os.path.join(home, self.kddLibraryFolder, dokDestFolder, targetFilename)
    with open(target, "wb") as out_file:
      out_file.write(content)
    self.dokumentMark(dok, dokDestFolder, targetFilename)

  def getDisksKtn(self, dok):
    try:
      dtField = dok.find("datafield[@tag='300']")
      sbField = dtField.find("subfield[@code='a']")
      disks = sbField.text.split()[0]
      result = int(disks)
    except:
      result = 0
    return result

  def dokumentDownloadKtn(self, dok):
    if self.libraryType == "KTNMP3" and self.getDisksKtn(dok) > 1:
      dlgMessage = "Zvolený titul se skládá z většího počtu souborů (celkem " + str(self.getDisksKtn(dok)) + "). Stahování může trvat delší dobu. Chcete nyní zahájit stahování?"
      dlgTitle = "Upozornění"
      result = gui.messageBox(
        message=dlgMessage,
        caption=dlgTitle,
        style=wx.YES | wx.NO | wx.ICON_WARNING
      )
      self.passKey = True
      if wx.NO == result:
        return
    ui.message("Stahuji titul z KTN, prosím čekejte.")
    dtField = dok.find("datafield[@tag='856']")
    sbField = dtField.find("subfield[@code='u']")
    dokLink = sbField.text
    targetFilename = dokLink.rsplit("/",1)[1]
    from os.path  import expanduser
    home = expanduser("~")
    destFolder = os.path.join(home, self.kddLibraryFolder)
    if not os.path.exists(destFolder):
      os.mkdir(destFolder)
    target = os.path.join(destFolder, targetFilename)
    if self.libraryType == "KTNMP3":
      self.downloadUrl = dokLink
      self.downloadDok = dok
      self.downloadTarget = target
      if self.getDisksKtn(dok) > 1:
        for dtField in dok.findall("datafield[@tag='856']"):
          sbField = dtField.find("subfield[@code='u']")
          self.downloadList.append(sbField.text)
        self.downloadList.remove(dokLink)
      else:
        self.downloadList.clear()
      self.startDownload()
    else:
      values = {'login': ktnUserLogin, 'password': ktnUserPassword, '__log_in':'1'}
      query_string = urllib.parse.urlencode(values)
      data = query_string.encode()
      request = urllib.request.Request(dokLink, method="POST")
      request.data = data
      try:
        response = urllib.request.urlopen(request)
      except:
        errorMessage = _(u"Titul nelze stáhnout, možná nejsou zadány správně uživatelské údaje do KTN.")
        errorTitle = _(u"Upozornění")
        wx.MessageBox(errorMessage, errorTitle)
        self.passKey = True
        return
      content = response.read()
      with open(target, "wb") as out_file:
        out_file.write(content)
      self.dokumentUnzip(dok, target)

  def dokumentDownloadZoraMp3(self, dok):
    # from selected magazine we reconstruct original magazine name on webpage
    magyear = dok.find("rok").text
    magyear = magyear[2:]
    magcislo = dok.find("cislo").text
    magtitle = dok.find("nazev").text
    findmag = magtitle + " - archiv celého čísla " + magcislo + "/" + magyear
    findmag = findmag.lower()
    # find magazine on the html page
    htmlpage = self.zoraMp3PageHtml.lower()
    indexmag = htmlpage.find(findmag)
    # find the download address of selected magazine
    indexaddr = htmlpage.rfind("http", 0, indexmag)
    indexendaddr = htmlpage.find('">', indexaddr)
    dokLink = self.zoraMp3PageHtml[indexaddr:indexendaddr]
    # downloading selected magazine
    ui.message("Stahuji titul ze Zory, prosím čekejte.")
    targetFilename = dokLink.rsplit("/",1)[1]
    from os.path import expanduser
    home = expanduser("~")
    destFolder = os.path.join(home, self.kddLibraryFolder)
    if not os.path.exists(destFolder):
      os.mkdir(destFolder)
    target = os.path.join(destFolder, targetFilename)
    self.downloadUrl = dokLink
    self.downloadDok = dok
    self.downloadTarget = target
    self.startDownload()

  def dokumentDownloadZoraText(self, dok):
    candidatesList = GetSubstrings(self.zoraMagPageHtml, "<h3>", ".zip'>")
    if len(candidatesList) == 0:
      return
    downloadAddress = ""
    for i in candidatesList:
      if dok.find("nazev").text.lower() in i.lower() and dok.find("cislo").text.lower() in i.lower():
        addrList = i.rsplit("href='", 1)
        downloadAddress = addrList[1]
        break
    url = "https://archiv.sons.cz/zora/text/" + downloadAddress + ".zip"
    from os.path import expanduser
    home = expanduser("~")
    destFolder = os.path.join(home, self.kddLibraryFolder)
    if not os.path.exists(destFolder):
      os.mkdir(destFolder)
    target = os.path.join(home, self.kddLibraryFolder, url.rsplit("/", 1)[1])
    ui.message("Stahuji titul, prosím čekejte.")
    if self.Download_file(url, target):
      self.dokumentUnzip(dok, target)

  def dokumentDownloadJedensvet(self, dok):
    # from selected magazine we reconstruct selected magazine name
    magyear = dok.find("rok").text
    magcislo = dok.find("cislo").text
    magtitle = dok.find("nazev").text
    magname = magyear + ", " + magtitle + " " + magcislo
    dokLink = self.jedenSvetDict[magname]
    # downloading selected magazine
    ui.message("Stahuji časopis Jeden svět, prosím čekejte.")
    targetFilename = dokLink.rsplit("/",1)[1]
    from os.path import expanduser
    home = expanduser("~")
    destFolder = os.path.join(home, self.kddLibraryFolder)
    if not os.path.exists(destFolder):
      os.mkdir(destFolder)
    target = os.path.join(destFolder, targetFilename)
    self.downloadUrl = dokLink
    self.downloadDok = dok
    self.downloadTarget = target
    self.startDownload()


  # downloading with progressbar (used for large audio data)
  # taken from NVDA source, slightly edited

  def startDownload(self):
    self._shouldCancel = False
    self._guiExecTimer = gui.NonReEntrantTimer(self._guiExecNotify)
    self._progressDialog = wx.ProgressDialog(_("Stahuji titul"),
      _("Připojuji"),
      style=wx.PD_CAN_ABORT | wx.PD_ELAPSED_TIME | wx.PD_REMAINING_TIME | wx.PD_AUTO_HIDE,
      )
    self._progressDialog.CentreOnScreen()
    self._progressDialog.Raise()
    t = threading.Thread(
      name=f"{self.__class__.__module__}.{self.startDownload.__qualname__}",
      target=self._bg
    )
    t.daemon = True
    t.start()

  def _guiExec(self, func, *args):
    self._guiExecFunc = func
    self._guiExecArgs = args
    if not self._guiExecTimer.IsRunning():
      wx.CallAfter(self._guiExecTimer.Start, 50, True)

  def _guiExecNotify(self):
    self._guiExecFunc(*self._guiExecArgs)

  def _bg(self):
    success=False
    try:
      self._download()
    except:
      log.error("Chyba při stahování titulu")
    else:
      if not self._shouldCancel:
        success=True
    if not success:
      self._guiExec(self._error)
      return
    self._guiExec(self._downloadSuccess)

  def _download(self):
    if self.libraryType == "KTNMP3":
      values = {'login': ktnUserLogin, 'password': ktnUserPassword, '__log_in':'1'}
      query_string = urllib.parse.urlencode(values)
      data = query_string.encode()
      request = urllib.request.Request(self.downloadUrl, method="POST")
      request.data = data
    else:
      request = urllib.request.Request(self.downloadUrl)
    remote = urllib.request.urlopen(request)
    if remote.code != 200:
      raise RuntimeError("Stahování selhalo s chybou %d" % remote.code)
    size = int(remote.headers["content-length"])
    with open(self.downloadTarget, "wb") as local:
      self._guiExec(self._downloadReport, 0, size)
      read = 0
      chunk=8192
      while True:
        if self._shouldCancel:
          return
        if size -read <chunk:
          chunk =size -read
        block = remote.read(chunk)
        if not block:
          break
        read += len(block)
        if self._shouldCancel:
          return
        local.write(block)
        self._guiExec(self._downloadReport, read, size)
      if read < size:
        raise RuntimeError("Stahovaný obsah je příliš krátký")
    self._guiExec(self._downloadReport, read, size)

  def _downloadReport(self, read, size):
    if self._shouldCancel:
      return
    percent = int(float(read) / size * 100)
    cont, skip = self._progressDialog.Update(percent, _("Stahuji"))
    if not cont:
      self._shouldCancel = True
      self._stopped()

  def _stopped(self):
    self._guiExecTimer = None
    self._guiExecFunc = None
    self._guiExecArgs = None
    self._progressDialog.Hide()
    self._progressDialog.Destroy()
    self._progressDialog = None

  def _error(self):
    #wx.CallAfter(self._progressDialog.done)
    self._stopped()
    if self.libraryType == "KTNMP3":
      errorMessage = _(u"Titul nelze stáhnout, možná nejsou zadány správně uživatelské údaje do KTN.")
    else:
      errorMessage = _(u"Titul nelze stáhnout.")
    errorTitle = _(u"Upozornění")
    wx.MessageBox(errorMessage, errorTitle)
    self.passKey = True
    # gui.messageBox(
    #   _("Chyba při stahování titulu."),
    #   _("Chyba"),
    #   wx.OK | wx.ICON_ERROR)

  def _downloadSuccess(self):
    self._stopped()
    if self.libraryType == "ZORAMP3" or self.libraryType == "JEDENSVET":
      self.dokumentUnzip(self.downloadDok, self.downloadTarget)
    elif self.libraryType == "KTNMP3":
      if len(self.downloadList) > 0:
        dokLinkDisk = self.downloadList[0]
        self.downloadList.pop(0)
        targetFilenameDisk = dokLinkDisk.rsplit("/",1)[1]
        from os.path  import expanduser
        home = expanduser("~")
        destFolderDisk = os.path.join(home, self.kddLibraryFolder)
        if not os.path.exists(destFolderDisk):
          os.mkdir(destFolderDisk)
        targetDisk = os.path.join(destFolderDisk, targetFilenameDisk)
        self.downloadUrl = dokLinkDisk
        self.downloadTarget = targetDisk
        self.startDownload()
      else:
        self.dokumentUnzip(self.downloadDok, self.downloadTarget)


    # end of download with progressbar


  def displayWhatToDo(self):
    WhatToDoDialog.titleDlg = "Co dělat?"
    WhatToDoDialog.sposition = 0
    self.Disable()
    try:
      WhatToDoDialog(self).ShowModal()
    except Exception as e:
      self.Enable()
      raise e

  def whereAmI(self):
    wherePath = ""
    if self.browsePath[0] == "":
      # blank path means main menu
      wherePath = "Hlavní nabídka."
    else:
      pos = 0
      while self.browsePath[pos] != "":
        wherePath = wherePath + self.browsePath[pos]
        if self.browsePath[pos+1] != "":
          wherePath = wherePath + ", "
        pos = pos+1
      if self.addPath[0] != "":
        wherePath = wherePath + ", "
        pos = 0
        while self.addPath[pos] != "":
          wherePath = wherePath + self.addPath[pos]
          if self.addPath[pos+1] != "":
            wherePath = wherePath + ", "
          pos = pos+1
      wherePath = wherePath + "."
    return wherePath

  def removefromBookmarks(self, dok):
    selItemNumber = self.mainList.GetSelection()
    bFile = os.path.join(libraryDataFolder, self.bookmarksFileName)
    bTree = ET.parse(bFile)
    bDokument = bTree.getroot()
    if self.libraryType.startswith("ZORA"):
      dokId = dok.find("nazev").text + dok.find("cislo").text + dok.find("rok").text + self.libraryType
      for curDok in bDokument.iter("zora"):
        if dokId in curDok.find("nazev").text + curDok.find("cislo").text + curDok.find("rok").text + curDok.find("librarytype").text:
          bDokument.remove(curDok)
          break
    elif self.libraryType.startswith("KTN"):
      ctrlField = dok.find("controlfield[@tag='001']")
      dokId = ctrlField.text
      for curDok in bDokument.iter("record"):
        ctrlField = curDok.find("controlfield[@tag='001']")
        curDokId = ctrlField.text
        if dokId == curDokId:
          bDokument.remove(curDok)
          break
    elif self.libraryType == "SPVK":
      dokId = dok.find("id-dokumentu").text
      for curDok in bDokument.iter("polozka"):
        if dokId in curDok.find("id-dokumentu").text:
          bDokument.remove(curDok)
          break
    elif self.libraryType == "Mathilda":
      bookmarkDeleted = False
      dokId = dok.find("id").text
      for curDok in bDokument.iter("book"):
        if dokId in curDok.find("id").text:
          bDokument.remove(curDok)
          bookmarkDeleted = True
          break
      if not bookmarkDeleted:
        for curDok in bDokument.iter("film"):
          if dokId in curDok.find("id").text:
            bDokument.remove(curDok)
            bookmarkDeleted = True
            break
    else:
      dokId = dok.find("id-dokumentu").text
      for curDok in bDokument.iter("dokument"):
        if dokId in curDok.find("id-dokumentu").text:
          bDokument.remove(curDok)
          break
    if selItemNumber != -1:
      self.mainList.Delete(selItemNumber)
      if self.mainList.GetCount() == 0:
        # find index of item Záložky
        bookmarkIndexList = [n for n, l in enumerate(self.categories) if l.startswith('Záložky')]
        # if the item is present
        if bookmarkIndexList:
          del self.categories[bookmarkIndexList[0]]
        self.levelRoot()
        os.remove(bFile)
      else:
        self.mainList.Selection = 0
    # save element to XML file
    if os.path.isfile(bFile):
      with open(bFile, 'wb') as f:
        f.write(ET.tostring(bDokument, encoding='utf-8', method='xml'))

  def addtoBookmarks(self, dok):
    bFile = os.path.join(libraryDataFolder, self.bookmarksFileName)
    if os.path.isfile(bFile):
      bTree = ET.parse(bFile)
      bDokument = bTree.getroot()
    else:
      bDokument = ET.Element('bookmarks')
    if self.libraryType.startswith("ZORA"):
      dokId = dok.find("nazev").text + dok.find("cislo").text + dok.find("rok").text + self.libraryType
    elif self.libraryType.startswith("KTN"):
      ctrlField = dok.find("controlfield[@tag='001']")
      dokId = ctrlField.text
    elif self.libraryType == "SPVK":
      dokId = dok.find("id-dokumentu").text
    elif self.libraryType == "Mathilda":
      dokId = dok.find("id").text
    else:
      dokId = dok.find("id-dokumentu").text
    alreadyAdded = False
    if self.libraryType == "ZORAMP3":
      for curDok in bDokument.iter("zora"):
        try:
          if dokId in curDok.find("nazev").text + curDok.find("cislo").text + curDok.find("rok").text + curDok.find("librarytype").text:
            dlgMessage = "Tento titul se již v záložkách nachází."
            dlgTitle = "Upozornění"
            wx.MessageBox(dlgMessage, dlgTitle)
            self.passKey = True
            alreadyAdded = True
            break
        except: pass
    elif self.libraryType.startswith("KTN"):
      for curDok in bDokument.iter("record"):
        ctrlField = curDok.find("controlfield[@tag='001']")
        curDokId = ctrlField.text
        if dokId == curDokId:
          dlgMessage = "Tento titul se již v záložkách nachází."
          dlgTitle = "Upozornění"
          wx.MessageBox(dlgMessage, dlgTitle)
          self.passKey = True
          alreadyAdded = True
          break
    elif self.libraryType == "SPVK":
      for curDok in bDokument.iter("polozka"):
        try:
          if dokId in curDok.find("id-dokumentu").text:
            dlgMessage = "Tento titul se již v záložkách nachází."
            dlgTitle = "Upozornění"
            wx.MessageBox(dlgMessage, dlgTitle)
            self.passKey = True
            alreadyAdded = True
            break
        except: pass
    elif self.libraryType == "Mathilda":
      for curDok in bDokument.iter("book"):
        try:
          if dokId in curDok.find("id").text:
            dlgMessage = "Tento titul se již v záložkách nachází."
            dlgTitle = "Upozornění"
            wx.MessageBox(dlgMessage, dlgTitle)
            self.passKey = True
            alreadyAdded = True
            break
        except: pass
      for curDok in bDokument.iter("film"):
        try:
          if dokId in curDok.find("id").text:
            dlgMessage = "Tento titul se již v záložkách nachází."
            dlgTitle = "Upozornění"
            wx.MessageBox(dlgMessage, dlgTitle)
            self.passKey = True
            alreadyAdded = True
            break
        except: pass
    else:
      for curDok in bDokument.iter("dokument"):
        try:
          if dokId in curDok.find("id-dokumentu").text:
            dlgMessage = "Tento titul se již v záložkách nachází."
            dlgTitle = "Upozornění"
            wx.MessageBox(dlgMessage, dlgTitle)
            self.passKey = True
            alreadyAdded = True
            break
        except: pass
    if alreadyAdded:
      return
    # add library type
    if self.libraryType:
      ltype = ET.Element("librarytype")
      ltype.text = self.libraryType
      dok.append(ltype)
    bDokument.append(dok)
    # save element to XML file
    with open(bFile, 'wb') as f:
      f.write(ET.tostring(bDokument, encoding='utf-8', method='xml'))
    # find index of item Záložky
    bookmarkIndexList = [n for n, l in enumerate(self.categories) if l.startswith('Záložky')]
    # if the item is not present
    if not bookmarkIndexList:
      # insert item Záložky after item Hledání
      self.categories.insert(self.categories.index("Hledání")+1, "Záložky")

  def findCurrentTitle(self):
    if self.browsePath[0] == "Jeden svět (Mathilda)":
      dok = self.createTreeDokJedensvet(self.mainList.StringSelection)
      return dok
    if self.browsePath[0] == "Zora MP3":
      dok = self.createTreeDokZoraMp3(self.mainList.StringSelection)
      return dok
    if self.browsePath[0] == "Zora Texty":
      dok = self.createTreeDokZoraTexty(self.mainList.StringSelection)
      return dok
    selAutori = ""
    selRokVydani = ""
    selPoradoveCislo = ""
    selCisloEpizody = ""
    if self.browsePath[0] == "MP3 KTN" or (self.browsePath[0].startswith("Novinky KTN") and self.browsePath[1].startswith("MP3")):
      selItem = self.mainList.StringSelection
      searchedTitleList = selItem.rsplit(" - ", 1)
      searchedTitle = searchedTitleList[0].strip()
      if len(searchedTitleList) > 1:
        selAutori = searchedTitleList[1].strip()
      else:
        selAutori = ""
      for dok in self.resultsTree.iter("record"):
        ctrlField = dok.find("controlfield[@tag='001']")
        if ctrlField.text[:2] == "MP":
          titleField = dok.find("datafield[@tag='245']")
          sbField = titleField.find("subfield[@code='a']")
          dokNazev = sbField.text
          dokAutor = self.ktnGetAuthors(dok)
          if searchedTitle == dokNazev and selAutori == dokAutor:
            self.libraryType = "KTNMP3"
            break
      return dok
    if self.browsePath[0] == "Texty KTN" or (self.browsePath[0].startswith("Novinky KTN") and self.browsePath[1].startswith("Texty")):
      selItem = self.mainList.StringSelection
      searchedTitleList = selItem.rsplit(" - ", 1)
      searchedTitle = searchedTitleList[0].strip()
      if len(searchedTitleList) > 1:
        selAutori = searchedTitleList[1].strip()
      else:
        selAutori = ""
      for dok in self.resultsTree.iter("record"):
        ctrlField = dok.find("controlfield[@tag='001']")
        if ctrlField.text[0] == "D" and ctrlField.text[1].isnumeric():
          titleField = dok.find("datafield[@tag='245']")
          sbField = titleField.find("subfield[@code='a']")
          dokNazev = sbField.text
          dokAutor = self.ktnGetAuthors(dok)
          if searchedTitle == dokNazev and selAutori == dokAutor:
            self.libraryType = "KTNTXT"
            break
      return dok
    if self.browsePath[0] == "Knihy SPVK":
      selItem = self.mainList.StringSelection
      searchedTitleList = selItem.rsplit(" - ", 1)
      searchedTitle = searchedTitleList[0].strip()
      if len(searchedTitleList) > 1:
        selAutori = searchedTitleList[1].strip()
      else:
        selAutori = ""
      for dok in self.resultsTree.iter("polozka"):
        try:
          if searchedTitle in dok.find("nazev").text:
            if not selAutori or selAutori == self.spvkGetAuthors(dok):
              break
              self.libraryType = "SPVK"
        except: pass
      return dok
    if self.browsePath[0] == "Knihy Mathilda":
      selItem = self.mainList.StringSelection
      searchedTitleList = selItem.rsplit(" - ", 1)
      searchedTitle = searchedTitleList[0].strip()
      if len(searchedTitleList) > 1:
        selAutori = searchedTitleList[1].strip()
      else:
        selAutori = ""
      for dok in self.resultsTree.iter("book"):
        try:
          if searchedTitle in dok.find("title").text:
            if not selAutori or selAutori == self.mathildaGetAuthors(dok):
              break
              self.libraryType = "Mathilda"
        except: pass
      return dok
    if self.browsePath[0] == "Filmy Mathilda":
      selItem = self.mainList.StringSelection
      searchedTitleList = selItem.rsplit(" - ", 1)
      searchedTitle = searchedTitleList[0].strip()
      if len(searchedTitleList) > 1:
        selAutori = searchedTitleList[1].strip()
      else:
        selAutori = ""
      for dok in self.resultsTree.iter("film"):
        try:
          if searchedTitle in dok.find("title").text:
            if not selAutori or selAutori == self.mathildaGetAuthors(dok):
              break
              self.libraryType = "Mathilda"
        except: pass
      return dok
    if self.browsePath[0] == "Záložky":
      selItem = self.mainList.StringSelection
      libType = ""
      bFile = os.path.join(libraryDataFolder, self.bookmarksFileName)
      if os.path.isfile(bFile):
        bTree = ET.parse(bFile)
        bDokument = bTree.getroot()
        foundinBookmarks = False
        for curDok in bDokument.iter("dokument"):
          try:
            if selItem in curDok.find("nazev").text:
              foundinBookmarks = True
              libType = curDok.find("librarytype").text
              break
          except: pass
        if not foundinBookmarks:
          for curDok in bDokument.iter("book"):
            try:
              if selItem in curDok.find("title").text:
                foundinBookmarks = True
                libType = curDok.find("librarytype").text
                break
            except: pass
        if not foundinBookmarks:
          for curDok in bDokument.iter("film"):
            try:
              if selItem in curDok.find("title").text:
                foundinBookmarks = True
                libType = curDok.find("librarytype").text
                break
            except: pass
        if not foundinBookmarks:
          for curDok in bDokument.iter("polozka"):
            try:
              if selItem in curDok.find("nazev").text:
                foundinBookmarks = True
                libType = curDok.find("librarytype").text
                break
            except: pass
        if not foundinBookmarks:
          for curDok in bDokument.iter("record"):
            try:
              titleField = curDok.find("datafield[@tag='245']")
              sbField = titleField.find("subfield[@code='a']")
              dokNazev = sbField.text
              if dokNazev == selItem:
                foundinBookmarks = True
                libType = curDok.find("librarytype").text
                break
            except: pass
        if not foundinBookmarks:
          for curDok in bDokument.iter("zora"):
            try:
              if selItem in curDok.find("nazev").text:
                foundinBookmarks = True
                libType = curDok.find("librarytype").text
                break
            except: pass
      self.libraryType = libType
      if self.libraryType == "ZORAMP3":
        for dok in self.resultsTree.iter("zora"):
          try:
            if selItem in dok.find("nazev").text:
              break
          except: pass
      elif self.libraryType.startswith("KTN"):
        for dok in self.resultsTree.iter("record"):
          titleField = dok.find("datafield[@tag='245']")
          sbField = titleField.find("subfield[@code='a']")
          dokNazev = sbField.text
          if selItem == dokNazev:
            break
      elif self.libraryType == "SPVK":
        for dok in self.resultsTree.iter("polozka"):
          try:
            if selItem in dok.find("nazev").text:
              break
          except: pass
      elif self.libraryType == "Mathilda":
        for dok in self.resultsTree.iter("book"):
          try:
            if selItem in dok.find("title").text:
              break
          except: pass
        for dok in self.resultsTree.iter("film"):
          try:
            if selItem in dok.find("title").text:
              break
          except: pass
      else:
        for dok in self.resultsTree.iter("dokument"):
          try:
            if selItem in dok.find("nazev").text:
              break
          except: pass
    if self.browsePath[0] == "Moje knihovna":
      selItem = self.mainList.StringSelection
      if ", přečteno" in selItem or ", přehráno" in selItem:
        selItemList = selItem.rsplit(", ", 1)
        selItem = selItemList[0]
      from os.path import expanduser
      home = expanduser("~")
      bookmarkFile = os.path.join(home, self.kddLibraryFolder, selItem, self.okoMarkFilename)
      dok = ET.parse(bookmarkFile)
      try:
        self.libraryType = dok.find("librarytype").text
      except:
        self.libraryType = ""
    if self.browsePath[0] == "Knihy" or self.browsePath[0] == "Časopisy" or self.browsePath[0] == "Texty" or self.browsePath[0] == "Zora" or self.browsePath[0] == "Zvukové dokumenty" or self.browsePath[0] == "Hledání" or self.browsePath[0].startswith("Nové tituly"):
      selItem = self.mainList.StringSelection
      if self.browsePath[0] == "Hledání":
        searchedselCatList = selItem.rsplit(";", 2)
        self.selCat = searchedselCatList[1].strip()
        if searchedselCatList[2].strip() == "KTN":
          if searchedselCatList[1].strip() == "mp3":
            self.libraryType = "KTNMP3"
          elif searchedselCatList[1].strip() == "text":
            self.libraryType = "KTNTXT"
        else:
          self.libraryType = searchedselCatList[2].strip()
        selItem = searchedselCatList[0].strip()
      else:
        if self.selCat == "kniha" or self.selCat == "text" or self.selCat == "audio_dokument":
          self.libraryType = "KDD"
        elif self.selCat == "film":
          self.libraryType = "Mathilda"
      if self.selCat == "kniha" or self.selCat == "text" or self.selCat == "audio_dokument" or self.selCat == "film" or self.selCat == "mp3":
        searchedTitleList = selItem.rsplit(" - ", 1)
        searchedTitle = searchedTitleList[0].strip()
        if len(searchedTitleList) > 1:
          selAutori = searchedTitleList[1].strip()
        else:
          selAutori = ""
        if self.browsePath[1] == "Seriály s AD":
          searchedTitleList = searchedTitle.split(" ", 1)
          selCisloEpizody = searchedTitleList[0]
          searchedTitle = searchedTitleList[1]
      elif self.selCat == "časopis" or self.selCat == "casopis_zora":
        self.libraryType = "KDD"
        if self.browsePath[0].startswith("Nové tituly"):
          searchedTitleList = selItem.rsplit("(")
          selRokVydani = searchedTitleList[1][:-1]
          searchedTitle = searchedTitleList[0].strip()
          if self.selCat == "casopis_zora":
            searchedTitleList = searchedTitle.rsplit(" ", 1)
            searchedTitle = searchedTitleList[0].strip()
            selPoradoveCislo = searchedTitleList[1].strip()
        else:
          searchedTitleList = selItem.split(",")
          searchedTitle = searchedTitleList[1].strip()
          selRokVydani = searchedTitleList[0].strip()
          if self.selCat == "casopis_zora":
            searchedTitleList = searchedTitle.rsplit(" ", 1)
            searchedTitle = searchedTitleList[0].strip()
            selPoradoveCislo = searchedTitleList[1].strip()
      if self.libraryType == "KDD":
        for dok in self.resultsTree.iter("dokument"):
          try:
            if self.selCat in dok.find("kategorie_dokumentu_nazev_cs").text:
              if (selCisloEpizody and searchedTitle in dok.find("nazev_epizody").text) or (searchedTitle in dok.find("nazev").text):
                if not selAutori or selAutori in dok.find("autori").text:
                  if not selRokVydani or selRokVydani in dok.find("rok_vydani").text:
                    if not selPoradoveCislo or selPoradoveCislo in dok.find("poradove_cislo").text:
                      break
          except: pass
      elif self.libraryType == "Mathilda":
        if self.selCat == "kniha":
          for dok in self.resultsTree.iter("book"):
            try:
              if searchedTitle in dok.find("title").text:
                if not selAutori or selAutori == self.mathildaGetAuthors(dok):
                  break
            except: pass
        elif self.selCat == "film":
          for dok in self.resultsTree.iter("film"):
            try:
              if searchedTitle in dok.find("title").text:
                if not selAutori or selAutori == self.mathildaGetAuthors(dok):
                  break
            except: pass
      elif self.libraryType == "SPVK":
        for dok in self.resultsTree.iter("polozka"):
          try:
            if searchedTitle in dok.find("nazev").text:
              if not selAutori or selAutori == self.spvkGetAuthors(dok):
                break
          except: pass
      elif self.libraryType == "KTNMP3":
        for dok in self.resultsTree.iter("record"):
          ctrlField = dok.find("controlfield[@tag='001']")
          if ctrlField.text[:2] == "MP":
            dtField = dok.find("datafield[@tag='245']")
            sbField = dtField.find("subfield[@code='a']")
            dokNazev = sbField.text
            if searchedTitle == dokNazev:
              if not selAutori or selAutori == self.ktnGetAuthors(dok):
                break
      elif self.libraryType == "KTNTXT":
        for dok in self.resultsTree.iter("record"):
          ctrlField = dok.find("controlfield[@tag='001']")
          if ctrlField.text[0] == "D" and ctrlField.text[1].isnumeric():
            dtField = dok.find("datafield[@tag='245']")
            sbField = dtField.find("subfield[@code='a']")
            dokNazev = sbField.text
            if searchedTitle == dokNazev:
              if not selAutori or selAutori == self.ktnGetAuthors(dok):
                break
    return dok

  def titleInfo(self, dok):
    TextReader.dlgTitle = "Informace o titulu"
    TextReader.dispText = self.dokumentInfo(dok)
    TextReader.isReader = False
    self.displayInfo()

  def kddSearch(self):
    res = KddSearchDialog(self).ShowModal()
    self.passKey = True
    if res:
      if not self.allCatalogsLoaded:
        dlgMessage = "Některé katalogy ještě nejsou načteny, prosím zkuste vyhledávání za chvíli."
        dlgTitle = "Upozornění"
        wx.MessageBox(dlgMessage, dlgTitle)
        self.passKey = True
        return
      self.initbrowsePath()
      self.browsePath[0] = "Hledání"
      self.currentLevel = 3
      ui.message("Hledám.")
      self.onmainListChoice(self)

  def sayDokumentAnotace(self, dok):
    dokAnotace = ""
    if not self.libraryType or self.libraryType == "KDD":
      dokPredmetoveSkupiny = dok.find("predmetove_skupiny").text
      dokAnotace = dok.find("anotace").text
      if dokAnotace == None:
        dokAnotace = ""
      if self.browsePath[0].startswith("Nové tituly") and self.browsePath[1].startswith("Knihy") and dokPredmetoveSkupiny:
        dokAnotace = dokPredmetoveSkupiny + ": " + dokAnotace
    elif self.libraryType == "Mathilda":
      dokAnotace = dok.find("description").text
    elif self.libraryType == "SPVK":
      dokAnotace = dok.find("anotace").text
    elif self.libraryType.startswith("KTN"):
      dtField = dok.find("datafield[@tag='520']")
      if dtField is not None:
        sbField = dtField.find("subfield[@code='a']")
        dokAnotace = sbField.text
    if not dokAnotace:
      ui.message("Žádná anotace není k dispozici.")
    else:
      ui.message(dokAnotace)

  def sortingMagazines(self, mag):
    try:
      year = int(mag.split(",")[0])
    except:
      year = 0
    numstr = mag.rsplit(" ", 1)[1]
    if "/" in numstr:
      numstr = numstr.split("/")[0]
    if "-" in numstr:
      numstr = numstr.split("-")[0]
    try:
      num = int(numstr)
    except:
      num = 0
    return (-year, -num)

  def fillmainList(self, itemsList):
    if len(itemsList) == 0:
      itemsList.append("Žádné položky.")
    itemsList = list(dict.fromkeys(itemsList))
    if self.currentLevel in range(2, 5) or (self.currentLevel == 1 and not self.browsePath[0].startswith("Nové tituly")):
      # KDD, Zora and Jeden svět magazines are sorted in descend order
      if self.browsePath[0] == "Zora" or self.browsePath[0].startswith("Jeden svět") or self.browsePath[0] == "Časopisy":
        if (self.currentLevel == 4) or (self.currentLevel == 2 and self.browsePath[0] == "Zora Texty"):
          if self.browsePath[0] == "Časopisy" or self.browsePath[0] == "Zora":
            # sort KDD magazines and Zora using special function due to one digit numbers
            itemsList.sort(key=self.sortingMagazines)
          else:
            itemsList.sort(reverse=True, key=locale.strxfrm)
        else:
          # sort list normally
          itemsList.sort(key=locale.strxfrm)
          # put Archiv starších časopisů to the end of list
          try:
            if self.currentLevel == 1:
              itemsList.append(itemsList.pop(itemsList.index("Archiv starších časopisů")))
          except: pass
      else:
        # sort list normally
        itemsList.sort(key=locale.strxfrm)
    # insert item "Vše" on the first position in the list
    if self.currentLevel in range(2, 4) and (self.browsePath[0] == "Knihy" or self.browsePath[0] == "Texty" or self.browsePath[0] == "Zvukové dokumenty" or self.browsePath[0] == "Filmy Mathilda" or self.browsePath[0] == "MP3 KTN" or self.browsePath[0] == "Texty KTN"):
      itemsList.insert(0, "Vše")
    self.mainList.Clear()
    self.mainList.InsertItems(itemsList, 0)
    self.mainList.Selection = 0

  def setmainListPosition(self, level):
    if self.addLevel >= 0:
      tmpIndex = self.mainList.FindString(self.addPath[level])  
    else:
      if level == 4:
        liststr = self.mainList.GetStrings()
        for n, i in enumerate(liststr):
          if i.startswith(self.browsePath[4]):
            tmpIndex = n
            break
      else:
        tmpIndex = self.mainList.FindString(self.browsePath[level])
    if tmpIndex < 0:
      tmpIndex = 0
    self.mainList.Selection = tmpIndex

  def onmainListChoice(self, evt):
    global finalSelection
    if self.mainList.StringSelection == "Žádné položky.":
      return
    flist = []
    if self.newsDisplayed:
      listBoundary = 14
    else:
      listBoundary = 13
    if self.currentLevel in range(0, 4) and self.addLevel < 0:
      self.browsePath[self.currentLevel] = self.mainList.StringSelection
    if self.currentLevel == 4 and self.addLevel < 0 and self.browsePath[0] == "Moje knihovna":
      selstr = self.mainList.StringSelection
      if selstr.find(", přečteno") != -1 or selstr.find(", přehráno") != -1:
        selstr = selstr.rsplit(", ")[0]
      self.browsePath[4] = selstr
    if self.currentLevel in range(0, 4) and self.addLevel >= 0:
      self.addPath[self.addLevel] = self.mainList.StringSelection
    if self.currentLevel in range(1,4) or (self.currentLevel == 1 and self.browsePath[0] != "Moje knihovna") or (self.currentLevel == 0 and self.mainList.GetSelection() in range(1, listBoundary)):
      ui.message("Načítám.")
    if self.browsePath[0].startswith("Nové tituly"):
      selCatPos = 1
    else:
      selCatPos = 0
    if self.browsePath[selCatPos].startswith("Knihy"):
      self.selCat = "kniha"
    elif self.browsePath[selCatPos].startswith("Časopisy"):
      self.selCat = "časopis"
    elif self.browsePath[selCatPos].startswith("Texty"):
      self.selCat = "text"
    elif self.browsePath[selCatPos].startswith("Zora"):
      self.selCat = "casopis_zora"
    elif self.browsePath[selCatPos].startswith("Zvukové dokumenty"):
      self.selCat = "audio_dokument"
    if self.currentLevel == 0:
      if self.browsePath[0] == "Záložky":
        xmlFile = os.path.join(libraryDataFolder, self.bookmarksFileName)
        treexml = ET.parse(xmlFile)
        self.resultsTree = treexml.getroot()
        for dok in self.resultsTree.iter("dokument"):
          try:
            dokitem = dok.find("nazev").text
            if dokitem is not None:
              dokstr = dokitem
              dokitem = dok.find("nazev_epizody").text
              if dokitem is not None:
                dokstr = dokitem
                dokitem = dok.find("cislo_epizody").text
                if dokitem is not None:
                  dokstr = dokitem + " " + dokstr
              flist.append(dokstr)
          except: pass
        for dok in self.resultsTree.iter("book"):
          try:
            dokitem = dok.find("title").text
            if dokitem is not None:
              flist.append(dokitem)
          except: pass
        for dok in self.resultsTree.iter("film"):
          try:
            dokitem = dok.find("title").text
            if dokitem is not None:
              flist.append(dokitem)
          except: pass
        for dok in self.resultsTree.iter("polozka"):
          try:
            dokitem = dok.find("nazev").text
            if dokitem is not None:
              flist.append(dokitem)
          except: pass
        for dok in self.resultsTree.iter("record"):
          dtField = dok.find("datafield[@tag='245']")
          sbField = dtField.find("subfield[@code='a']")
          dokitem = sbField.text
          if dokitem is not None:
            flist.append(dokitem)
        for dok in self.resultsTree.iter("zora"):
          try:
            dokitem = dok.find("nazev").text
            if dokitem is not None:
              flist.append(dokitem)
          except: pass
        self.currentLevel = 4
        self.fillmainList(flist)
        return
      if self.browsePath[0].startswith("Nové tituly"):
        if not self.catalogKddLoaded:
          dlgMessage = "Katalog KDD ještě není načten, prosím o chvíli strpení."
          dlgTitle = "Upozornění"
          wx.MessageBox(dlgMessage, dlgTitle)
          self.passKey = True
          return
        self.resultsTree.clear()
        newsDate = timestamptoDateTime(kddUpdateBefore)
        knihaCount = 0
        casopisCount = 0
        textCount = 0
        zoraCount = 0
        audioDokumentCount =0
        for dok in self.tree.iter("dokument"):
          try:
            dokDateStr = dok.find("datum_schvaleni").text 
            if dokDateStr:
              dokDate = strtoDate(dokDateStr)
              if dokDate > newsDate:
                self.resultsTree.append(dok)
                if "kniha" in dok.find("kategorie_dokumentu_nazev_cs").text:
                  knihaCount += 1
                elif "časopis" in dok.find("kategorie_dokumentu_nazev_cs").text:
                  casopisCount += 1
                elif "text" in dok.find("kategorie_dokumentu_nazev_cs").text:
                  textCount += 1
                elif "casopis_zora" in dok.find("kategorie_dokumentu_nazev_cs").text:
                  zoraCount += 1
                elif "audio_dokument" in dok.find("kategorie_dokumentu_nazev_cs").text:
                  audioDokumentCount += 1
          except: pass
        self.categoriesNews.clear()
        self.categoriesNews.append("Knihy (" + str(knihaCount) + ")")
        self.categoriesNews.append("Časopisy (" + str(casopisCount) + ")")
        self.categoriesNews.append("Texty (" + str(textCount) + ")")
        self.categoriesNews.append("Zvukové dokumenty (" + str(audioDokumentCount) + ")")
        self.categoriesNews.append("Zora (" + str(zoraCount) + ")")
        flist = self.categoriesNews
        self.currentLevel = 1
        self.fillmainList(flist)
        return
      if self.browsePath[0] == "Moje knihovna":
        self.displayMyLibrary()
        return
      if self.browsePath[0].startswith("Novinky KTN"):
        if not self.catalogKtnLoaded:
          dlgMessage = "Katalog KTN ještě není načten, prosím o chvíli strpení."
          dlgTitle = "Upozornění"
          wx.MessageBox(dlgMessage, dlgTitle)
          self.passKey = True
          return
        self.resultsTree.clear()
        todayDateKtn = datetime.datetime.now()
        mp3CountKtn = 0
        textCountKtn = 0
        for dok in self.treeKtn.iter("record"):
          ctrlField = dok.find("controlfield[@tag='001']")
          if (ctrlField.text[:2] == "MP") or (ctrlField.text[0] == "D" and ctrlField.text[1].isnumeric()):
            ctrlField2 = dok.find("controlfield[@tag='008']")
            rawDate = ctrlField2.text[:6]
            if rawDate:
              tyear = todayDateKtn.year
              kyear = int("20" + rawDate[0:2])
              if tyear < kyear:
                kyear = kyear - 100
              titleDateKtn = datetime.datetime(kyear, int(rawDate[2:4]), int(rawDate[4:6]))
              diffDateKtn = todayDateKtn - titleDateKtn
              daysKtn = diffDateKtn.days
              if daysKtn < 61:
                self.resultsTree.append(dok)
                if ctrlField.text[:2] == "MP":
                  mp3CountKtn += 1
                elif ctrlField.text[0] == "D" and ctrlField.text[1].isnumeric():
                  textCountKtn += 1
        self.categoriesNewsKtn.clear()
        self.categoriesNewsKtn.append("MP3 (" + str(mp3CountKtn) + ")")
        self.categoriesNewsKtn.append("Texty (" + str(textCountKtn) + ")")
        flist = self.categoriesNewsKtn
        self.currentLevel = 1
        self.fillmainList(flist)
        return
      if self.browsePath[0] == "Jeden svět (Mathilda)":
        jedenSvetPageHtml = ""
        jedenSvetPageText = ""
        self.jedenSvetDict.clear()
        jedenSvetUrl = "https://www.mathilda.cz/projekt/casopis"
        request = urllib.request.Request(jedenSvetUrl)
        try:
          response = urllib.request.urlopen(request)
          if response.code != 200:
            errorMessage = _(u"Nelze načíst čísla časopisu Jeden svět.")
            errorTitle = _(u"Upozornění")
            wx.MessageBox(errorMessage, errorTitle)
            self.passKey = True
            return
        except:
          errorMessage = _(u"Nelze načíst čísla časopisu Jeden svět.")
          errorTitle = _(u"Upozornění")
          wx.MessageBox(errorMessage, errorTitle)
          self.passKey = True
          return
        content = response.read()
        content = content.decode()
        jedenSvetPageHtml = content
        # remove HTML, convert to plain text
        jedenSvetPageText = cleanhtml(content)
        jedenSvetList = []
        for line in jedenSvetPageText.splitlines():
          if line.strip().startswith("Jeden svět"):
            line2 = line.split("Jeden svět")[1]
            line2 = line2.strip()
            if line2 and line2[0].isnumeric():
              jedenSvetList.append(line.strip())
        for line in jedenSvetList:
          pos = jedenSvetPageHtml.find(line)
          pos2 = jedenSvetPageHtml.rfind('a href=', 0, pos)
          pos2 = pos2 + 8
          pos3 = jedenSvetPageHtml.find('"', pos2)
          addr = jedenSvetPageHtml[pos2:pos3]
          self.jedenSvetDict[self.jedensvetNormalizeName(line)] = addr
        # get archive magazines
        jedenSvetUrl = "https://www.mathilda.cz/casopis-archiv"
        request = urllib.request.Request(jedenSvetUrl)
        try:
          response = urllib.request.urlopen(request)
          if response.code != 200:
            errorMessage = _(u"Nelze načíst čísla časopisu Jeden svět.")
            errorTitle = _(u"Upozornění")
            wx.MessageBox(errorMessage, errorTitle)
            self.passKey = True
            return
        except:
          errorMessage = _(u"Nelze načíst čísla časopisu Jeden svět.")
          errorTitle = _(u"Upozornění")
          wx.MessageBox(errorMessage, errorTitle)
          self.passKey = True
          return
        content = response.read()
        content = content.decode()
        jedenSvetPageHtml = content
        # remove HTML, convert to plain text
        jedenSvetPageText = cleanhtml(content)
        jedenSvetList = []
        for line in jedenSvetPageText.splitlines():
          if line.strip().startswith("Jeden svět"):
            line2 = line.split("Jeden svět")[1]
            line2 = line2.strip()
            if line2 and line2[0].isnumeric():
              jedenSvetList.append(line.strip())
        for line in jedenSvetList:
          pos = jedenSvetPageHtml.find(line)
          pos2 = jedenSvetPageHtml.rfind('a href=', 0, pos)
          pos2 = pos2 + 8
          pos3 = jedenSvetPageHtml.find('"', pos2)
          addr = jedenSvetPageHtml[pos2:pos3]
          self.jedenSvetDict[self.jedensvetNormalizeName(line)] = addr
        flist = []
        self.libraryType = "JEDENSVET"
        self.currentLevel = 4
        self.fillmainList(self.jedenSvetDict.keys())
        return
      if self.browsePath[0] == "Zora MP3":
        self.zoraMp3Page = ""
        self.zoraMp3PageHtml = ""
        global zoraUserLogin
        global zoraUserPassword
        if not zoraUserLogin:
          errorMessage = _(u"Nelze se přihlásit do Zory, protože nejsou zadány uživatelské údaje.")
          errorTitle = _(u"Upozornění")
          wx.MessageBox(errorMessage, errorTitle)
          self.passKey = True
          return
        zoraMp3Link = "https://archiv.sons.cz/zora/mp3/index.php"
        values = {'login_name': zoraUserLogin, 'login_pw': zoraUserPassword}
        query_string = urllib.parse.urlencode(values)
        data = query_string.encode()
        request = urllib.request.Request(zoraMp3Link, method="POST")
        request.data = data
        try:
          response = urllib.request.urlopen(request)
          if response.code != 200:
            errorMessage = _(u"Nelze se přihlásit do Zory, možná nejsou zadány správně uživatelské údaje.")
            errorTitle = _(u"Upozornění")
            wx.MessageBox(errorMessage, errorTitle)
            self.passKey = True
            return
        except:
          errorMessage = _(u"Nelze se přihlásit do Zory, možná nejsou zadány správně uživatelské údaje.")
          errorTitle = _(u"Upozornění")
          wx.MessageBox(errorMessage, errorTitle)
          self.passKey = True
          return
        content = response.read()
        content = content.decode("windows-1250", errors="ignore")
        self.zoraMp3PageHtml = content
        # remove HTML, convert to plain text
        self.zoraMp3Page = cleanhtml(content)
        # login is successful string
        indexActive = self.zoraMp3Page.find("Aktivovány máte tyto časopisy:")
        # when user is not logged
        if indexActive == -1:
          errorMessage = _(u"Nelze se přihlásit do Zory, možná nejsou zadány správně uživatelské údaje.")
          errorTitle = _(u"Upozornění")
          wx.MessageBox(errorMessage, errorTitle)
          self.passKey = True
          return
        endIndex = self.zoraMp3Page.find("   ", indexActive)
        # starting index of the active magazines
        indexActive = self.zoraMp3Page.find("\n",indexActive)+1
        # string with active magazines
        aktCasopisy = self.zoraMp3Page[indexActive:endIndex].strip()
        # list of active magazines
        aktCasopisyList = aktCasopisy.split(",")
        # stripped list of active magazines
        aktCasopisyList = [s.strip() for s in aktCasopisyList]
        # list of active magazines without empty items
        aktCasopisyList = list(filter(None, aktCasopisyList))
        # old magazines are not listed under active magazines, so we add them here
        aktCasopisyList.append("Technická revue")
        aktCasopisyList.append("Vstupte")
        aktCasopisyList.append("Masér")
        if len(aktCasopisyList) == 0:
          dlgMessage = "Nemáte aktivovány žádné časopisy."
          dlgTitle = "Upozornění"
          wx.MessageBox(dlgMessage, dlgTitle)
          self.passKey = True
          return
        else:
          self.currentLevel = 1
          self.fillmainList(aktCasopisyList)
          return
      if self.browsePath[0] == "Zora Texty":
        self.zoraTextyPage = ""
        self.zoraTextyPageHtml = ""
        if not zoraUserLogin:
          self.zoraTextyLogged = False
        zoraTextyLink = "https://archiv.sons.cz/zora/text/index.php"
        values = {'login_name': zoraUserLogin, 'login_pw': zoraUserPassword}
        query_string = urllib.parse.urlencode(values)
        data = query_string.encode()
        request = urllib.request.Request(zoraTextyLink, method="POST")
        request.data = data
        try:
          response = urllib.request.urlopen(request)
          if response.code != 200:
            errorMessage = _(u"Nelze se přihlásit do Zory, možná nejsou zadány správně uživatelské údaje.")
            errorTitle = _(u"Upozornění")
            wx.MessageBox(errorMessage, errorTitle)
            self.passKey = True
            return
        except:
          errorMessage = _(u"Nelze se přihlásit do Zory, možná nejsou zadány správně uživatelské údaje.")
          errorTitle = _(u"Upozornění")
          wx.MessageBox(errorMessage, errorTitle)
          self.passKey = True
          return
        content = response.read()
        content = content.decode("windows-1250", errors="ignore")
        self.zoraTextyPageHtml = content
        # remove HTML, convert to plain text
        self.zoraTextyPage = cleanhtml(content)
        # login is successful string
        indexLogin = self.zoraTextyPage.find("Přihlášený uživatel:")
        # when user is not logged
        if indexLogin == -1:
          self.zoraTextyLogged = False
          indexLogin = 0
        else:
          self.zoraTextyLogged = True
        startMagIndex = self.zoraTextyPage.find("Časopis:",indexLogin)
        startMagIndex = self.zoraTextyPage.find("\n", startMagIndex)+1
        magPageList = self.zoraTextyPage[startMagIndex:].splitlines()
        magInserted = False
        flist = []
        for s in magPageList:
          if not s.strip() and magInserted:
            break
          if s.strip():
            magInserted = True
            flist.append(s.strip())
        self.currentLevel = 1
        self.fillmainList(flist)
        return
      if self.browsePath[0] == "MP3 KTN":
        if not self.catalogKtnLoaded:
          dlgMessage = "Katalog KTN ještě není načten, prosím o chvíli strpení."
          dlgTitle = "Upozornění"
          wx.MessageBox(dlgMessage, dlgTitle)
          self.passKey = True
          return
        catLevel = 1
        for dok in self.treeKtn.iter("record"):
          ctrlField = dok.find("controlfield[@tag='001']")
          if ctrlField.text[:2] == "MP":
            for dtField in dok.findall("datafield[@tag='653']"):
              if dtField.attrib["ind1"] == str(catLevel):
                sbField = dtField.findall("subfield[@code='a']")
                for cat in sbField:
                  dokNazev = cat.text
                  if dokNazev is not None:
                    flist.append(dokNazev)
        self.currentLevel = 1
        self.fillmainList(flist)
        return
      if self.browsePath[0] == "Texty KTN":
        if not self.catalogKtnLoaded:
          dlgMessage = "Katalog KTN ještě není načten, prosím o chvíli strpení."
          dlgTitle = "Upozornění"
          wx.MessageBox(dlgMessage, dlgTitle)
          self.passKey = True
          return
        catLevel = 1
        for dok in self.treeKtn.iter("record"):
          ctrlField = dok.find("controlfield[@tag='001']")
          if ctrlField.text[0] == "D" and ctrlField.text[1].isnumeric():
            for dtField in dok.findall("datafield[@tag='653']"):
              if dtField.attrib["ind1"] == str(catLevel):
                sbField = dtField.findall("subfield[@code='a']")
                for cat in sbField:
                  dokNazev = cat.text
                  if dokNazev is not None:
                    flist.append(dokNazev)
        self.currentLevel = 1
        self.fillmainList(flist)
        return
      if self.browsePath[0] == "Knihy SPVK":
        if not self.catalogSpvkLoaded:
          dlgMessage = "Katalog SPVK ještě není načten, prosím o chvíli strpení."
          dlgTitle = "Upozornění"
          wx.MessageBox(dlgMessage, dlgTitle)
          self.passKey = True
          return
        for dok in self.treeSpvk.iter("kategorie"):
          dokNazev = dok.find("nazev_kategorie").text
          if dokNazev is not None:
            flist.append(dokNazev)
        self.currentLevel = 1
        self.fillmainList(flist)
        return
      if self.browsePath[0] == "Knihy Mathilda":
        if not self.catalogMathildaBookLoaded:
          dlgMessage = "Katalog Knihy Mathilda ještě není načten, prosím o chvíli strpení."
          dlgTitle = "Upozornění"
          wx.MessageBox(dlgMessage, dlgTitle)
          self.passKey = True
          return
        for dok in self.treeMathildaBooks.iter("book"):
          dokitem = dok.find("category").text
          if dokitem is not None:
            if "/" in dokitem:
              for x in dokitem.split("/"):
                flist.append(x.strip())
            else:
              flist.append(dokitem)
        self.currentLevel = 1
        self.fillmainList(flist)
        return
      if self.browsePath[0] == "Filmy Mathilda":
        if not self.catalogMathildaMovieLoaded:
          dlgMessage = "Katalog Filmy Mathilda ještě není načten, prosím o chvíli strpení."
          dlgTitle = "Upozornění"
          wx.MessageBox(dlgMessage, dlgTitle)
          self.passKey = True
          return
        for dok in self.treeMathildaMovies.iter("film"):
          dokitem = dok.find("category").text
          if dokitem is not None:
            if "/" in dokitem:
              for x in dokitem.split("/"):
                flist.append(x.strip())
            else:
              flist.append(dokitem)
        if flist:
          for x in flist:
            if ">" in x:
              flist[flist.index(x)] = x.split(">")[0]
        self.currentLevel = 1
        self.fillmainList(flist)
        return
      if self.browsePath[0] == "Knihy" or self.browsePath[0] == "Texty" or self.browsePath[0] == "Zvukové dokumenty":
        if not self.catalogKddLoaded:
          dlgMessage = "Katalog KDD ještě není načten, prosím o chvíli strpení."
          dlgTitle = "Upozornění"
          wx.MessageBox(dlgMessage, dlgTitle)
          self.passKey = True
          return
        for dok in self.tree.iter("dokument"):
          if self.selCat in dok.find("kategorie_dokumentu_nazev_cs").text:
            dokitem = dok.find("typ_dokumentu_nazev_cs").text
            if dokitem is not None:
              if ";" in dokitem:
                for x in dokitem.split(";"):
                  flist.append(x)
              else:
                flist.append(dokitem)
        self.currentLevel = 1
        self.fillmainList(flist)
        return
      if self.browsePath[0] == "Časopisy" or self.browsePath[0] == "Zora":
        if not self.catalogKddLoaded:
          dlgMessage = "Katalog KDD ještě není načten, prosím o chvíli strpení."
          dlgTitle = "Upozornění"
          wx.MessageBox(dlgMessage, dlgTitle)
          self.passKey = True
          return
        self.magArchiveList = []
        curYear = datetime.datetime.now().year
        for dok in self.tree.iter("dokument"):
          if self.selCat in dok.find("kategorie_dokumentu_nazev_cs").text:
            dokYear = dok.find("rok_vydani").text
            try:
              dokYearNum = int(dokYear)
            except:
              dokYearNum = 0
            if dokYearNum > (curYear -2):
              dokitem = dok.find("typ_dokumentu_nazev_cs").text
              if dokitem is not None:
                if ";" in dokitem:
                  for x in dokitem.split(";"):
                    flist.append(x)
                else:
                  flist.append(dokitem)
            else:
              dokitem = dok.find("typ_dokumentu_nazev_cs").text
              if dokitem is not None:
                if ";" in dokitem:
                  for x in dokitem.split(";"):
                    self.magArchiveList.append(x)
                else:
                  self.magArchiveList.append(dokitem)
        if self.magArchiveList:
          # difference between all magazines (magArchiveList) and latest magazines (flist)
          self.magArchiveList = list(set(self.magArchiveList) - set(flist))
          flist.append("Archiv starších časopisů")
        self.currentLevel = 1
        self.fillmainList(flist)
        return
      if self.browsePath[0] == "Hledání":
        self.kddSearch()
        return
      if self.browsePath[0] == "Informace o uživateli":
        self.userInfo()
        return
      if self.browsePath[0] == "Nastavení":
        flist = ["KDD - uživatelské údaje", "Mathilda - uživatelské údaje", "SPVK - uživatelské údaje", "KTN - uživatelské údaje"]
        self.currentLevel = 1
        self.fillmainList(flist)
        return
    if self.currentLevel == 1:
      if self.browsePath[0].startswith("Novinky KTN"):
        if self.browsePath[1].endswith("(0)"):
          self.browsePath[1] = ""
          dlgMessage = "V této sekci nejsou žádné nové tituly."
          dlgTitle = "Informace"
          wx.MessageBox(dlgMessage, dlgTitle)
          self.passKey = True
          return
        if self.browsePath[1].startswith("MP3"):
          for dok in self.resultsTree.iter("record"):
            ctrlField = dok.find("controlfield[@tag='001']")
            if ctrlField.text[:2] == "MP":
              titleField = dok.find("datafield[@tag='245']")
              sbField = titleField.find("subfield[@code='a']")
              dokNazev = sbField.text
              if dokNazev is not None:
                dokstr = dokNazev
                autorstr = self.ktnGetAuthors(dok)
                if autorstr:
                  dokstr = dokstr + " - " + autorstr
                flist.append(dokstr)
          self.libraryType = "KTNMP3"
          self.currentLevel = 4
          self.fillmainList(flist)
          return
        if self.browsePath[1].startswith("Texty"):
          for dok in self.resultsTree.iter("record"):
            ctrlField = dok.find("controlfield[@tag='001']")
            if ctrlField.text[0] == "D" and ctrlField.text[1].isnumeric():
              titleField = dok.find("datafield[@tag='245']")
              sbField = titleField.find("subfield[@code='a']")
              dokNazev = sbField.text
              if dokNazev is not None:
                dokstr = dokNazev
                autorstr = self.ktnGetAuthors(dok)
                if autorstr:
                  dokstr = dokstr + " - " + autorstr
                flist.append(dokstr)
          self.libraryType = "KTNTXT"
          self.currentLevel = 4
          self.fillmainList(flist)
          return
      if self.browsePath[0] == "Zora MP3":
        self.currentLevel = 3
        self.browsePath[2] = ""
        self.browsePath[3] = ""
      if self.browsePath[0] == "Zora Texty":
        today = datetime.date.today()
        curYear = today.year
        if self.zoraTextyLogged:
          stopYear = curYear
        else:
          stopYear = curYear -1
        flist = [str(x) for x in range(2000, stopYear+1)]
        self.currentLevel = 2
        self.fillmainList(flist)
        return
      if self.browsePath[0] == "MP3 KTN":
        catLevel = 2
        for dok in self.treeKtn.iter("record"):
          ctrlField = dok.find("controlfield[@tag='001']")
          if ctrlField.text[:2] == "MP":
            for dtField in dok.findall("datafield[@tag='653']"):
              if dtField.attrib["ind1"] == str(catLevel-1) and dtField.find("subfield").text == self.browsePath[1]:
                for dtField2 in dok.findall("datafield[@tag='653']"):
                  if dtField2.attrib["ind1"] == str(catLevel):
                    sbField = dtField2.findall("subfield[@code='a']")
                    for cat in sbField:
                      dokNazev = cat.text
                      if dokNazev is not None:
                        flist.append(dokNazev)
        self.currentLevel = 2
        if len(flist) == 0:
          self.browsePath[self.currentLevel] = ""
        else:
          self.fillmainList(flist)
          return
      if self.browsePath[0] == "Texty KTN":
        catLevel = 2
        for dok in self.treeKtn.iter("record"):
          ctrlField = dok.find("controlfield[@tag='001']")
          if ctrlField.text[0] == "D" and ctrlField.text[1].isnumeric():
            for dtField in dok.findall("datafield[@tag='653']"):
              if dtField.attrib["ind1"] == str(catLevel-1) and dtField.find("subfield").text == self.browsePath[1]:
                for dtField2 in dok.findall("datafield[@tag='653']"):
                  if dtField2.attrib["ind1"] == str(catLevel):
                    sbField = dtField2.findall("subfield[@code='a']")
                    for cat in sbField:
                      dokNazev = cat.text
                      if dokNazev is not None:
                        flist.append(dokNazev)
        self.currentLevel = 2
        if len(flist) == 0:
          self.browsePath[self.currentLevel] = ""
        else:
          self.fillmainList(flist)
          return
      if self.browsePath[0] == "Knihy SPVK":
        self.currentLevel = 3
        self.browsePath[2] = ""
        self.browsePath[3] = ""
      if self.browsePath[0] == "Knihy Mathilda":
        self.currentLevel = 3
        self.browsePath[2] = ""
        self.browsePath[3] = ""
      if self.browsePath[0] == "Filmy Mathilda":
        for dok in self.treeMathildaMovies.iter("film"):
          dokitem = dok.find("category").text
          if dokitem is not None:
            for i in dokitem.split("/"):
              if ">" in i:
                if self.browsePath[1] == i.split(">")[0]:
                  flist.append(i.split(">")[1].strip())
        if flist:
          for x in flist:
            if ">" in x:
              flist[flist.index(x)] = x.split(">")[0]
          self.currentLevel = 2
          self.fillmainList(flist)
          return
        else:
          self.currentLevel = 3
          self.browsePath[2] = ""
          self.browsePath[3] = ""
      if self.browsePath[0].startswith("Nové tituly"):
        if self.browsePath[1].endswith("(0)"):
          self.browsePath[1] = ""
          dlgMessage = "V této sekci nejsou žádné nové tituly."
          dlgTitle = "Informace"
          wx.MessageBox(dlgMessage, dlgTitle)
          self.passKey = True
          return
        for dok in self.resultsTree.iter("dokument"):
          if self.selCat in dok.find("kategorie_dokumentu_nazev_cs").text:
            dokitem = dok.find("nazev").text
            if dokitem is not None:
              dokstr = dokitem
              if self.selCat == "kniha" or self.selCat == "text" or self.selCat == "audio_dokument":
                try:
                  dokitemep = dok.find("nazev_epizody").text
                except:
                  dokitemep = None
                if dokitemep is not None:
                  dokstr = dokitem
                try:
                  dokitem = dok.find("autori").text
                except:
                  dokitem = None
                if dokitem is not None:
                  dokstr = dokstr + " - " + dokitem
                if dokitemep:
                  try:
                    dokitem = dok.find("cislo_epizody").text
                  except:
                    dokitem = None
                  if dokitem is not None:
                    dokstr = dokitem + " " + dokstr
              elif self.selCat == "časopis" or self.selCat == "casopis_zora":
                if self.selCat == "casopis_zora":
                  try:
                    dokitem = dok.find("poradove_cislo").text
                  except:
                    dokitem = None
                  if dokitem is not None:
                    dokstr = dokstr + " " + dokitem
                try:
                  dokitem = dok.find("rok_vydani").text
                except:
                  dokitem = None
                if dokitem is not None:
                  dokstr = dokstr + " (" + dokitem + ")"
              flist.append(dokstr)
        self.currentLevel = 4
        self.fillmainList(flist)
        return
      if self.browsePath[0] == "Časopisy" or self.browsePath[0] == "Zora":
        if self.browsePath[1] == "Archiv starších časopisů":
          self.currentLevel = 2     
          self.fillmainList(self.magArchiveList)
          return
        else:
          self.currentLevel = 3
          self.browsePath[2] = ""
          self.browsePath[3] = ""
      elif self.browsePath[0] == "Knihy" or self.browsePath[0] == "Texty" or self.browsePath[0] == "Zvukové dokumenty":
        for dok in self.tree.iter("dokument"):
          try:
            if self.selCat in dok.find("kategorie_dokumentu_nazev_cs").text and self.browsePath[1] in dok.find("typ_dokumentu_nazev_cs").text:
              dokitem = dok.find("predmetove_skupiny").text
              if dokitem is not None:
                if ";" in dokitem:
                  for x in dokitem.split(";"):
                    flist.append(x.strip().lower())
                else:
                  flist.append(dokitem.strip().lower())
          except: pass
        self.currentLevel = 2
        if len(flist) == 0:
          self.browsePath[self.currentLevel] = ""
        else:
          self.fillmainList(flist)
          return
      if self.browsePath[1] == "KDD - uživatelské údaje":
        self.userData()
        self.passKey = True
        return
      if self.browsePath[1] == "Mathilda - uživatelské údaje":
        self.userDataMathilda()
        self.passKey = True
        return
      if self.browsePath[1] == "SPVK - uživatelské údaje":
        self.userDataSpvk()
        self.passKey = True
        return
      if self.browsePath[1] == "KTN - uživatelské údaje":
        self.userDataKtn()
        self.passKey = True
        return
      if self.browsePath[1] == "Zora - uživatelské údaje":
        self.userDataZora()
        self.passKey = True
        return
    if self.currentLevel == 2:
      if self.browsePath[0] == "Zora Texty":
        self.currentLevel = 3
        self.browsePath[3] = ""
      if self.browsePath[0] == "MP3 KTN" or self.browsePath[0] == "Texty KTN":
        self.currentLevel = 3
        self.browsePath[3] = ""
      if self.browsePath[0] == "Filmy Mathilda":
        for dok in self.treeMathildaMovies.iter("film"):
          dokitem = dok.find("category").text
          if dokitem is not None:
            for i in dokitem.split("/"):
              if ">" in i:
                igList = [i.strip() for i in i.split(">")]
                if len(igList) > 2 and self.browsePath[1] == igList[0] and (self.browsePath[2] == "Vše" or self.browsePath[2] == igList[1]):
                  flist.append(igList[2].strip())
        if flist:
          for x in flist:
            if ">" in x:
              flist[flist.index(x)] = x.split(">")[0]
          self.currentLevel = 3
          self.fillmainList(flist)
          return
        else:
          self.currentLevel = 3
          self.browsePath[3] = ""
      if self.browsePath[0] == "Knihy" or self.browsePath[0] == "Texty" or self.browsePath[0] == "Zvukové dokumenty":
        if self.browsePath[2] == "Vše":
          self.browsePath[3] = ""
          self.currentLevel = 3
        else:
          for dok in self.tree.iter("dokument"):
            try:
              if self.selCat in dok.find("kategorie_dokumentu_nazev_cs").text and self.browsePath[1] in dok.find("typ_dokumentu_nazev_cs").text:
                if self.browsePath[2] == "" or self.browsePath[2] in dok.find("predmetove_skupiny").text.strip().lower():
                  dokitem = dok.find("formy").text
                  if dokitem is not None:
                    if ";" in dokitem:
                      for x in dokitem.split(";"):
                        flist.append(x.strip().lower())
                    else:
                      flist.append(dokitem.strip().lower())
            except: pass
          self.currentLevel = 3
          if len(flist) == 0:
            self.browsePath[3] = ""
            self.currentLevel = 3
          else:
            self.fillmainList(flist)
            return
      if self.browsePath[0] == "Časopisy" or self.browsePath[0] == "Zora":
        self.currentLevel = 3
        self.browsePath[3] = ""
    if self.currentLevel == 3:
      if self.browsePath[0] == "Zvukové dokumenty" and self.browsePath[1] == "Seriály s AD" and self.addLevel < 0:
        self.addLevel = 0
        for dok in self.tree.iter("dokument"):
          try:
            if self.selCat in dok.find("kategorie_dokumentu_nazev_cs").text and self.browsePath[1] in dok.find("typ_dokumentu_nazev_cs").text:
              if self.browsePath[2] == "" or self.browsePath[2] == "Vše" or self.browsePath[2] in dok.find("predmetove_skupiny").text.strip().lower():
                if self.browsePath[3] == "" or self.browsePath[3] == "Vše" or self.browsePath[3] in dok.find("formy").text.strip().lower():
                  dokitem = dok.find("serie").text
                  if dokitem is not None:
                    if ";" in dokitem:
                      for x in dokitem.split(";"):
                        flist.append(x.strip().lower())
                    else:
                      flist.append(dokitem.strip().lower())
          except: pass
        if len(flist) == 0:
          self.addPath[0] = ""
          self.addLevel = -1
        else:
          self.fillmainList(flist)
          return
      if self.browsePath[0] == "Filmy Mathilda" and self.addLevel < 0:
        self.addLevel = 0
        if self.browsePath[3] == "Vše":
          self.addPath[0] = ""
          self.addLevel = -1
        else:
          for dok in self.treeMathildaMovies.iter("film"):
            dokitem = dok.find("category").text
            if dokitem is not None:
              for i in dokitem.split("/"):
                if ">" in i:
                  igList = [i.strip() for i in i.split(">")]
                  if len(igList) > 3 and self.browsePath[1] == igList[0] and self.browsePath[2] == igList[1] and self.browsePath[3] == igList[2]:
                    flist.append(igList[3].strip())
          if flist:
            for x in flist:
              if ">" in x:
                flist[flist.index(x)] = x.split(">")[0]
            self.addLevel = 0
            self.fillmainList(flist)
            return
          else:
            self.addLevel = -1
            self.addPath[0] = ""
      if not self.browsePath[0].startswith("Nové tituly") and self.browsePath[0] != "Záložky":
        self.resultsTree.clear()
      if self.browsePath[0] == "Zora MP3":
        # selected item will be processed in lower case
        selMagazine = self.browsePath[1].lower()
        # convert all page to lowercase
        contentlower = self.zoraMp3Page.lower()
        # cut end of the page from "v případě"
        contentlower = contentlower.rsplit("v případě", 1)[0]
        # sometimes there are two spaces before -, replace them with one space
        contentlower = contentlower.replace("  - ", " - ")
        # find list of archive magazines on page and put it in magazinestr
        findmag = selMagazine + " - archiv čísel"
        findmag = findmag.lower()
        indexmag = contentlower.find(findmag)
        indexmag = contentlower.find("\n", indexmag)+1
        magazinestr = contentlower[indexmag:].strip()
        # list of archive magazines put in the magnumlist (space in startwith is added because of obzor)
        magnumlist = [x for x in magazinestr.splitlines() if x.startswith(selMagazine + " ")]
        # we edit all magazines to required format
        for i, s in enumerate(magnumlist):
          x = s.rsplit("(", 1)[0].strip()
          xlist = x.rsplit(" ", 1)
          xdatelist = xlist[1].split("/")
          titlemag = "20" + xdatelist[1] + ", " + self.browsePath[1] + " " + xdatelist[0]
          magnumlist[i] = titlemag
        self.libraryType = "ZORAMP3"
        self.currentLevel = 4
        self.fillmainList(magnumlist)
        return
      if self.browsePath[0] == "Zora Texty":
        # find short name for selected magazine
        magShortName = self.getMagShortNameZora(self.zoraTextyPageHtml, self.browsePath[1])
        if magShortName == "":
          errorMessage = "Nelze načíst čísla zvoleného časopisu."
          errorTitle = _(u"Upozornění")
          wx.MessageBox(errorMessage, errorTitle)
          self.passKey = True
          return
        dokLink = "https://archiv.sons.cz/zora/text/index.php"
        values = {'filtr_casopis': magShortName, 'filtr_rok': self.browsePath[2]}
        query_string = urllib.parse.urlencode(values)
        data = query_string.encode()
        request = urllib.request.Request(dokLink, method="POST")
        request.data = data
        try:
          response = urllib.request.urlopen(request)
          if response.code != 200:
            errorMessage = _(u"Nelze zobrazit čísla zvoleného časopisu, možná nejsou zadány správně uživatelské údaje.")
            errorTitle = _(u"Upozornění")
            wx.MessageBox(errorMessage, errorTitle)
            self.passKey = True
            return
        except:
          errorMessage = _(u"Nelze zobrazit čísla zvoleného časopisu, možná nejsou zadány správně uživatelské údaje.")
          errorTitle = _(u"Upozornění")
          wx.MessageBox(errorMessage, errorTitle)
          self.passKey = True
          return
        content = response.read()
        content = content.decode("windows-1250", errors="ignore")
        self.zoraMagPageHtml = content
        # create list of headings level 3 which contains magazine name, number and year
        headList = GetSubstrings(self.zoraMagPageHtml, "<h3>", "</h3>")
        if len(headList) > 0:
          magYear = self.browsePath[2]
          magNumbersList = []
          czMonths = ["leden", "únor", "březen", "duben", "květen", "červen", "červenec", "srpen", "září", "říjen", "listopad", "prosinec"]
          # create list of magazine numbers
          for i in headList:
            # find the year of this magazine number
            thisMagNumber = i.split()[-1]
            if thisMagNumber.isdigit() and int(thisMagNumber) > 1900:
              magYear = i.split()[-1]
            for j in i.split():
              if j.endswith(","):
                j = j[:-1]
              if j.isdigit() and magYear not in j:
                magNumbersList.append(str(j).zfill(2))
                break
              # some magazines can contain two numbers together, eg. téčko 7-8
              elif "-" in j:
                jdash = j.replace("-", "")
                if jdash.isdigit() and magYear not in jdash:
                  magNumbersList.append(str(j).zfill(2))
                  break
          if len(magNumbersList) == 0:
            # in some cases magazine is marked by month instead of number
            monthAdded = False
            for i in headList:
              for czm in czMonths:
                if czm in i.lower():
                  magNumbersList.append(czm)
                  monthAdded = True
                  break
          flist =[self.browsePath[1].rsplit("(", 1)[0].strip() + " " +x +", " + magYear for x in magNumbersList]
        self.libraryType = "ZORATXT"
        self.currentLevel = 4
        self.fillmainList(flist)
        return
      if self.browsePath[0] == "MP3 KTN":
        catLevel = 2
        for dok in self.treeKtn.iter("record"):
          ctrlField = dok.find("controlfield[@tag='001']")
          if ctrlField.text[:2] == "MP":
            for dtField in dok.findall("datafield[@tag='653']"):
              if dtField.attrib["ind1"] == str(catLevel-1) and dtField.find("subfield").text == self.browsePath[1]:
                if self.browsePath[2] == "" or self.browsePath[2] == "Vše":
                  titleField = dok.find("datafield[@tag='245']")
                  sbField = titleField.find("subfield[@code='a']")
                  dokNazev = sbField.text
                  if dokNazev is not None:
                    self.resultsTree.append(dok)
                    dokstr = dokNazev
                    autorstr = self.ktnGetAuthors(dok)
                    if autorstr:
                      dokstr = dokstr + " - " + autorstr
                    flist.append(dokstr)
                else:
                  for dtField2 in dok.findall("datafield[@tag='653']"):
                    if dtField2.attrib["ind1"] == str(catLevel):
                      for dtField3 in dtField2.findall("subfield[@code='a']"):
                        if dtField3.text == self.browsePath[2]:
                          titleField = dok.find("datafield[@tag='245']")
                          sbField = titleField.find("subfield[@code='a']")
                          dokNazev = sbField.text
                          if dokNazev is not None:
                            self.resultsTree.append(dok)
                            dokstr = dokNazev
                            autorstr = self.ktnGetAuthors(dok)
                            if autorstr:
                              dokstr = dokstr + " - " + autorstr
                            flist.append(dokstr)
        self.libraryType = "KTNMP3"
        self.currentLevel = 4
        self.fillmainList(flist)
        return
      if self.browsePath[0] == "Texty KTN":
        catLevel = 2
        for dok in self.treeKtn.iter("record"):
          ctrlField = dok.find("controlfield[@tag='001']")
          if ctrlField.text[0] == "D" and ctrlField.text[1].isnumeric():
            for dtField in dok.findall("datafield[@tag='653']"):
              if dtField.attrib["ind1"] == str(catLevel-1) and dtField.find("subfield").text == self.browsePath[1]:
                if self.browsePath[2] == "" or self.browsePath[2] == "Vše":
                  titleField = dok.find("datafield[@tag='245']")
                  sbField = titleField.find("subfield[@code='a']")
                  dokNazev = sbField.text
                  if dokNazev is not None:
                    self.resultsTree.append(dok)
                    dokstr = dokNazev
                    autorstr = self.ktnGetAuthors(dok)
                    if autorstr:
                      dokstr = dokstr + " - " + autorstr
                    flist.append(dokstr)
                else:
                  for dtField2 in dok.findall("datafield[@tag='653']"):
                    if dtField2.attrib["ind1"] == str(catLevel):
                      for dtField3 in dtField2.findall("subfield[@code='a']"):
                        if dtField3.text == self.browsePath[2]:
                          titleField = dok.find("datafield[@tag='245']")
                          sbField = titleField.find("subfield[@code='a']")
                          dokNazev = sbField.text
                          if dokNazev is not None:
                            self.resultsTree.append(dok)
                            dokstr = dokNazev
                            autorstr = self.ktnGetAuthors(dok)
                            if autorstr:
                              dokstr = dokstr + " - " + autorstr
                            flist.append(dokstr)
        self.libraryType = "KTNTXT"
        self.currentLevel = 4
        self.fillmainList(flist)
        return
      if self.browsePath[0] == "Knihy SPVK":
        for dok in self.treeSpvk.iter("polozka"):
          try:
            for dokcat in dok.iter("kategorie"):
              if self.browsePath[1] in dokcat.find("nazev_kategorie").text:
                self.resultsTree.append(dok)
                dokitem = dok.find("nazev").text
                if dokitem is not None:
                  dokstr = dokitem
                  autorstr = self.spvkGetAuthors(dok)
                  if autorstr:
                    dokstr = dokstr + " - " + autorstr
                  flist.append(dokstr)
                break
          except: pass
        self.libraryType = "SPVK"
        self.currentLevel = 4
        self.fillmainList(flist)
        return
      if self.browsePath[0] == "Knihy Mathilda":
        for dok in self.treeMathildaBooks.iter("book"):
          try:
            if self.browsePath[1] in dok.find("category").text:
              self.resultsTree.append(dok)
              dokitem = dok.find("title").text
              if dokitem is not None:
                dokstr = dokitem
                autorstr = self.mathildaGetAuthors(dok)
                if autorstr:
                  dokstr = dokstr + " - " + autorstr
                flist.append(dokstr)
          except: pass
        self.libraryType = "Mathilda"
        self.currentLevel = 4
        self.fillmainList(flist)
        return
      if self.browsePath[0] == "Filmy Mathilda":
        for dok in self.treeMathildaMovies.iter("film"):
          dokMovCat = dok.find("category").text
          if dokMovCat is not None:
            for i in dokMovCat.split("/"):
              igList = [i.strip() for i in i.split(">")]
              if self.browsePath[1] == igList[0]:
                if self.browsePath[2] == "" or self.browsePath[2] == "Vše" or (len(igList) > 1 and self.browsePath[2] == igList[1]):
                  if self.browsePath[3] == "" or self.browsePath[3] == "Vše" or (len(igList) > 2 and self.browsePath[3] == igList[2]):
                    if self.addPath[0] == "" or self.addPath[0] == "Vše" or (len(igList) > 3 and self.addPath[0] == igList[3]):
                      self.resultsTree.append(dok)
                      dokitem = dok.find("title").text
                      if dokitem is not None:
                        dokstr = dokitem
                        autorstr = self.mathildaGetAuthors(dok)
                        if autorstr:
                          dokstr = dokstr + " - " + autorstr
                        flist.append(dokstr)
        self.libraryType = "Mathilda"
        self.currentLevel = 4
        self.fillmainList(flist)
        return
      if self.browsePath[0] == "Knihy" or self.browsePath[0] == "Časopisy" or self.browsePath[0] == "Texty" or self.browsePath[0] == "Zora" or self.browsePath[0] == "Zvukové dokumenty":
        if self.browsePath[1] == "Archiv starších časopisů":
          typdokumentuIndex = 2
        else:
          typdokumentuIndex = 1
        for dok in self.tree.iter("dokument"):
          try:
            if self.selCat in dok.find("kategorie_dokumentu_nazev_cs").text and self.browsePath[typdokumentuIndex] in dok.find("typ_dokumentu_nazev_cs").text:
              if typdokumentuIndex == 2 or self.browsePath[2] == "" or self.browsePath[2] == "Vše" or self.browsePath[2] in dok.find("predmetove_skupiny").text.strip().lower():
                if self.browsePath[3] == "" or self.browsePath[3] == "Vše" or self.browsePath[3] in dok.find("formy").text.strip().lower():
                  if self.addPath[0] == "" or self.addPath[0] == "Vše" or self.addPath[0] in dok.find("serie").text.strip().lower():
                    if self.addPath[1] == "" or self.addPath[1] == "Vše" or self.addPath[1] in dok.find("cislo_serie").text.strip().lower():
                      self.resultsTree.append(dok)
                      dokitem = dok.find("nazev").text
                      if dokitem is not None:
                        dokstr = dokitem
                        if self.browsePath[0] == "Knihy" or self.browsePath[0] == "Texty" or self.browsePath[0] == "Zvukové dokumenty":
                          try:
                            dokitemep = dok.find("nazev_epizody").text
                          except:
                            dokitemep = None
                          if dokitemep is not None:
                            dokstr = dokitemep
                          try:
                            dokitem = dok.find("autori").text
                          except:
                            dokitem = None
                          if dokitem is not None:
                            dokstr = dokstr + " - " + dokitem
                          if dokitemep:
                            try:
                              dokitem = dok.find("cislo_epizody").text
                            except:
                              dokitem = None
                            if dokitem is not None:
                              dokstr = dokitem + " " + dokstr
                        elif self.browsePath[0] == "Časopisy":
                          dokitem = dok.find("rok_vydani").text
                          if dokitem is not None:
                            dokstr = dokitem + ", " + dokstr
                        elif self.browsePath[0] == "Zora":
                          dokitem = dok.find("rok_vydani").text
                          if dokitem is not None:
                            dokstr = dokitem + ", " + dokstr
                          dokitem = dok.find("poradove_cislo").text
                          if dokitem is not None:
                            dokstr = dokstr + " " + dokitem
                        flist.append(dokstr)
          except: pass
        self.libraryType = "KDD"
        self.currentLevel = 4
        self.fillmainList(flist)
        return
      elif self.browsePath[0] == "Hledání":
        if self.sNazev or self.sAutor or self.sAnotace or self.sRokVydani or self.sMistoVydani or self.sNakladatelstvi or self.sIsbn or self.sVse:
          for dok in self.tree.iter("dokument"):
            dokNazev = ""
            dokAutor = ""
            dokAnotace = ""
            dokVydani = ""
            dokNakladatelstvi = ""
            dokMistoVydani = ""
            dokIsbn = ""
            if dok.find("nazev") is not None and dok.find("nazev").text is not None:
              dokNazev = dok.find("nazev").text
            if dok.find("autori") is not None and dok.find("autori").text is not None:
              dokAutor = dok.find("autori").text
            if dok.find("anotace") is not None and dok.find("anotace").text is not None:
              dokAnotace = dok.find("anotace").text
            if dok.find("rok_vydani") is not None and dok.find("rok_vydani").text is not None:
              dokVydani = dok.find("rok_vydani").text
            if dok.find("nakladatelstvi_nazev_cs") is not None and dok.find("nakladatelstvi_nazev_cs").text is not None:
              dokNakladatelstvi = dok.find("nakladatelstvi_nazev_cs").text
            if dok.find("misto_vydani") is not None and dok.find("misto_vydani").text is not None:
              dokMistoVydani = dok.find("misto_vydani").text
            if dok.find("isbn") is not None and dok.find("isbn").text is not None:
              dokIsbn = dok.find("isbn").text
            if self.sNazev == "" or self.sNazev.upper() in dokNazev.upper():
              if self.sAutor == "" or self.sAutor.upper() in dokAutor.upper():
                if self.sAnotace == "" or self.sAnotace.upper() in dokAnotace.upper():
                  if self.sRokVydani == "" or self.sRokVydani.upper() in dokVydani.upper():
                    if self.sNakladatelstvi == "" or self.sNakladatelstvi.upper() in dokNakladatelstvi.upper():
                      if self.sMistoVydani == "" or self.sMistoVydani.upper() in dokMistoVydani.upper():
                        if self.sIsbn == "" or self.sIsbn.upper() in dokIsbn.upper():
                          if (self.sVse == "" or 
                            self.sVse.upper() in dokNazev.upper() or
                            self.sVse.upper() in dokAutor.upper() or
                            self.sVse.upper() in dokAnotace.upper() or
                            self.sVse.upper() in dokVydani.upper() or
                            self.sVse.upper() in dokNakladatelstvi.upper() or
                            self.sVse.upper() in dokMistoVydani.upper() or
                            self.sVse.upper() in dokIsbn.upper()
                          ):
                            self.resultsTree.append(dok)
                            dokkat = dok.find("kategorie_dokumentu_nazev_cs").text
                            dokitem = dok.find("nazev").text
                            if dokitem is not None:
                              dokstr = dokitem
                              if dokkat == "kniha" or dokkat == "text":
                                dokitem = dok.find("autori").text
                                if dokitem is not None:
                                  dokstr = dokstr + " - " + dokitem
                              elif dokkat == "časopis":
                                dokitem = dok.find("rok_vydani").text
                                if dokitem is not None:
                                  dokstr = dokitem + ", " + dokstr
                              elif dokkat == "casopis_zora":
                                dokitem = dok.find("rok_vydani").text
                                if dokitem is not None:
                                  dokstr = dokitem + ", " + dokstr
                                dokitem = dok.find("poradove_cislo").text
                                if dokitem is not None:
                                  dokstr = dokstr + " " + dokitem
                              dokstr = dokstr + "; " + dokkat
                              dokstr = dokstr + "; KDD"
                              flist.append(dokstr)
        if self.sNazev or self.sAutor or self.sAnotace or self.sVse:
          for dok in self.treeMathildaBooks.iter("book"):
            dokNazev = ""
            dokAnotace = ""
            if dok.find("title") is not None and dok.find("title").text is not None:
              dokNazev = dok.find("title").text
            if dok.find("description") is not None and dok.find("description").text is not None:
              dokAnotace = dok.find("description").text
            if self.sNazev == "" or self.sNazev.upper() in dokNazev.upper():
              if self.sAutor == "" or self.sAutor.upper() in self.mathildaGetAuthors(dok).upper():
                if self.sAnotace == "" or self.sAnotace.upper() in dokAnotace.upper():
                  if (self.sVse == "" or
                    self.sVse.upper() in dokNazev.upper() or
                    self.sVse.upper() in self.mathildaGetAuthors(dok).upper() or
                    self.sVse.upper() in dokAnotace.upper()
                  ):
                    self.resultsTree.append(dok)
                    dokitem = dok.find("title").text
                    if dokitem is not None:
                      dokstr = dokitem
                      dokitem = self.mathildaGetAuthors(dok)
                      if dokitem is not None:
                        dokstr = dokstr + " - " + dokitem
                      dokstr = dokstr + "; kniha"
                      dokstr = dokstr + "; Mathilda"
                      flist.append(dokstr)
        if self.sNazev or self.sAutor or self.sAnotace or self.sVse:
          for dok in self.treeMathildaMovies.iter("film"):
            dokNazev = ""
            dokAnotace = ""
            if dok.find("title") is not None and dok.find("title").text is not None:
              dokNazev = dok.find("title").text
            if dok.find("description") is not None and dok.find("description").text is not None:
              dokAnotace = dok.find("description").text
            if self.sNazev == "" or self.sNazev.upper() in dokNazev.upper():
              if self.sAutor == "" or self.sAutor.upper() in self.mathildaGetAuthors(dok).upper():
                if self.sAnotace == "" or self.sAnotace.upper() in dokAnotace.upper():
                  if (self.sVse == "" or
                    self.sVse.upper() in dokNazev.upper() or
                    self.sVse.upper() in self.mathildaGetAuthors(dok).upper() or
                    self.sVse.upper() in dokAnotace.upper()
                  ):
                    self.resultsTree.append(dok)
                    dokitem = dok.find("title").text
                    if dokitem is not None:
                      dokstr = dokitem
                      dokitem = self.mathildaGetAuthors(dok)
                      if dokitem is not None:
                        dokstr = dokstr + " - " + dokitem
                      dokstr = dokstr + "; film"
                      dokstr = dokstr + "; Mathilda"
                      flist.append(dokstr)
        if self.sNazev or self.sAutor or self.sAnotace or self.sVse:
          for dok in self.treeSpvk.iter("polozka"):
            dokNazev = ""
            dokAnotace = ""
            if dok.find("nazev") is not None and dok.find("nazev").text is not None:
              dokNazev = dok.find("nazev").text
            if dok.find("anotace") is not None and dok.find("anotace").text is not None:
              dokAnotace = dok.find("anotace").text
            if self.sNazev == "" or self.sNazev.upper() in dokNazev.upper():
              if self.sAutor == "" or self.sAutor.upper() in self.spvkGetAuthors(dok).upper():
                if self.sAnotace == "" or self.sAnotace.upper() in dokAnotace.upper():
                  if (self.sVse == "" or
                    self.sVse.upper() in dokNazev.upper() or
                    self.sVse.upper() in self.spvkGetAuthors(dok).upper() or
                    self.sVse.upper() in dokAnotace.upper()
                  ):
                    self.resultsTree.append(dok)
                    dokitem = dok.find("nazev").text
                    if dokitem is not None:
                      dokstr = dokitem
                      dokitem = self.spvkGetAuthors(dok)
                      if dokitem is not None:
                        dokstr = dokstr + " - " + dokitem
                      dokstr = dokstr + "; kniha"
                      dokstr = dokstr + "; SPVK"
                      flist.append(dokstr)
        if self.sNazev or self.sAutor or self.sAnotace or self.sRokVydani or self.sMistoVydani or self.sIsbn or self.sNacitatel or self.sVse:
          for dok in self.treeKtn.iter("record"):
            ctrlField = dok.find("controlfield[@tag='001']")
            if (ctrlField.text[:2] == "MP") or (ctrlField.text[0] == "D" and ctrlField.text[1].isnumeric()):
              dokNazev = ""
              dokAnotace = ""
              dokVydani = ""
              dokIsbn = ""
              dokNacitatel = ""
              dtField = dok.find("datafield[@tag='245']")
              sbField = dtField.find("subfield[@code='a']")
              dokNazev = sbField.text
              dtField = dok.find("datafield[@tag='520']")
              if dtField is not None:
                sbField = dtField.find("subfield[@code='a']")
                dokAnotace = sbField.text
              dokVydani = ""
              dtField = dok.find("datafield[@tag='534']")
              if dtField is not None:
                sbField = dtField.find("subfield[@code='c']")
                if sbField is not None and sbField.text:
                  dokVydani = sbField.text
                sbField = dtField.find("subfield[@code='b']")
                if sbField is not None and sbField.text:
                  if dokVydani:
                    dokVydani = dokVydani + ", " + sbField.text
                  else:
                    dokVydani = sbField.text
                sbField = dtField.find("subfield[@code='z']")
                if sbField is not None and sbField.text:
                  dokIsbn = sbField.text
              ctrlField = dok.find("controlfield[@tag='001']")
              if ctrlField.text[:2] == "MP":
                dtField = dok.find("datafield[@tag='511']")
                if dtField is not None:
                  sbField = dtField.find("subfield[@code='a']")
                  dokNacitatel = sbField.text
              if self.sNazev == "" or self.sNazev.upper() in dokNazev.upper():
                if self.sAutor == "" or self.sAutor.upper() in self.ktnGetAuthors(dok).upper():
                  if self.sAnotace == "" or self.sAnotace.upper() in dokAnotace.upper():
                    if self.sRokVydani == "" or self.sRokVydani.upper() in dokVydani.upper():
                      if self.sNakladatelstvi == "" or self.sNakladatelstvi.upper() in dokVydani.upper():
                        if self.sMistoVydani == "" or self.sMistoVydani.upper() in dokVydani.upper():
                          if self.sIsbn == "" or self.sIsbn.upper() in dokIsbn.upper():
                            if self.sNacitatel == "" or self.sNacitatel.upper() in dokNacitatel.upper():
                              if (self.sVse == "" or
                                self.sVse.upper() in dokNazev.upper() or
                                self.sVse.upper() in self.ktnGetAuthors(dok).upper() or
                                self.sVse.upper() in dokAnotace.upper() or
                                self.sVse.upper() in dokVydani.upper() or
                                self.sVse.upper() in dokVydani.upper() or
                                self.sVse.upper() in dokVydani.upper() or
                                self.sVse.upper() in dokIsbn.upper() or
                                self.sVse.upper() in dokNacitatel.upper()
                              ):
                                self.resultsTree.append(dok)
                                dokitem = dokNazev
                                if dokitem is not None:
                                  dokstr = dokitem
                                  dokitem = self.ktnGetAuthors(dok)
                                  if dokitem is not None:
                                    dokstr = dokstr + " - " + dokitem
                                  ctrlField = dok.find("controlfield[@tag='001']")
                                  if ctrlField.text[:2] == "MP":
                                    dokstr = dokstr + "; mp3"
                                  elif ctrlField.text[0] == "D" and ctrlField.text[1].isnumeric():
                                    dokstr = dokstr + "; text"
                                  dokstr = dokstr + "; KTN"
                                  flist.append(dokstr)
        self.currentLevel = 4
        self.fillmainList(flist)
        return
    if self.currentLevel == 4:
      dok = self.findCurrentTitle()
      try:
        if self.libraryType == "KTNMP3" or self.libraryType == "KTNTXT":
          dokAutor = self.ktnGetAuthors(dok)
        elif self.libraryType == "SPVK":
          dokAutor = self.spvkGetAuthors(dok)
        elif self.libraryType == "Mathilda":
          dokAutor = self.mathildaGetAuthors(dok)
        else:
          dokAutor = dok.find("autori").text
      except:
        dokAutor = ""
      try:
        dokEdice = dok.find("edice_nazev_cs").text
      except:
        dokEdice = ""
      try:
        dokFile = dok.find("filetoread").text
      except:
        dokFile = ""
      try:
        dokFormat = dok.find("format_dokumentu").text
      except:
        dokFormat = ""
      try:
        dokKategorie = dok.find("kategorie_dokumentu_nazev_cs").text
      except:
        dokKategorie = ""
      # show What to do dialog
      if self.browsePath[0] == "Moje knihovna":
        if ".MP3" in dokFile.upper():
          WhatToDoDialog.selectionItems = ["Přehrát", "Informace o titulu", "Smazat titul", "Zpět"]
          if self.libraryType == "ZORAMP3" or self.libraryType == "JEDENSVET" or (self.libraryType == "KDD" and dokKategorie == "casopis_zora"):
            WhatToDoDialog.selectionItems.insert(1, "Zobrazit obsah časopisu")
        else:
          WhatToDoDialog.selectionItems = ["Číst", "Informace o titulu", "Smazat titul", "Zpět"]
      elif self.browsePath[0] == "Záložky":
        WhatToDoDialog.selectionItems = ["Stáhnout titul", "Informace o titulu", "Odebrat titul ze záložek", "Zpět"]
      else:
        WhatToDoDialog.selectionItems = ["Stáhnout titul", "Informace o titulu", "Přidat titul do záložek", "Zpět"]
        if dokFormat:
          WhatToDoDialog.selectionItems.remove("Stáhnout titul")
          if "audio" in dokFormat:
            WhatToDoDialog.selectionItems.insert(0, "Stáhnout titul jako zvuk")
          if "text" in dokFormat:
            WhatToDoDialog.selectionItems.insert(0, "Stáhnout titul jako text")
        # for now KDD and Jeden svět magazines will not have possibility adding to bookmarks
        if self.browsePath[0] == "Časopisy" or self.browsePath[0] == "Zora" or (self.browsePath[0].startswith("Nové tituly") and self.browsePath[1].startswith("Časopisy")) or (self.browsePath[0].startswith("Nové tituly") and self.browsePath[1].startswith("Zora")) or self.browsePath[0].startswith("Jeden svět"):
          WhatToDoDialog.selectionItems.remove("Přidat titul do záložek")
        if dokAutor:
          WhatToDoDialog.selectionItems.insert(2, "Další tituly od autora " + dokAutor)
        if dokEdice:
          WhatToDoDialog.selectionItems.insert(2, "Zobrazit celou sérii " + dokEdice)
      # find if the title is already in library
      if self.isinLibrary(dok):
        for i, s in enumerate(WhatToDoDialog.selectionItems):
          if (s == "Stáhnout titul"):
            s = s.replace("Stáhnout", "Znovu stáhnout", 1)
            WhatToDoDialog.selectionItems[i] = s
            break
      # special condition for "stáhnout jako text" and "stáhnout jako zvuk" items displayed in KDD for Zora
      if self.libraryType == "KDD":
        from os.path import expanduser
        home = expanduser("~")
        kddLibraryPath = os.path.join(home, self.kddLibraryFolder)
        curdokpath = os.path.join(kddLibraryPath, self.dokumentFolderName(dok))
        for i, s in enumerate(WhatToDoDialog.selectionItems):
          if (s == "Stáhnout titul jako zvuk" and os.path.isdir(curdokpath + " MP3")) or (s == "Stáhnout titul jako text" and os.path.isdir(curdokpath + " Text")):
            s = s.replace("Stáhnout", "Znovu stáhnout", 1)
            WhatToDoDialog.selectionItems[i] = s
      self.displayWhatToDo()
      if finalSelection == "Odebrat titul ze záložek":
        self.removefromBookmarks(dok)
      elif finalSelection == "Přidat titul do záložek":
        self.addtoBookmarks(dok)
      elif finalSelection == "Zobrazit obsah časopisu":
        if self.libraryType == "JEDENSVET":
          flist = self.getListofArticlesJedensvet(dok)
          flist, curIndex = self.markReadArticleJedensvet(flist, dok)
          self.resultsTree = dok
          self.currentLevel = 5
          self.fillmainList(flist)
          if curIndex != -1:
            self.mainList.Selection = curIndex
          return
        elif self.libraryType == "ZORAMP3":
          flist = self.getListofArticlesZoraMp3(dok)
          flist, curIndex = self.markReadArticleZora(flist, dok)
          self.resultsTree = dok
          self.currentLevel = 5
          self.fillmainList(flist)
          if curIndex != -1:
            self.mainList.Selection = curIndex
          return
        elif self.libraryType == "KDD":
          flist = self.getListofArticlesZoraKddMp3(dok)
          flist, curIndex = self.markReadArticleKddMp3(flist, dok)
          self.resultsTree = dok
          self.currentLevel = 5
          self.fillmainList(flist)
          if curIndex != -1:
            self.mainList.Selection = curIndex
          return
      elif finalSelection.startswith("Další tituly od autora"):
        ui.message("Načítám.")
        if self.libraryType == "KTNMP3":
          for dok in self.treeKtn.iter("record"):
            ctrlField = dok.find("controlfield[@tag='001']")
            if ctrlField.text[:2] == "MP":
              autorstr = self.ktnGetAuthors(dok)
              if autorstr == dokAutor:
                self.resultsTree.append(dok)
                titleField = dok.find("datafield[@tag='245']")
                sbField = titleField.find("subfield[@code='a']")
                dokitem = sbField.text
                if dokitem is not None:
                  dokstr = dokitem
                  autorstr = self.ktnGetAuthors(dok)
                  if autorstr:
                    dokstr = dokstr + " - " + autorstr
                  flist.append(dokstr)
          self.currentLevel = 4
          self.fillmainList(flist)
          return
        elif self.libraryType == "KTNTXT":
          for dok in self.treeKtn.iter("record"):
            ctrlField = dok.find("controlfield[@tag='001']")
            if ctrlField.text[0] == "D" and ctrlField.text[1].isnumeric():
              autorstr = self.ktnGetAuthors(dok)
              if autorstr == dokAutor:
                self.resultsTree.append(dok)
                titleField = dok.find("datafield[@tag='245']")
                sbField = titleField.find("subfield[@code='a']")
                dokitem = sbField.text
                if dokitem is not None:
                  dokstr = dokitem
                  autorstr = self.ktnGetAuthors(dok)
                  if autorstr:
                    dokstr = dokstr + " - " + autorstr
                  flist.append(dokstr)
          self.currentLevel = 4
          self.fillmainList(flist)
          return
        elif self.libraryType == "SPVK":
          for dok in self.treeSpvk.iter("polozka"):
            try:
              autorstr = self.spvkGetAuthors(dok)
              if dokAutor in autorstr:
                self.resultsTree.append(dok)
                dokitem = dok.find("nazev").text
                if dokitem is not None:
                  dokstr = dokitem
                  autorstr = self.spvkGetAuthors(dok)
                  if autorstr:
                    dokstr = dokstr + " - " + autorstr
                  flist.append(dokstr)
            except: pass
          self.currentLevel = 4
          self.fillmainList(flist)
          return
        elif self.libraryType == "Mathilda":
          if self.browsePath[0] == "Filmy Mathilda":
            for dok in self.treeMathildaMovies.iter("film"):
              try:
                autorstr = self.mathildaGetAuthors(dok)
                if dokAutor in autorstr:
                  self.resultsTree.append(dok)
                  dokitem = dok.find("title").text
                  if dokitem is not None:
                    dokstr = dokitem
                    autorstr = self.mathildaGetAuthors(dok)
                    if autorstr:
                      dokstr = dokstr + " - " + autorstr
                    flist.append(dokstr)
              except: pass
            self.currentLevel = 4
            self.fillmainList(flist)
            return
          else:
            for dok in self.treeMathildaBooks.iter("book"):
              try:
                autorstr = self.mathildaGetAuthors(dok)
                if dokAutor in autorstr:
                  self.resultsTree.append(dok)
                  dokitem = dok.find("title").text
                  if dokitem is not None:
                    dokstr = dokitem
                    autorstr = self.mathildaGetAuthors(dok)
                    if autorstr:
                      dokstr = dokstr + " - " + autorstr
                    flist.append(dokstr)
              except: pass
            self.currentLevel = 4
            self.fillmainList(flist)
            return
        else:
          for dok in self.tree.iter("dokument"):
            try:
              if dokAutor in dok.find("autori").text:
                self.resultsTree.append(dok)
                dokitem = dok.find("nazev").text
                if dokitem is not None:
                  dokstr = dokitem
                  dokitem = dok.find("autori").text
                  if dokitem is not None:
                    dokstr = dokstr + " - " + dokitem
                  # when function is called from search results, the category has to be added
                  if self.browsePath[0] == "Hledání":
                    dokkat = dok.find("kategorie_dokumentu_nazev_cs").text
                    dokstr = dokstr + "; " + dokkat
                  flist.append(dokstr)
            except: pass
          self.currentLevel = 4
          self.fillmainList(flist)
          return
      elif finalSelection.startswith("Zobrazit celou sérii"):
        ui.message("Načítám.")
        for dok in self.tree.iter("dokument"):
          try:
            if dokEdice in dok.find("edice_nazev_cs").text:
              if dokAutor == "" or dokAutor in dok.find("autori").text:
                self.resultsTree.append(dok)
                dokitem = dok.find("nazev").text
                if dokitem is not None:
                  dokstr = dokitem
                  dokitem = dok.find("poradi_edice").text
                  if dokitem is not None:
                    dokstr = dokitem + " " + dokstr
                  dokitem = dok.find("autori").text
                  if dokitem is not None:
                    dokstr = dokstr + " - " + dokitem
                  # when function is called from search results, the category has to be added
                  if self.browsePath[0] == "Hledání":
                    dokkat = dok.find("kategorie_dokumentu_nazev_cs").text
                    dokstr = dokstr + "; " + dokkat
                  flist.append(dokstr)
          except: pass
        self.currentLevel = 4
        self.fillmainList(flist)
        return
      elif finalSelection == "Číst":
        if not self.libraryType or self.libraryType == "KDD":
          if dok.find("kategorie_dokumentu_nazev_cs").text == "časopis":
            flist = self.getMagazineContent(dok)
            self.resultsTree = dok
            self.currentLevel = 5
            self.fillmainList(flist)
            return
        self.currentIndex = self.mainList.GetSelection()
        self.dokumentRead(dok)
      elif finalSelection == "Přehrát":
        self.currentIndex = self.mainList.GetSelection()
        self.dokumentPlay(dok)
      elif finalSelection == "Stáhnout titul" or finalSelection == "Stáhnout titul jako text" or finalSelection == "Stáhnout titul jako zvuk" or finalSelection.startswith("Znovu stáhnout titul"):
        if self.libraryType == "JEDENSVET":
          self.dokumentDownloadJedensvet(dok)
        elif self.libraryType == "ZORAMP3":
          self.dokumentDownloadZoraMp3(dok)
        elif self.libraryType == "ZORATXT":
          self.dokumentDownloadZoraText(dok)
        elif self.libraryType == "KTNMP3" or self.libraryType == "KTNTXT":
          if self.dokumentAccessKtn():
            self.dokumentDownloadKtn(dok)
        elif self.libraryType == "SPVK":
          if self.dokumentAccessSpvk():
            self.dokumentDownloadSpvk(dok)
        elif self.libraryType == "Mathilda":
          if self.dokumentAccessMathilda():
            self.dokumentDownloadMathilda(dok)
        else:
          if self.dokumentAccess(dok):
            if self.dokumentDownload(dok):
              self.dokumentUnzip(dok)
              global kddCredits
              kddCredits = int(kddCredits) - self.dokumentPrice(dok)
              if kddCredits < 0:
                kddCredits = 0
          return
      elif finalSelection == "Informace o titulu":
        self.titleInfo(dok)
      elif finalSelection == "Smazat titul":
        self.dokumentDelete(dok)
    if self.currentLevel == 5:
      if self.libraryType == "JEDENSVET":
        curTitle = self.mainList.StringSelection.split(", přehráno")[0]
        from os.path import expanduser
        home = expanduser("~")
        dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderNameJedensvet(self.resultsTree))
        bFile = os.path.join(dokFolder, self.okoMarkFilename)
        bTree = ET.parse(bFile)
        bDokument = bTree.getroot()
        fplay = bDokument.find("filetoread")
        fposition = bDokument.find("position")
        if fplay.text != curTitle:
          fposition.text = "0"
          fplay.text = curTitle
        self.dokumentPlay(bDokument)
        flist = self.getListofArticlesJedensvet(bDokument)
        flist, curIndex = self.markReadArticleJedensvet(flist, bDokument)
        self.fillmainList(flist)
        self.mainList.Selection = curIndex
      elif self.libraryType == "ZORAMP3":
        idTitle = self.mainList.StringSelection.split(", přehráno")[0]
        from os.path import expanduser
        home = expanduser("~")
        dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderNameZora(self.resultsTree))
        curTitle = self.getFilenameforId3(idTitle, dokFolder)
        bFile = os.path.join(dokFolder, self.okoMarkFilename)
        bTree = ET.parse(bFile)
        bDokument = bTree.getroot()
        fplay = bDokument.find("filetoread")
        fposition = bDokument.find("position")
        if fplay.text != curTitle:
          fposition.text = "0"
          fplay.text = curTitle
        self.dokumentPlay(bDokument)
        flist = self.getListofArticlesZoraMp3(bDokument)
        flist, curIndex = self.markReadArticleZora(flist, bDokument)
        self.fillmainList(flist)
        self.mainList.Selection = curIndex
      elif self.libraryType == "KDD":
        idTitle = self.mainList.StringSelection.split(", přehráno")[0]
        from os.path import expanduser
        home = expanduser("~")
        dokFolder = os.path.join(home, self.kddLibraryFolder, self.dokumentFolderName(self.resultsTree) + " MP3")
        curTitle = self.getFilenameforId3(idTitle, dokFolder)
        bFile = os.path.join(dokFolder, self.okoMarkFilename)
        bTree = ET.parse(bFile)
        bDokument = bTree.getroot()
        fplay = bDokument.find("filetoread")
        fposition = bDokument.find("position")
        if fplay.text != curTitle:
          fposition.text = "0"
          fplay.text = curTitle
        self.dokumentPlay(bDokument)
        flist = self.getListofArticlesZoraKddMp3(bDokument)
        flist, curIndex = self.markReadArticleKddMp3(flist, bDokument)
        self.fillmainList(flist)
        self.mainList.Selection = curIndex
      else:
        artFileName = self.getMagArticleFile(self.resultsTree, self.mainList.StringSelection)
        self.magArticleRead(self.resultsTree, artFileName)

  def levelRoot(self):
    if self.startupMyLibrary:
      self.initbrowsePath()
      self.browsePath[0] = "Moje knihovna"
      self.displayMyLibrary()
    else:
      self.initbrowsePath()
      self.currentLevel = 0
      self.addLevel = -1
      self.mainList.Clear()
      self.mainList.InsertItems(self.categories, 0)
      self.mainList.Selection = 0

  def levelUp(self):
    flist = []
    if self.currentLevel > 1 and self.browsePath[0] != "Hledání" and self.browsePath[0] != "Moje knihovna" and self.browsePath[0] != "Záložky" and self.browsePath[0] != "Jeden svět (Mathilda)":
      ui.message("Načítám.")
    if self.currentLevel == 0:
      return
    if self.browsePath[0].startswith("Nové tituly"):
      selCatPos = 1
    else:
      selCatPos = 0
    if self.browsePath[selCatPos].startswith("Knihy"):
      self.selCat = "kniha"
    elif self.browsePath[selCatPos].startswith("Časopisy"):
      self.selCat = "časopis"
    elif self.browsePath[selCatPos].startswith("Texty"):
      self.selCat = "text"
    elif self.browsePath[selCatPos].startswith("Zora"):
      self.selCat = "casopis_zora"
    elif self.browsePath[selCatPos].startswith("Zvukové dokumenty"):
      self.selCat = "audio_dokument"
    if self.currentLevel == 5:
      self.currentLevel = 4
      flist = self.myLibrary()
      self.fillmainList(flist)
      self.setmainListPosition(4)
      self.initbrowsePath()
      self.browsePath[0] = "Moje knihovna"
      return
    if self.currentLevel == 4:
      if self.browsePath[0] == "Jeden svět (Mathilda)":
        self.currentLevel = 0
        self.fillmainList(self.categories)
        self.setmainListPosition(0)
        self.initbrowsePath()
        return
      if self.browsePath[0] == "Zora MP3":
        # successful login string
        indexActive = self.zoraMp3Page.find("Aktivovány máte tyto časopisy:")
        endIndex = self.zoraMp3Page.find("   ", indexActive)
        # starting index of the active magazines
        indexActive = self.zoraMp3Page.find("\n",indexActive)+1
        # string with active magazines
        aktCasopisy = self.zoraMp3Page[indexActive:endIndex].strip()
        # list of active magazines
        aktCasopisyList = aktCasopisy.split(",")
        # stripped list of active magazines
        aktCasopisyList = [s.strip() for s in aktCasopisyList]
        # list of active magazines without empty items
        aktCasopisyList = list(filter(None, aktCasopisyList))
        # old magazines are not listed under active magazines, so we add them here
        aktCasopisyList.append("Technická revue")
        aktCasopisyList.append("Vstupte")
        aktCasopisyList.append("Masér")
        self.currentLevel = 1
        self.fillmainList(aktCasopisyList)
        self.setmainListPosition(1)
        self.browsePath[1] = ""
        return
      if self.browsePath[0] == "Zora Texty":
        today = datetime.date.today()
        curYear = today.year
        if self.zoraTextyLogged:
          stopYear = curYear
        else:
          stopYear = curYear -1
        flist = [str(x) for x in range(2000, stopYear+1)]
        self.currentLevel = 2
        self.fillmainList(flist)
        self.setmainListPosition(2)
        self.browsePath[2] = ""
        return
      if self.browsePath[0] == "MP3 KTN":
        if self.browsePath[2] == "":
          self.currentLevel = 2
        else:
          catLevel = 2
          for dok in self.treeKtn.iter("record"):
            ctrlField = dok.find("controlfield[@tag='001']")
            if ctrlField.text[:2] == "MP":
              for dtField in dok.findall("datafield[@tag='653']"):
                if dtField.attrib["ind1"] == str(catLevel-1) and dtField.find("subfield").text == self.browsePath[1]:
                  for dtField2 in dok.findall("datafield[@tag='653']"):
                    if dtField2.attrib["ind1"] == str(catLevel):
                      sbField = dtField2.findall("subfield[@code='a']")
                      for cat in sbField:
                        dokNazev = cat.text
                        if dokNazev is not None:
                          flist.append(dokNazev)
          self.currentLevel = 2
          self.fillmainList(flist)
          self.setmainListPosition(2)
          self.browsePath[2] = ""
          return
      if self.browsePath[0] == "Texty KTN":
        if self.browsePath[2] == "":
          self.currentLevel = 2
        else:
          catLevel = 2
          for dok in self.treeKtn.iter("record"):
            ctrlField = dok.find("controlfield[@tag='001']")
            if ctrlField.text[0] == "D" and ctrlField.text[1].isnumeric():
              for dtField in dok.findall("datafield[@tag='653']"):
                if dtField.attrib["ind1"] == str(catLevel-1) and dtField.find("subfield").text == self.browsePath[1]:
                  for dtField2 in dok.findall("datafield[@tag='653']"):
                    if dtField2.attrib["ind1"] == str(catLevel):
                      sbField = dtField2.findall("subfield[@code='a']")
                      for cat in sbField:
                        dokNazev = cat.text
                        if dokNazev is not None:
                          flist.append(dokNazev)
          self.currentLevel = 2
          self.fillmainList(flist)
          self.setmainListPosition(2)
          self.browsePath[2] = ""
          return
      if self.browsePath[0] == "Knihy SPVK":
        for dok in self.treeSpvk.iter("kategorie"):
          dokNazev = dok.find("nazev_kategorie").text
          if dokNazev is not None:
            flist.append(dokNazev)
        self.currentLevel = 1
        self.fillmainList(flist)
        self.setmainListPosition(1)
        self.browsePath[1] = ""
        return
      if self.browsePath[0] == "Knihy Mathilda":
        for dok in self.treeMathildaBooks.iter("book"):
          dokitem = dok.find("category").text
          if dokitem is not None:
            if "/" in dokitem:
              for x in dokitem.split("/"):
                flist.append(x.strip())
            else:
              flist.append(dokitem)
        self.currentLevel = 1
        self.fillmainList(flist)
        self.setmainListPosition(1)
        return
      if self.browsePath[0] == "Filmy Mathilda":
        if self.addLevel == 0:
          for dok in self.treeMathildaMovies.iter("film"):
            dokitem = dok.find("category").text
            if dokitem is not None:
              for i in dokitem.split("/"):
                if ">" in i:
                  igList = [i.strip() for i in i.split(">")]
                  if len(igList) > 3 and self.browsePath[1] == igList[0] and self.browsePath[2] == igList[1] and self.browsePath[3] == igList[2]:
                    flist.append(igList[3].strip())
          if flist:
            for x in flist:
              if ">" in x:
                flist[flist.index(x)] = x.split(">")[0]
            self.currentLevel = 3
            self.fillmainList(flist)
            self.setmainListPosition(0)
            self.addPath[0] = ""
            return
        if self.browsePath[3] == "":
          self.currentLevel = 3
        else:
          for dok in self.treeMathildaMovies.iter("film"):
            dokitem = dok.find("category").text
            if dokitem is not None:
              for i in dokitem.split("/"):
                if ">" in i:
                  igList = [i.strip() for i in i.split(">")]
                  if len(igList) > 2 and self.browsePath[1] == igList[0] and (self.browsePath[2] == "Vše" or self.browsePath[2] == igList[1]):
                    flist.append(igList[2].strip())
          if flist:
            for x in flist:
              if ">" in x:
                flist[flist.index(x)] = x.split(">")[0]
            self.currentLevel = 3
            self.fillmainList(flist)
            self.setmainListPosition(3)
            self.browsePath[3] = ""
            return
      if self.browsePath[0] == "Knihy" or self.browsePath[0] == "Časopisy" or self.browsePath[0] == "Texty" or self.browsePath[0] == "Zora" or self.browsePath[0] == "Zvukové dokumenty":
        if self.addLevel == 0:
          for dok in self.tree.iter("dokument"):
            try:
              if self.selCat in dok.find("kategorie_dokumentu_nazev_cs").text and self.browsePath[1] in dok.find("typ_dokumentu_nazev_cs").text:
                if self.browsePath[2] == "" or self.browsePath[2] == "Vše" or self.browsePath[2] in dok.find("predmetove_skupiny").text.strip().lower():
                  if self.browsePath[3] == "" or self.browsePath[3] == "Vše" or self.browsePath[3] in dok.find("formy").text.strip().lower():
                    dokitem = dok.find("serie").text
                    if dokitem is not None:
                      if ";" in dokitem:
                        for x in dokitem.split(";"):
                          flist.append(x.strip().lower())
                      else:
                        flist.append(dokitem.strip().lower())
            except: pass
          self.currentLevel = 3
          self.fillmainList(flist)
          self.setmainListPosition(0)
          self.addPath[0] = ""
          return
        if self.browsePath[3] == "":
          self.currentLevel = 3
        else:
          for dok in self.tree.iter("dokument"):
            try:
              if self.selCat in dok.find("kategorie_dokumentu_nazev_cs").text:
                if self.browsePath[1] == "" or self.browsePath[1] in dok.find("typ_dokumentu_nazev_cs").text:
                  if self.browsePath[2] == "" or self.browsePath[2] in dok.find("predmetove_skupiny").text.strip().lower():
                    dokitem = dok.find("formy").text
                    if dokitem is not None:
                      if ";" in dokitem:
                        for x in dokitem.split(";"):
                          flist.append(x.strip().lower())
                      else:
                        flist.append(dokitem.strip().lower())
            except: pass
          self.currentLevel = 3
          self.fillmainList(flist)
          self.setmainListPosition(3)
          self.browsePath[3] = ""
          return
      elif self.browsePath[0] == "Hledání" or self.browsePath[0] == "Moje knihovna" or self.browsePath[0] == "Záložky":
        self.currentLevel = 0
        flist = self.categories
        self.fillmainList(flist)
        self.setmainListPosition(0)
        self.initbrowsePath()
        return
      elif self.browsePath[0].startswith("Nové tituly"):
        flist = self.categoriesNews
        self.currentLevel = 1
        self.setmainListPosition(1)
        self.fillmainList(flist)
        return
      elif self.browsePath[0].startswith("Novinky KTN"):
        flist = self.categoriesNewsKtn
        self.currentLevel = 1
        self.setmainListPosition(1)
        self.fillmainList(flist)
        return
    if self.currentLevel == 3:
      if self.browsePath[0] == "Filmy Mathilda":
        if self.addLevel == 0:
          for dok in self.treeMathildaMovies.iter("film"):
            dokitem = dok.find("category").text
            if dokitem is not None:
              for i in dokitem.split("/"):
                if ">" in i:
                  igList = [i.strip() for i in i.split(">")]
                  if len(igList) > 2 and self.browsePath[1] == igList[0] and self.browsePath[2] == igList[1]:
                    flist.append(igList[2].strip())
          if flist:
            for x in flist:
              if ">" in x:
                flist[flist.index(x)] = x.split(">")[0]
            self.addLevel = -1
            self.fillmainList(flist)
            self.setmainListPosition(3)
            self.browsePath[3] = ""
            return
        if self.browsePath[2] == "":
          self.currentLevel = 2
        else:
          for dok in self.treeMathildaMovies.iter("film"):
            dokitem = dok.find("category").text
            if dokitem is not None:
              for i in dokitem.split("/"):
                if ">" in i:
                  if self.browsePath[1] == i.split(">")[0]:
                    flist.append(i.split(">")[1].strip())
          if flist:
            for x in flist:
              if ">" in x:
                flist[flist.index(x)] = x.split(">")[0]
            self.currentLevel = 2
            self.fillmainList(flist)
            self.setmainListPosition(2)
            self.browsePath[2] = ""
            return
      if self.browsePath[0] == "Knihy" or self.browsePath[0] == "Texty" or self.browsePath[0] == "Zvukové dokumenty":
        if self.addLevel == 0:
          self.addLevel = -1
        if self.browsePath[2] == "":
          self.currentLevel = 2
        else:
          for dok in self.tree.iter("dokument"):
            try:
              if self.selCat in dok.find("kategorie_dokumentu_nazev_cs").text and self.browsePath[1] in dok.find("typ_dokumentu_nazev_cs").text:
                dokitem = dok.find("predmetove_skupiny").text
                if dokitem is not None:
                  if ";" in dokitem:
                    for x in dokitem.split(";"):
                      flist.append(x.strip().lower())
                  else:
                    flist.append(dokitem.strip().lower())
            except: pass
          self.currentLevel = 2
          self.fillmainList(flist)
          self.setmainListPosition(2)
          self.browsePath[2] = ""
          return
      if self.browsePath[0] == "Časopisy" or self.browsePath[0] == "Zora":
        self.currentLevel = 2
        if self.browsePath[1] == "Archiv starších časopisů":
          self.fillmainList(self.magArchiveList)
          self.setmainListPosition(2)
          self.browsePath[2] = ""
          return
        self.browsePath[2] = ""
    if self.currentLevel == 2:
      if self.browsePath[0] == "Zora Texty":
              # login is successful string
        indexLogin = self.zoraTextyPage.find("Přihlášený uživatel:")
        # when user is not logged
        if indexLogin == -1:
          self.zoraTextyLogged = False
          indexLogin = 0
        else:
          self.zoraTextyLogged = True
        startMagIndex = self.zoraTextyPage.find("Časopis:",indexLogin)
        startMagIndex = self.zoraTextyPage.find("\n", startMagIndex)+1
        magPageList = self.zoraTextyPage[startMagIndex:].splitlines()
        magInserted = False
        flist = []
        for s in magPageList:
          if not s.strip() and magInserted:
            break
          if s.strip():
            magInserted = True
            flist.append(s.strip())
        self.currentLevel = 1
        self.fillmainList(flist)
        self.setmainListPosition(1)
        self.browsePath[1] = ""
        return
      if self.browsePath[0] == "MP3 KTN":
        catLevel = 1
        for dok in self.treeKtn.iter("record"):
          ctrlField = dok.find("controlfield[@tag='001']")
          if ctrlField.text[:2] == "MP":
            for dtField in dok.findall("datafield[@tag='653']"):
              if dtField.attrib["ind1"] == str(catLevel):
                sbField = dtField.findall("subfield[@code='a']")
                for cat in sbField:
                  dokNazev = cat.text
                  if dokNazev is not None:
                    flist.append(dokNazev)
        self.currentLevel = 1
        self.fillmainList(flist)
        self.setmainListPosition(1)
        self.browsePath[1] = ""
        return
      if self.browsePath[0] == "Texty KTN":
        catLevel = 1
        for dok in self.treeKtn.iter("record"):
          ctrlField = dok.find("controlfield[@tag='001']")
          if ctrlField.text[0] == "D" and ctrlField.text[1].isnumeric():
            for dtField in dok.findall("datafield[@tag='653']"):
              if dtField.attrib["ind1"] == str(catLevel):
                sbField = dtField.findall("subfield[@code='a']")
                for cat in sbField:
                  dokNazev = cat.text
                  if dokNazev is not None:
                    flist.append(dokNazev)
        self.currentLevel = 1
        self.fillmainList(flist)
        self.setmainListPosition(1)
        self.browsePath[1] = ""
        return
      if self.browsePath[0] == "Filmy Mathilda":
        for dok in self.treeMathildaMovies.iter("film"):
          dokitem = dok.find("category").text
          if dokitem is not None:
            if "/" in dokitem:
              for x in dokitem.split("/"):
                flist.append(x.strip())
            else:
              flist.append(dokitem)
        if flist:
          for x in flist:
            if ">" in x:
              flist[flist.index(x)] = x.split(">")[0]
        self.currentLevel = 1
        self.fillmainList(flist)
        self.setmainListPosition(1)
        self.browsePath[1] = ""
        return
      if self.browsePath[0] == "Knihy" or self.browsePath[0] == "Texty" or self.browsePath[0] == "Zvukové dokumenty":
        for dok in self.tree.iter("dokument"):
          try:
            if self.selCat in dok.find("kategorie_dokumentu_nazev_cs").text:
              dokitem = dok.find("typ_dokumentu_nazev_cs").text
              if dokitem is not None:
                if ";" in dokitem:
                  for x in dokitem.split(";"):
                    flist.append(x)
                else:
                  flist.append(dokitem)
          except: pass
        self.currentLevel = 1
        self.fillmainList(flist)
        self.setmainListPosition(1)
        self.browsePath[1] = ""
        return
      if self.browsePath[0] == "Časopisy" or self.browsePath[0] == "Zora":
        self.magArchiveList = []
        curYear = datetime.datetime.now().year
        for dok in self.tree.iter("dokument"):
          if self.selCat in dok.find("kategorie_dokumentu_nazev_cs").text:
            dokYear = dok.find("rok_vydani").text
            try:
              dokYearNum = int(dokYear)
            except:
              dokYearNum = 0
            if dokYearNum > (curYear -2):
              dokitem = dok.find("typ_dokumentu_nazev_cs").text
              if dokitem is not None:
                if ";" in dokitem:
                  for x in dokitem.split(";"):
                    flist.append(x)
                else:
                  flist.append(dokitem)
            else:
              dokitem = dok.find("typ_dokumentu_nazev_cs").text
              if dokitem is not None:
                if ";" in dokitem:
                  for x in dokitem.split(";"):
                    self.magArchiveList.append(x)
                else:
                  self.magArchiveList.append(dokitem)
        if self.magArchiveList:
          # difference between all magazines (magArchiveList) and latest magazines (flist)
          self.magArchiveList = list(set(self.magArchiveList) - set(flist))
          flist.append("Archiv starších časopisů")
        self.currentLevel = 1
        self.fillmainList(flist)
        self.setmainListPosition(1)
        self.browsePath[1] = ""
        return
    if self.currentLevel == 1:
      self.currentLevel = 0
      self.fillmainList(self.categories)
      self.setmainListPosition(0)
      self.initbrowsePath()
      return

  def Download_file(self, url, target):
    try:
      with urllib.request.urlopen(url, timeout=30) as res:
        with open(target, "wb") as out:
          out.write(res.read())
    except:
      res = False
      errorMessage = _(u"Při stahování došlo k chybě.")
      errorTitle = _(u"Upozornění")
      wx.MessageBox(errorMessage, errorTitle)
      self.passKey = True
    return res

  def dokumentInfoItem(self, evt):
    if evt != None:
      self.passKey = True
    if self.currentLevel == 4:
      dok = self.findCurrentTitle()
      self.titleInfo(dok)

  def sayDokumentAnotaceItem(self, evt):
    if evt != None:
      self.passKey = True
    if self.currentLevel == 4:
      dok = self.findCurrentTitle()
      self.sayDokumentAnotace(dok)

  def kddSearchItem(self, evt):
    self.kddSearch()

  def whereamiItem(self, evt):
    if evt != None:
      self.passKey = True
    ui.message(self.whereAmI())

  def myLibraryItem(self, evt):
    if evt != None:
      self.passKey = True
    self.initbrowsePath()
    self.browsePath[0] = "Moje knihovna"
    self.displayMyLibrary()

  def userInfoItem(self, evt):
    self.userInfo()

  def levelRootItem(self, evt):
    if evt != None:
      self.passKey = True
    if not self.startupMyLibrary or (self.startupMyLibrary and self.currentLevel == 5):
      self.levelRoot()

  def jedensvetNormalizeName(self, magname):
    l = magname.split()
    res = l[3] + ", Jeden svět " + l[2].zfill(2) 
    return res

  def onmainListKeyUp(self, evt):
    key = evt.GetKeyCode()
    if self.passKey:
      self.passKey = False
      return
    if key == wx.WXK_RETURN or key == wx.WXK_NUMPAD_ENTER:
      if not self.allowEnter:
        self.allowEnter = True
        return
      if evt.AltDown():
        if self.currentLevel == 4:
          dok = self.findCurrentTitle()
          self.titleInfo(dok)
      else:
        self.onmainListChoice(self)
    elif key == wx.WXK_BACK or key == wx.WXK_ESCAPE:
      if not self.startupMyLibrary or (self.startupMyLibrary and self.currentLevel == 5):
        self.levelUp()
    elif key == wx.WXK_DELETE:
      if self.browsePath[0] == "Moje knihovna" and self.currentLevel == 4:
        dok = self.findCurrentTitle()
        self.dokumentDelete(dok)
    evt.Skip()

  def onClose(self, evt):
    self.Destroy()
    KddMainDialog._instance = None


class WhatToDoDialog(wx.Dialog):

  global finalSelection
  selectionItems = []
  finalSelection = ""
  titleDlg = ""
  passkey = False
  sposition = 0

  def __init__(self, parent, pkey=False):

    self.passkey = pkey
    # The title of a dialog
    super(WhatToDoDialog, self).__init__(parent, size=(600, 400), title=self.titleDlg)

    self.Bind(wx.EVT_CLOSE, self.onClose)
    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self,orientation=wx.VERTICAL)

    # listbox
    self.choices = []

    # assign choices
    self.choices = self.selectionItems

    self.choicesList = wx.ListBox(self, choices=self.choices)
    self.choicesList.Selection = self.sposition
    self.choicesList.Bind(wx.EVT_LISTBOX_DCLICK, self.onchoicesListChoice)
    self.choicesList.Bind(wx.EVT_KEY_UP, self.onchoicesListKeyUp)
    self.choicesList.SetMinSize((600,400));
    sHelper.addItem(self.choicesList, flag = wx.EXPAND)

    mainSizer.Add(sHelper.sizer, flag=wx.ALL, border=guiHelper.BORDER_FOR_DIALOGS)
    self.Sizer = mainSizer

    self.choicesList.SetFocus()
    self.CentreOnScreen()

  def __del__(self):
    WhatToDoDialog._instance = None

  def onchoicesListChoice(self, evt):
    global finalSelection
    finalSelection = self.choicesList.StringSelection
    self.onClose(self)

  def onchoicesListKeyUp(self, evt):
    global finalSelection
    if self.passkey:
      self.passkey = False
      return
    key = evt.GetKeyCode()
    if key == wx.WXK_RETURN or key == wx.WXK_NUMPAD_ENTER:
      self.onchoicesListChoice(self)
    elif key == wx.WXK_BACK or key == wx.WXK_ESCAPE:
      finalSelection = ""
      self.onClose(self)
    evt.Skip()

  def onClose(self, evt):
    self.Parent.Enable()
    if "TextReader" in str(self.GetParent()):
      self.Parent.outputCtrl.SetFocus()
    elif "AudioPlayer" in str(self.GetParent()):
      self.Parent.panel.SetFocus()
    else:
      self.Parent.mainList.SetFocus()
    self.Destroy()


class TextReader(wx.Frame):

  dispText = ""
  dlgTitle = ""
  titleEncoding = ""
  cursorPos = 0
  isReader = False
  dokPath = ""
  sayAllRunning = False
  utfEncoding = True
  findData = None
  passKey = False
  libType = ""

  def __init__(self, parent):
    # The title of the details dialog
    super(TextReader, self).__init__(parent, size=(1000, 800), title=self.dlgTitle)
    self.Bind(wx.EVT_CLOSE, self.onClose)
    self.Bind(wx.EVT_FIND, self.onFind)
    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self, orientation=wx.VERTICAL)

    if self.isReader:
      # the menu bar
      menuBar = wx.MenuBar()
      menu = wx.Menu()
      
      # menu items
      item = menu.Append(wx.ID_ANY, "Spustit nebo zastavit plynulé čtení\tSpace")
      self.Bind(wx.EVT_MENU, self.startstopReading, item)
      item = menu.Append(wx.ID_ANY, "Seznam &kapitol\tF4")
      self.Bind(wx.EVT_MENU, self.titleContent, item)
      item = menu.Append(wx.ID_ANY, "Hledat\tCtrl+F")
      self.Bind(wx.EVT_MENU, self.searchText, item)
      item = menu.Append(wx.ID_ANY, "Najít další\tF3")
      self.Bind(wx.EVT_MENU, self.findNext, item)
      item = menu.Append(wx.ID_ANY, "Najít předchozí\tShift+F3")
      self.Bind(wx.EVT_MENU, self.findPrevious, item)
      item = menu.Append(wx.ID_ANY, "Další soubor\tCtrl+Space")
      self.Bind(wx.EVT_MENU, self.gotonextReadingFile, item)
      item = menu.Append(wx.ID_ANY, "Předchozí soubor\tCtrl+Backspace")
      self.Bind(wx.EVT_MENU, self.gotoprevReadingFile, item)
      menu.AppendSeparator()
      item = menu.Append(wx.ID_EXIT, "&Zavřít")
      self.Bind(wx.EVT_MENU, self.onCloseItem, item)
      
      # the menu title
      menuBar.Append(menu, "Čtečka")
      self.SetMenuBar(menuBar)

    # The details edit field
    self.outputCtrl = wx.TextCtrl(self, wx.ID_ANY, style=wx.TE_MULTILINE | wx.TE_READONLY)
    self.outputCtrl.Bind(wx.EVT_KEY_UP, self.onOutputKeyUp)
    sHelper.addItem(self.outputCtrl, proportion=1, flag=wx.EXPAND)

    if self.isReader:
      self.loadReadingFile(self.dispText)
      # move cursor to the last reading position
      self.jumptoPlace(self.cursorPos)
    else:
      self.outputCtrl.SetValue(self.dispText)
    self.outputCtrl.SetFocus()
    # automatically starts to read
    if self.isReader:
      timer = threading.Timer(0.500, self.startReading)
      timer.start()

  def loadReadingFile(self, fileName):
    self.outputCtrl.LoadFile(fileName)
    if os.path.splitext(fileName)[1] == ".rtf":
      # convert from RTF to plain text
      self.outputCtrl.SetValue(striprtf.rtf_to_text(self.outputCtrl.GetValue()))
    self.utfEncoding = True
    if self.titleEncoding:
      if "DETECT" in self.titleEncoding:
        cFile = open(fileName, 'rb')
        fileContent = cFile.read()
        cFile.close()
        try:
          fileContent.decode('UTF-8')
        except UnicodeDecodeError:
          self.utfEncoding = False
        else:
          self.utfEncoding = True
      # when attribute "kodovani" in catalog is not UTF-8, convert from windows-1250 to utf-8
      elif "UTF" not in self.titleEncoding:
        self.utfEncoding = False
    if not self.utfEncoding:
      self.outputCtrl.SetValue(self.outputCtrl.GetValue().encode('latin1', 'ignore').decode('windows-1250', 'ignore'))
      self.utfEncoding = False

  def startReading(self):
    speech.cancelSpeech()
    if not self.sayAllRunning:
      sayAll.SayAllHandler.readText(sayAll.CURSOR.CARET)
      self.sayAllRunning = True

  def saveReadingPosition(self):
    from os.path import expanduser
    home = expanduser("~")
    bookmarkFile = os.path.join(self.dokPath, KddMainDialog.okoMarkFilename)
    tree = ET.parse(bookmarkFile)
    dokument = tree.getroot()
    dokCursor = dokument.find("position")
    dokCursor.text = str(self.cursorPos)
    dokName = dokument.find("filetoread")
    dokName.text = os.path.basename(self.dispText)
    with open(bookmarkFile, 'wb') as f:
      f.write(ET.tostring(dokument, encoding='utf-8', method='xml'))

  def jumptoPlace(self, place):
    # move cursor to the found text position
    self.outputCtrl.SetInsertionPoint(place)
    # and scroll to this position
    self.outputCtrl.ShowPosition(place)
    self.outputCtrl.Refresh()
    self.outputCtrl.Update()

  def searchText(self, evt):
    if not self.isReader:
      return
    self.findData = wx.FindReplaceData()
    self.searchDlg = wx.FindReplaceDialog(self, self.findData, title="Hledání textu", style=wx.FR_NOUPDOWN | wx.FR_NOMATCHCASE | wx.FR_NOWHOLEWORD)
    self.searchDlg.Show()
    self.passKey = True

  def onFind(self, event):
    if not self.findData:
      self.searchText(None)
    else:
      fstring = self.findData.GetFindString().upper()
      self.searchDlg.Close()
      sText = self.outputCtrl.GetValue().upper()
      currentFindPos = self.outputCtrl.GetInsertionPoint()
      (par, charPos, lineNumber) = self.outputCtrl.PositionToXY(currentFindPos)
      i = lineNumber
      found = False
      while i <= self.outputCtrl.GetNumberOfLines():
        line = self.outputCtrl.GetLineText(i).upper()
        if i == lineNumber:
          startPos = line.find(fstring, charPos+1)
        else:
          startPos = line.find(fstring)
        if startPos != -1:
          found = True
          jumpPos = self.outputCtrl.XYToPosition(startPos, i)
          self.jumptoPlace(jumpPos)
          speech.speakMessage(line)
          break
        i = i +1
      if not found:
        errorMessage = "Text " + fstring + " nebyl nalezen."
        errorTitle = _(u"Upozornění")
        wx.MessageBox(errorMessage, errorTitle)
        self.passKey = True

  def findNext(self, evt):
    self.onFind(self)

  def findPrevious(self, evt):
    if not self.findData:
      self.searchText(None)
    else:
      fstring = self.findData.GetFindString().upper()
      sText = self.outputCtrl.GetValue().upper()
      currentFindPos = self.outputCtrl.GetInsertionPoint()
      (par, charPos, lineNumber) = self.outputCtrl.PositionToXY(currentFindPos)
      i = lineNumber
      found = False
      while i >= 0:
        line = self.outputCtrl.GetLineText(i).upper()
        if i == lineNumber:
          startPos = line.rfind(fstring, 0, charPos)
        else:
          startPos = line.rfind(fstring)
        if startPos != -1:
          found = True
          jumpPos = self.outputCtrl.XYToPosition(startPos, i)
          # move cursor to the found text position
          self.outputCtrl.SetInsertionPoint(jumpPos)
          # and scroll to this position
          self.outputCtrl.ShowPosition(jumpPos)
          self.outputCtrl.Refresh()
          self.outputCtrl.Update()
          speech.speakMessage(line)
          break
        i = i -1
      if not found:
        errorMessage = "Text " + fstring + " nebyl nalezen."
        errorTitle = _(u"Upozornění")
        wx.MessageBox(errorMessage, errorTitle)
        self.passKey = True

  def titleContent(self, evt):
    global finalSelection
    if not self.isReader:
      return
    WhatToDoDialog.selectionItems.clear()
    chapCounter = 0
    for i in range(self.outputCtrl.GetNumberOfLines()):
      line = self.outputCtrl.GetLineText(i)
      if line.startswith("#"):
        # character # can be on beginning of line more than once
        spPos = 0
        for spCh in line:
          if spCh == "#":
            spPos += 1
          else:
            break
        line = line[spPos:].strip()
        if not line:
          if self.libType == "ZORATXT" or self.libType == "KDD":
            line = self.outputCtrl.GetLineText(i+1)
          else:
            chapCounter = chapCounter + 1
            line = "Kapitola " + str(chapCounter)
        WhatToDoDialog.selectionItems.append(line)
    if WhatToDoDialog.selectionItems:
      WhatToDoDialog.selectionItems.append("Zpět")
    if WhatToDoDialog.selectionItems:
      if evt is not None:
        self.displayContent(True)
      else:
        self.displayContent()
      if finalSelection == "Zpět" or finalSelection == "":
        return
      lineChapCounter = 0
      if chapCounter > 0:
        try:
          chapIndex = int(finalSelection.split()[1])
        except:
          chapIndex = -1
      for i in range(self.outputCtrl.GetNumberOfLines()):
        line = self.outputCtrl.GetLineText(i)
        if line.startswith("#"):
          # character # can be on beginning of line more than once
          spPos = 0
          for spCh in line:
            if spCh == "#":
              spPos += 1
            else:
              break
          line = line[spPos:].strip()
          if not line:
            if self.libType == "ZORATXT" or self.libType == "KDD":
              line = self.outputCtrl.GetLineText(i+1)
            else:
              lineChapCounter = lineChapCounter + 1
          if line == finalSelection or (chapCounter > 0 and chapIndex == lineChapCounter):
            jumpPos = self.outputCtrl.XYToPosition(0, i)
            self.jumptoPlace(jumpPos)
            break
    else:
      errorMessage = _(u"V tomto titulu nebylo detekováno členění na kapitoly.")
      errorTitle = _(u"Upozornění")
      wx.MessageBox(errorMessage, errorTitle)
      self.passKey = True

  def displayContent(self, keypass=False):
    global finalSelection
    WhatToDoDialog.titleDlg = "Seznam kapitol"
    WhatToDoDialog.sposition = 0
    self.Disable()
    try:
      WhatToDoDialog(self, keypass).ShowModal()
    except Exception as e:
      self.Enable()
      raise e

  def startstopReading(self, evt):
    if self.sayAllRunning:
      self.sayAllRunning = False
      sayAll.SayAllHandler.stop()
    else:
      sayAll.SayAllHandler.readText(sayAll.CURSOR.CARET)
      self.sayAllRunning = True

  def gotonextReadingFile(self, evt):
    if not self.nextReadingFile():
      ui.message("Toto je poslední soubor.")
    else:
      ui.message(os.path.basename(self.dispText))
  
  def gotoprevReadingFile(self, evt):
    if not self.prevReadingFile():
      ui.message("Toto je první soubor.")
    else:
      ui.message(os.path.basename(self.dispText))

  def nextReadingFile(self):
    dirList = os.listdir(self.dokPath)
    try:
      dirList.remove(KddMainDialog.okoMarkFilename)
    except: pass
    fileName = os.path.basename(self.dispText)
    fileIndex = dirList.index(fileName)
    fileIndex = fileIndex +1
    if fileIndex < len(dirList):
      self.dispText = self.dispText.replace(fileName, dirList[fileIndex])
      self.loadReadingFile(self.dispText)
      return True
    else:
      return False

  def prevReadingFile(self):
    dirList = os.listdir(self.dokPath)
    try:
      dirList.remove(KddMainDialog.okoMarkFilename)
    except: pass
    fileName = os.path.basename(self.dispText)
    fileIndex = dirList.index(fileName)
    fileIndex = fileIndex -1
    if fileIndex >= 0:
      self.dispText = self.dispText.replace(fileName, dirList[fileIndex])
      self.loadReadingFile(self.dispText)
      return True
    else:
      return False

  def gotoLineStartingWith(self, ch):
    currentFindPos = self.outputCtrl.GetInsertionPoint()
    (par, charPos, lineNumber) = self.outputCtrl.PositionToXY(currentFindPos)
    i = lineNumber+1
    while i != lineNumber:
      if i > self.outputCtrl.GetNumberOfLines():
        i = 0
      line = self.outputCtrl.GetLineText(i).upper()
      if line and line[0] == ch:
        jumpPos = self.outputCtrl.XYToPosition(1, i)
        self.jumptoPlace(jumpPos)
        speech.speakMessage(line)
        break
      if i == lineNumber:
        break
      else:
        i = i +1

  def onOutputKeyUp(self, evt):
    key = evt.GetKeyCode()
    if self.passKey:
      self.passKey = False
      return
    if key in range(65, 91) and not self.isReader:
      self.gotoLineStartingWith(chr(key))
    elif key == wx.WXK_BACK:
      if not evt.ControlDown():
        self.onClose(self)
    elif key == wx.WXK_ESCAPE:
      self.onClose(self)
    elif key == wx.WXK_F5:
      if self.utfEncoding:
        self.utfEncoding = False
        self.outputCtrl.SetValue(self.outputCtrl.GetValue().encode('latin1', 'ignore').decode('windows-1250', 'ignore'))
        # move cursor to the last reading position
        self.outputCtrl.SetInsertionPoint(self.cursorPos)
        # and scroll to this position
        self.outputCtrl.ShowPosition(self.cursorPos)
        self.outputCtrl.Refresh()
        self.outputCtrl.Update()
        ui.message("Kódování Windows.")
    else:
      if key != wx.WXK_SPACE:
        self.sayAllRunning = False
    evt.Skip()

  def onCloseItem(self, evt):
    if evt != None:
      self.Parent.passKey = True
    self.onClose(self)

  def onClose(self, evt):
      if self.isReader:
        self.cursorPos = self.outputCtrl.GetInsertionPoint()
        self.saveReadingPosition()
      self.Parent.Enable()
      self.Parent.afterReadingOrPlayingFinish()
      self.Parent.Show()
      self.Destroy()


class AudioPlayer(wx.Frame):

  dlgTitle = ""
  playingPos = 0
  dokPath = ""
  source = ""
  currentVolume = 100
  currentRate = 1.0
  rateSpeech = {
    0.5: "Poloviční",
    0.75: "Tříčtvrteční",
    1.0: "Normální",
    1.25: "Jeden a čtvrtnásobná",
    1.5: "Jeden a půlnásobná",
    1.75: "Jeden a třičtvrtěnásobná",
    2.0: "Dvojnásobná"
  }
  if vlcInstalled:
    vlc_instance = vlc.Instance()
    player = vlc_instance.media_player_new()
    events = player.event_manager()

  def __init__(self, parent):
    # The title of the AudioPlayer dialog
    super(AudioPlayer, self).__init__(parent, size=(1000, 800), title=self.dlgTitle)
    self.Bind(wx.EVT_CLOSE, self.onClose)
    self.panel = wx.Panel(self)
    # standard self.Bind(wx.EVT_KEY_UP, self.audioPlayerKeyUp) does not work
    accel_tbl = wx.AcceleratorTable([
      (wx.ACCEL_NORMAL, wx.WXK_BACK, wx.ID_ANY),
      (wx.ACCEL_NORMAL, wx.WXK_ESCAPE, wx.ID_ANY)
    ])
    self.SetAcceleratorTable(accel_tbl)
    self.Bind(wx.EVT_MENU, self.audioPlayerKeyUp, id=wx.ID_ANY)
    
    # the menu bar
    menuBar = wx.MenuBar()
    menu = wx.Menu()
    
    # menu items
    item = menu.Append(wx.ID_ANY, "Spustit nebo zastavit &přehrávání\tSpace")
    self.Bind(wx.EVT_MENU, self.startstopPlaying, item)
    item = menu.Append(wx.ID_ANY, "Přesun na &začátek\tHome")
    self.Bind(wx.EVT_MENU, self.gotoStart, item)
    item = menu.Append(wx.ID_ANY, "Přesun na &konec\tEnd")
    self.Bind(wx.EVT_MENU, self.gotoEnd, item)
    item = menu.Append(wx.ID_ANY, "Ohlásit aktuální pozici\tF2")
    self.Bind(wx.EVT_MENU, self.announceCurrentPosition, item)
    item = menu.Append(wx.ID_ANY, "Seznam &souborů\tF4")
    self.Bind(wx.EVT_MENU, self.fileContent, item)
    item = menu.Append(wx.ID_ANY, "Další soubor\tCtrl+Space")
    self.Bind(wx.EVT_MENU, self.gotonextPlayingFile, item)
    item = menu.Append(wx.ID_ANY, "Předchozí soubor\tCtrl+Backspace")
    self.Bind(wx.EVT_MENU, self.gotoprevPlayingFile, item)
    item = menu.Append(wx.ID_ANY, "10 sekund vpřed\tRight")
    self.Bind(wx.EVT_MENU, lambda event: self.goForward(10000), item)
    item = menu.Append(wx.ID_ANY, "10 sekund zpět\tLeft")
    self.Bind(wx.EVT_MENU, lambda event: self.goBackward(10000), item)
    item = menu.Append(wx.ID_ANY, "1 minuta vpřed\tCtrl+Right")
    self.Bind(wx.EVT_MENU, lambda event: self.goForward(60000), item)
    item = menu.Append(wx.ID_ANY, "1 minuta zpět\tCtrl+Left")
    self.Bind(wx.EVT_MENU, lambda event: self.goBackward(60000), item)
    item = menu.Append(wx.ID_ANY, "3 sekundy vpřed\tShift+Right")
    self.Bind(wx.EVT_MENU, lambda event: self.goForward(3000), item)
    item = menu.Append(wx.ID_ANY, "3 sekundy zpět\tShift+Left")
    self.Bind(wx.EVT_MENU, lambda event: self.goBackward(3000), item)
    item = menu.Append(wx.ID_ANY, "10 minut vpřed\tAlt+Right")
    self.Bind(wx.EVT_MENU, lambda event: self.goForward(600000), item)
    item = menu.Append(wx.ID_ANY, "10 minut zpět\tAlt+Left")
    self.Bind(wx.EVT_MENU, lambda event: self.goBackward(600000), item)
    item = menu.Append(wx.ID_ANY, "5 minut vpřed\tCtrl+Alt+Right")
    self.Bind(wx.EVT_MENU, lambda event: self.goForward(300000), item)
    item = menu.Append(wx.ID_ANY, "5 minut zpět\tCtrl+Alt+Left")
    self.Bind(wx.EVT_MENU, lambda event: self.goBackward(300000), item)
    item = menu.Append(wx.ID_ANY, "Zesílit\tUp")
    self.Bind(wx.EVT_MENU, lambda event: self.changeVolume(5), item)
    item = menu.Append(wx.ID_ANY, "Zeslabit\tDown")
    self.Bind(wx.EVT_MENU, lambda event: self.changeVolume(-5), item)
    item = menu.Append(wx.ID_ANY, "Zrychlit\tCtrl+Up")
    self.Bind(wx.EVT_MENU, lambda event: self.changeRate(0.25), item)
    item = menu.Append(wx.ID_ANY, "Zpomalit\tCtrl+Down")
    self.Bind(wx.EVT_MENU, lambda event: self.changeRate(-0.25), item)
    menu.AppendSeparator()
    item = menu.Append(wx.ID_EXIT, "&Zavřít")
    self.Bind(wx.EVT_MENU, self.onCloseItem, item)
    
    # the menu title
    menuBar.Append(menu, "Přehrávač")
    self.SetMenuBar(menuBar)

    self.panel.SetFocus()
    menuBar.EnableTop(0, True)  
    self.media = self.vlc_instance.media_new(self.source)
    self.player.set_media(self.media)
    self.events.event_attach(vlc.EventType.MediaPlayerEndReached, self.playingFinished)
    self.player.set_rate(self.currentRate)
    self.player.audio_set_volume(self.currentVolume)
    self.player.play()
    # move cursor to the last reading position
    self.player.set_position(self.playingPos)

  def getId3Title(self, mpfile):
    idr = id3reader.Reader(mpfile)
    btit = idr.getRawData('TIT2')
    # when no tag is available, return real filename
    if btit is None:
      return os.path.basename(mpfile)
    elif btit[0] == 1:
      # first bit is 1, set encoding to UTF-8
      utfEncoding = True
    else:
      utfEncoding = False
    tagTitle = idr.getValue('TIT2')
    if not utfEncoding:
      tagTitle = tagTitle.encode('latin1', 'ignore').decode('windows-1250', 'ignore')
    return tagTitle
  
  def getFilenameforId3(self, tag, folder):
    # when name is not a tag but real filename, return this filename
    if os.path.splitext(tag)[1] == ".mp3":
      return tag
    for x in os.listdir(folder):
      if x.lower().endswith(".mp3"):
        if tag == self.getId3Title(os.path.join(folder, x)):
          fname = x
          break
    return fname
    
  def getListofFiles(self):
    dokFolder = self.dokPath
    fileList = []
    for x in os.listdir(dokFolder):
      if x.lower().endswith(".mp3"):
        fileList.append(self.getId3Title(os.path.join(dokFolder, x)))
    return fileList

  def fileContent(self, evt):
    global finalSelection
    WhatToDoDialog.selectionItems.clear()
    WhatToDoDialog.selectionItems = self.getListofFiles()
    curIndex = WhatToDoDialog.selectionItems.index(self.getId3Title(self.source))
    WhatToDoDialog.sposition = curIndex
    if WhatToDoDialog.selectionItems:
      WhatToDoDialog.selectionItems.append("Zpět")
    if WhatToDoDialog.selectionItems:
      if evt is not None:
        self.displayFileList(True)
      else:
        self.displayFileList()
      if finalSelection == "Zpět" or finalSelection == "":
        return
      idTitle = finalSelection
      dokFolder = self.dokPath
      curTitle = self.getFilenameforId3(idTitle, dokFolder)
      self.source = os.path.join(dokFolder, curTitle)
      self.media = self.vlc_instance.media_new(self.source)
      state = self.player.get_state()
      if state == vlc.State.Ended:
        self.events.event_detach(vlc.EventType.MediaPlayerEndReached)
        self.player = self.vlc_instance.media_player_new()
        self.events = self.player.event_manager()
        self.events.event_attach(vlc.EventType.MediaPlayerEndReached, self.playingFinished)
      self.player.set_media(self.media)
      self.player.set_rate(self.currentRate)
      self.player.audio_set_volume(self.currentVolume)
      self.player.play()

  def displayFileList(self, keypass=False):
    global finalSelection
    WhatToDoDialog.titleDlg = "Seznam souborů"
    self.Disable()
    try:
      WhatToDoDialog(self, keypass).ShowModal()
    except Exception as e:
      self.Enable()
      raise e

  def playingFinished(self, event):
    if not self.nextPlayingFile():
      ui.message("přehrávání skončilo.")

  def playingTime(self, pTime):
    curTime = datetime.datetime.utcfromtimestamp(pTime/1000.0)
    strTime = ""
    if curTime.hour > 0:
      strTime = strTime + str(curTime.hour)
      if curTime.hour == 1:
        strTime = strTime + " hodina"
      elif 1 <curTime.hour <5:
        strTime = strTime + " hodiny"
      else:
        strTime = strTime + " hodin"
      strTime = strTime + ", "
    if curTime.minute > 0:
      strTime = strTime + str(curTime.minute)
      if curTime.minute == 1:
        strTime = strTime + " minuta"
      elif 1 <curTime.minute <5:
        strTime = strTime + " minuty"
      else:
        strTime = strTime + " minut"
    if strTime:
      strTime = strTime + " a "
    strTime = strTime + str(curTime.second)
    if curTime.second == 1:
      strTime = strTime + " vteřina"
    elif 1 <curTime.second <5:
      strTime = strTime + " vteřiny"
    else:
        strTime = strTime + " vteřin"
    return strTime

  def nextPlayingFile(self):
    dirList = os.listdir(self.dokPath)
    try:
      dirList.remove("00-playlist.m3u")
    except: pass
    try:
      dirList.remove(KddMainDialog.okoMarkFilename)
    except: pass
    fileName = os.path.basename(self.source)
    fileIndex = dirList.index(fileName)
    fileIndex = fileIndex +1
    if fileIndex < len(dirList):
      self.source = self.source.replace(fileName, dirList[fileIndex])
      self.media = self.vlc_instance.media_new(self.source)
      state = self.player.get_state()
      if state == vlc.State.Ended:
        self.events.event_detach(vlc.EventType.MediaPlayerEndReached)
        self.player = self.vlc_instance.media_player_new()
        self.events = self.player.event_manager()
        self.events.event_attach(vlc.EventType.MediaPlayerEndReached, self.playingFinished)
      self.player.set_media(self.media)
      self.player.set_rate(self.currentRate)
      self.player.audio_set_volume(self.currentVolume)
      self.player.play()
      return True
    else:
      return False

  def prevPlayingFile(self):
    dirList = os.listdir(self.dokPath)
    try:
      dirList.remove(KddMainDialog.okoMarkFilename)
    except: pass
    try:
      dirList.remove("00-playlist.m3u")
    except: pass
    fileName = os.path.basename(self.source)
    fileIndex = dirList.index(fileName)
    fileIndex = fileIndex -1
    if fileIndex >= 0:
      self.source = self.source.replace(fileName, dirList[fileIndex])
      self.media = self.vlc_instance.media_new(self.source)
      self.player.set_media(self.media)
      self.player.set_rate(self.currentRate)
      self.player.audio_set_volume(self.currentVolume)
      self.player.play()
      return True
    else:
      return False

  def startstopPlaying(self, evt):
    state = self.player.get_state()
    if state == vlc.State.Ended:
      self.player.release()
      self.player = self.vlc_instance.media_player_new()
      self.player.set_media(self.media)
      self.player.play()
    elif state == vlc.State.Stopped:
      self.player.play()
    else:
      self.player.pause()
  
  def gotoStart(self, evt):
    self.player.set_position(0)
    speech.cancelSpeech()
    ui.message("Začátek.")

  def gotoEnd(self, evt):
    endTime = self.player.get_length()
    if self.player.is_playing():
      self.player.pause()
    self.player.set_time(endTime)
    speech.cancelSpeech()
    ui.message("Konec: " + self.playingTime(endTime))

  def announceCurrentPosition(self, evt):
    endTime = self.player.get_length()
    curTime = self.player.get_time()
    remTime = endTime - curTime
    speech.cancelSpeech()
    ui.message("Uplynulý čas: " + self.playingTime(curTime) + ", zbývající čas: " + self.playingTime(remTime))

  def gotonextPlayingFile(self, evt):
    if not self.nextPlayingFile():
      ui.message("Toto je poslední soubor.")

  def gotoprevPlayingFile(self, evt):
    if not self.prevPlayingFile():
      ui.message("Toto je první soubor.")

  def goForward(self, timeOffset):
    endTime = self.player.get_length()
    curTime = self.player.get_time()
    curTime = curTime + timeOffset
    if curTime > endTime:
      curTime = endTime
    self.player.set_time(curTime)
    if not self.player.is_playing():
      speech.cancelSpeech()
      ui.message(self.playingTime(curTime))

  def goBackward(self, timeOffset):
    curTime = self.player.get_time()
    curTime = curTime - timeOffset
    if curTime < 0:
      curTime = 0
    self.player.set_time(curTime)
    if not self.player.is_playing():
      speech.cancelSpeech()
      ui.message(self.playingTime(curTime))

  def changeRate(self, rate):
    self.currentRate = self.currentRate + rate
    if self.currentRate > 2.0:
      self.currentRate = 2.0
    if self.currentRate < 0.5:
      self.currentRate = 0.5
    self.player.set_rate(self.currentRate)
    if not self.player.is_playing():
      speech.cancelSpeech()
      ui.message(self.rateSpeech[self.currentRate] + " rychlost přehrávání.")

  def changeVolume(self, volume):
    self.currentVolume = self.currentVolume + volume
    if self.currentVolume > 100:
      self.currentVolume = 100
    if self.currentVolume < 0:
      self.currentVolume = 0
    self.player.audio_set_volume(self.currentVolume)
    if not self.player.is_playing():
      speech.cancelSpeech()
      ui.message(str(self.currentVolume) + " je hlasitost.")

  def audioPlayerKeyUp(self, evt):
    # performs according to acceleratortable on Backspace and Escape
    self.Parent.passKey = True
    self.onClose(self)

  def saveListeningPosition(self):
    from os.path import expanduser
    home = expanduser("~")
    bookmarkFile = os.path.join(self.dokPath, KddMainDialog.okoMarkFilename)
    tree = ET.parse(bookmarkFile)
    dokument = tree.getroot()
    dokPlPos = dokument.find("position")
    dokPlPos.text = str(self.playingPos)
    dokPlName = dokument.find("filetoread")
    dokPlName.text = os.path.basename(self.source)
    with open(bookmarkFile, 'wb') as f:
      f.write(ET.tostring(dokument, encoding='utf-8', method='xml'))

  def onCloseItem(self, evt):
    if evt != None:
      self.Parent.passKey = True
    self.onClose(self)

  def onClose(self, evt):
    self.playingPos = self.player.get_position()
    self.saveListeningPosition()
    self.Parent.Enable()
    self.Parent.afterReadingOrPlayingFinish()
    self.Parent.Show()
    self.Destroy()
    self.player.stop()


class KddUserDataDialog(wx.Dialog):

  def __init__(self, parent):
    
    # The title of a dialog.
    super(KddUserDataDialog, self).__init__(parent, title="Nastavení údajů o uživateli KDD")
    self.Bind(wx.EVT_CLOSE, self.onClose)

    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self,orientation=wx.VERTICAL)

    # The label of a user login text field
    userLoginLabel = "Zadejte uživatelské &jméno:"
    self.userLoginEdit = sHelper.addLabeledControl(userLoginLabel, wx.TextCtrl)

    # The label of a password text field
    userPasswordLabel = "Zadejte &heslo:"
    self.userPasswordEdit = sHelper.addLabeledControl(userPasswordLabel, wx.TextCtrl, style=wx.TE_PASSWORD)
  
    #button row
    sHelper_Buttons = guiHelper.BoxSizerHelper(self, orientation=wx.VERTICAL)

    # The label of an OK button 
    self.okButton = wx.Button(self, label="&OK")
    self.okButton.Bind(wx.EVT_BUTTON, self.onOk)
    self.AffirmativeId = self.okButton.Id
    self.okButton.SetDefault()
    sHelper_Buttons.addItem(self.okButton)

    # The label of a Cancel button 
    cancelButton = wx.Button(self, wx.ID_CLOSE, label=_(u"&Zrušit"))
    cancelButton.Bind(wx.EVT_BUTTON, lambda evt: self.Close())
    sHelper_Buttons.addDialogDismissButtons(cancelButton)
    self.Bind(wx.EVT_CLOSE, self.onCancel)
    self.EscapeId = wx.ID_CLOSE

    sHelper.addItem(sHelper_Buttons, flag = wx.CENTER)

    mainSizer.Add(sHelper.sizer, flag=wx.ALL, border=guiHelper.BORDER_FOR_DIALOGS)
    mainSizer.Fit(self)
    self.Sizer = mainSizer
    self.userLoginEdit.SetValue(kddUserLogin)
    self.userPasswordEdit.SetValue(kddUserPassword)
    self.userLoginEdit.SetFocus()
    self.CentreOnScreen()

  def onOk(self, evt):
    # assign field to variables
    global kddUserLogin
    global kddUserPassword    
    global kddUser
    kddUserLogin = self.userLoginEdit.Value
    kddUserPassword = self.userPasswordEdit.Value

    # save configuration
    global kddConfig
    kddConfig['KddUserLogin'] = kddUserLogin 
    kddConfig['KddUserPassword'] = base64.b64encode(kddUserPassword.encode("utf-8")).decode('utf-8')
    SetFile = os.path.join(libraryDataFolder, "configkdd.json")
    with open(SetFile, 'w') as f:
      json.dump(kddConfig, f)    
    KddMainDialog.passKey = True
    ui.message("Probíhá přihlašování do knihovny.")
    if kddLogin():
      dlgMessage = "Úspěšně jste se přihlásili jako " + kddUser + "."
      dlgTitle = "Informace"
      wx.MessageBox(dlgMessage, dlgTitle)
      KddMainDialog.passKey = True
    else:
      dlgMessage = "Nepodařilo se přihlásit."
      dlgTitle = "Informace"
      wx.MessageBox(dlgMessage, dlgTitle)
      KddMainDialog.passKey = True
    self.onClose(self)

  def onCancel(self, evt):
    self.onClose(self)

  def onClose(self, evt):
    self.Destroy()

class MathildaUserDataDialog(wx.Dialog):

  def __init__(self, parent):
    
    # The title of a dialog.
    super(MathildaUserDataDialog, self).__init__(parent, title="Nastavení údajů o uživateli Mathildy")
    self.Bind(wx.EVT_CLOSE, self.onClose)

    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self,orientation=wx.VERTICAL)

    # The label of a user login text field
    userLoginLabel = "Zadejte uživatelské &jméno:"
    self.userLoginEdit = sHelper.addLabeledControl(userLoginLabel, wx.TextCtrl)

    # The label of a password text field
    userPasswordLabel = "Zadejte &heslo:"
    self.userPasswordEdit = sHelper.addLabeledControl(userPasswordLabel, wx.TextCtrl, style=wx.TE_PASSWORD)
  
    #button row
    sHelper_Buttons = guiHelper.BoxSizerHelper(self, orientation=wx.VERTICAL)

    # The label of an OK button 
    self.okButton = wx.Button(self, label="&OK")
    self.okButton.Bind(wx.EVT_BUTTON, self.onOk)
    self.AffirmativeId = self.okButton.Id
    self.okButton.SetDefault()
    sHelper_Buttons.addItem(self.okButton)

    # The label of a Cancel button 
    cancelButton = wx.Button(self, wx.ID_CLOSE, label=_(u"&Zrušit"))
    cancelButton.Bind(wx.EVT_BUTTON, lambda evt: self.Close())
    sHelper_Buttons.addDialogDismissButtons(cancelButton)
    self.Bind(wx.EVT_CLOSE, self.onCancel)
    self.EscapeId = wx.ID_CLOSE

    sHelper.addItem(sHelper_Buttons, flag = wx.CENTER)

    mainSizer.Add(sHelper.sizer, flag=wx.ALL, border=guiHelper.BORDER_FOR_DIALOGS)
    mainSizer.Fit(self)
    self.Sizer = mainSizer
    self.userLoginEdit.SetValue(mathildaUserLogin)
    self.userPasswordEdit.SetValue(mathildaUserPassword)
    self.userLoginEdit.SetFocus()
    self.CentreOnScreen()

  def onOk(self, evt):
    # assign field to variables
    global mathildaUserLogin
    global mathildaUserPassword    
    mathildaUserLogin = self.userLoginEdit.Value
    mathildaUserPassword = self.userPasswordEdit.Value

    # save configuration
    global mathildaConfig
    mathildaConfig['MathildaUserLogin'] = mathildaUserLogin 
    mathildaConfig['MathildaUserPassword'] = base64.b64encode(mathildaUserPassword.encode("utf-8")).decode('utf-8')
    SetFile = os.path.join(libraryDataFolder, "configmathilda.json")
    with open(SetFile, 'w') as f:
      json.dump(mathildaConfig, f)    
    KddMainDialog.passKey = True
    dlgMessage = "Uživatelské údaje pro Mathildu byly uloženy."
    dlgTitle = "Informace"
    wx.MessageBox(dlgMessage, dlgTitle)
    self.onClose(self)

  def onCancel(self, evt):
    self.onClose(self)

  def onClose(self, evt):
    self.Destroy()


class SpvkUserDataDialog(wx.Dialog):

  def __init__(self, parent):
    
    # The title of a dialog.
    super(SpvkUserDataDialog, self).__init__(parent, title="Nastavení údajů o uživateli SPVK")
    self.Bind(wx.EVT_CLOSE, self.onClose)

    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self,orientation=wx.VERTICAL)

    # The label of a user login text field
    userLoginLabel = "Zadejte uživatelské &jméno:"
    self.userLoginEdit = sHelper.addLabeledControl(userLoginLabel, wx.TextCtrl)

    # The label of a password text field
    userPasswordLabel = "Zadejte &heslo:"
    self.userPasswordEdit = sHelper.addLabeledControl(userPasswordLabel, wx.TextCtrl, style=wx.TE_PASSWORD)
  
    #button row
    sHelper_Buttons = guiHelper.BoxSizerHelper(self, orientation=wx.VERTICAL)

    # The label of an OK button 
    self.okButton = wx.Button(self, label="&OK")
    self.okButton.Bind(wx.EVT_BUTTON, self.onOk)
    self.AffirmativeId = self.okButton.Id
    self.okButton.SetDefault()
    sHelper_Buttons.addItem(self.okButton)

    # The label of a Cancel button 
    cancelButton = wx.Button(self, wx.ID_CLOSE, label=_(u"&Zrušit"))
    cancelButton.Bind(wx.EVT_BUTTON, lambda evt: self.Close())
    sHelper_Buttons.addDialogDismissButtons(cancelButton)
    self.Bind(wx.EVT_CLOSE, self.onCancel)
    self.EscapeId = wx.ID_CLOSE

    sHelper.addItem(sHelper_Buttons, flag = wx.CENTER)

    mainSizer.Add(sHelper.sizer, flag=wx.ALL, border=guiHelper.BORDER_FOR_DIALOGS)
    mainSizer.Fit(self)
    self.Sizer = mainSizer
    self.userLoginEdit.SetValue(spvkUserLogin)
    self.userPasswordEdit.SetValue(spvkUserPassword)
    self.userLoginEdit.SetFocus()
    self.CentreOnScreen()

  def onOk(self, evt):
    # assign field to variables
    global spvkUserLogin
    global spvkUserPassword    
    spvkUserLogin = self.userLoginEdit.Value
    spvkUserPassword = self.userPasswordEdit.Value

    # save configuration
    global spvkConfig
    spvkConfig['SpvkUserLogin'] = spvkUserLogin 
    spvkConfig['SpvkUserPassword'] = base64.b64encode(spvkUserPassword.encode("utf-8")).decode('utf-8')
    SetFile = os.path.join(libraryDataFolder, "configspvk.json")
    with open(SetFile, 'w') as f:
      json.dump(spvkConfig, f)    
    KddMainDialog.passKey = True
    dlgMessage = "Uživatelské údaje pro SPVK byly uloženy."
    dlgTitle = "Informace"
    wx.MessageBox(dlgMessage, dlgTitle)
    self.onClose(self)

  def onCancel(self, evt):
    self.onClose(self)

  def onClose(self, evt):
    self.Destroy()


class KtnUserDataDialog(wx.Dialog):

  def __init__(self, parent):
    
    # The title of a dialog.
    super(KtnUserDataDialog, self).__init__(parent, title="Nastavení údajů o uživateli KTN")
    self.Bind(wx.EVT_CLOSE, self.onClose)

    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self,orientation=wx.VERTICAL)

    # The label of a user login text field
    userLoginLabel = "Zadejte uživatelské &jméno:"
    self.userLoginEdit = sHelper.addLabeledControl(userLoginLabel, wx.TextCtrl)

    # The label of a password text field
    userPasswordLabel = "Zadejte &heslo:"
    self.userPasswordEdit = sHelper.addLabeledControl(userPasswordLabel, wx.TextCtrl, style=wx.TE_PASSWORD)
  
    #button row
    sHelper_Buttons = guiHelper.BoxSizerHelper(self, orientation=wx.VERTICAL)

    # The label of an OK button 
    self.okButton = wx.Button(self, label="&OK")
    self.okButton.Bind(wx.EVT_BUTTON, self.onOk)
    self.AffirmativeId = self.okButton.Id
    self.okButton.SetDefault()
    sHelper_Buttons.addItem(self.okButton)

    # The label of a Cancel button 
    cancelButton = wx.Button(self, wx.ID_CLOSE, label=_(u"&Zrušit"))
    cancelButton.Bind(wx.EVT_BUTTON, lambda evt: self.Close())
    sHelper_Buttons.addDialogDismissButtons(cancelButton)
    self.Bind(wx.EVT_CLOSE, self.onCancel)
    self.EscapeId = wx.ID_CLOSE

    sHelper.addItem(sHelper_Buttons, flag = wx.CENTER)

    mainSizer.Add(sHelper.sizer, flag=wx.ALL, border=guiHelper.BORDER_FOR_DIALOGS)
    mainSizer.Fit(self)
    self.Sizer = mainSizer
    self.userLoginEdit.SetValue(ktnUserLogin)
    self.userPasswordEdit.SetValue(ktnUserPassword)
    self.userLoginEdit.SetFocus()
    self.CentreOnScreen()

  def onOk(self, evt):
    # assign field to variables
    global ktnUserLogin
    global ktnUserPassword    
    ktnUserLogin = self.userLoginEdit.Value
    ktnUserPassword = self.userPasswordEdit.Value

    # save configuration
    global ktnConfig
    ktnConfig['KtnUserLogin'] = ktnUserLogin 
    ktnConfig['KtnUserPassword'] = base64.b64encode(ktnUserPassword.encode("utf-8")).decode('utf-8')
    SetFile = os.path.join(libraryDataFolder, "configktn.json")
    with open(SetFile, 'w') as f:
      json.dump(ktnConfig, f)    
    KddMainDialog.passKey = True
    dlgMessage = "Uživatelské údaje pro KTN byly uloženy."
    dlgTitle = "Informace"
    wx.MessageBox(dlgMessage, dlgTitle)
    self.onClose(self)

  def onCancel(self, evt):
    self.onClose(self)

  def onClose(self, evt):
    self.Destroy()


class ZoraUserDataDialog(wx.Dialog):

  def __init__(self, parent):
    
    # The title of a dialog.
    super(ZoraUserDataDialog, self).__init__(parent, title="Nastavení údajů o uživateli Zory")
    self.Bind(wx.EVT_CLOSE, self.onClose)

    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self,orientation=wx.VERTICAL)

    # The label of a user login text field
    userLoginLabel = "Zadejte uživatelské &jméno:"
    self.userLoginEdit = sHelper.addLabeledControl(userLoginLabel, wx.TextCtrl)

    # The label of a password text field
    userPasswordLabel = "Zadejte &heslo:"
    self.userPasswordEdit = sHelper.addLabeledControl(userPasswordLabel, wx.TextCtrl, style=wx.TE_PASSWORD)
  
    #button row
    sHelper_Buttons = guiHelper.BoxSizerHelper(self, orientation=wx.VERTICAL)

    # The label of an OK button 
    self.okButton = wx.Button(self, label="&OK")
    self.okButton.Bind(wx.EVT_BUTTON, self.onOk)
    self.AffirmativeId = self.okButton.Id
    self.okButton.SetDefault()
    sHelper_Buttons.addItem(self.okButton)

    # The label of a Cancel button 
    cancelButton = wx.Button(self, wx.ID_CLOSE, label=_(u"&Zrušit"))
    cancelButton.Bind(wx.EVT_BUTTON, lambda evt: self.Close())
    sHelper_Buttons.addDialogDismissButtons(cancelButton)
    self.Bind(wx.EVT_CLOSE, self.onCancel)
    self.EscapeId = wx.ID_CLOSE

    sHelper.addItem(sHelper_Buttons, flag = wx.CENTER)

    mainSizer.Add(sHelper.sizer, flag=wx.ALL, border=guiHelper.BORDER_FOR_DIALOGS)
    mainSizer.Fit(self)
    self.Sizer = mainSizer
    self.userLoginEdit.SetValue(zoraUserLogin)
    self.userPasswordEdit.SetValue(zoraUserPassword)
    self.userLoginEdit.SetFocus()
    self.CentreOnScreen()

  def onOk(self, evt):
    # assign field to variables
    global zoraUserLogin
    global zoraUserPassword    
    zoraUserLogin = self.userLoginEdit.Value
    zoraUserPassword = self.userPasswordEdit.Value

    # save configuration
    global zoraConfig
    zoraConfig['ZoraUserLogin'] = zoraUserLogin 
    zoraConfig['ZoraUserPassword'] = base64.b64encode(zoraUserPassword.encode("utf-8")).decode('utf-8')
    SetFile = os.path.join(libraryDataFolder, "configzora.json")
    with open(SetFile, 'w') as f:
      json.dump(zoraConfig, f)    
    
    # verify enterred credentials
    successConnect = True
    zoraMp3Link = "https://archiv.sons.cz/zora/mp3/index.php"
    values = {'login_name': zoraUserLogin, 'login_pw': zoraUserPassword}
    query_string = urllib.parse.urlencode(values)
    data = query_string.encode()
    request = urllib.request.Request(zoraMp3Link, method="POST")
    request.data = data
    try:
      response = urllib.request.urlopen(request)
      if response.code != 200:
        successConnect = False
    except:
      successConnect = False
    if successConnect:
      content = response.read()
      content = content.decode("windows-1250", errors="ignore")
      # remove HTML, convert to plain text
      contenttxt = cleanhtml(content)
      # login is successful string
      indexActive = contenttxt.find("Aktivovány máte tyto časopisy:")
      # when user is not logged
      if indexActive == -1:
          successConnect = False
      if successConnect:
        dlgMessage = "Uživatelské údaje pro Zoru jsou správné a byly uloženy."
        dlgTitle = "Informace"
        wx.MessageBox(dlgMessage, dlgTitle)
        self.passKey = True
        KddMainDialog.passKey = True
        self.onClose(self)
      else:
        dlgMessage = "Zadané uživatelské údaje pro Zoru nejsou platné. Zkuste je, prosím, překontrolovat."
        dlgTitle = "Informace"
        wx.MessageBox(dlgMessage, dlgTitle)
        self.passKey = True

  def onCancel(self, evt):
    self.onClose(self)

  def onClose(self, evt):
    self.Destroy()


class KddSearchDialog(wx.Dialog):

  def __init__(self, parent):
    
    # The title of a dialog.
    super(KddSearchDialog, self).__init__(parent, title="Hledání")

    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self,orientation=wx.VERTICAL)

    # The search text fields
    searchNazevLabel = "&Název:"
    self.searchNazevEdit = sHelper.addLabeledControl(searchNazevLabel, wx.TextCtrl)

    searchAutorLabel = "&Autor:"
    self.searchAutorEdit = sHelper.addLabeledControl(searchAutorLabel, wx.TextCtrl)
  
    searchAnotaceLabel = "&Anotace:"
    self.searchAnotaceEdit = sHelper.addLabeledControl(searchAnotaceLabel, wx.TextCtrl)

    searchRokVydaniLabel = "&Rok vydání:"
    self.searchRokVydaniEdit = sHelper.addLabeledControl(searchRokVydaniLabel, wx.TextCtrl)

    searchNakladatelstviLabel = "Na&kladatelství:"
    self.searchNakladatelstviEdit = sHelper.addLabeledControl(searchNakladatelstviLabel, wx.TextCtrl)

    searchMistoVydaniLabel = "&Místo vydání:"
    self.searchMistoVydaniEdit = sHelper.addLabeledControl(searchMistoVydaniLabel, wx.TextCtrl)

    searchIsbnLabel = "&ISBN:"
    self.searchIsbnEdit = sHelper.addLabeledControl(searchIsbnLabel, wx.TextCtrl)

    searchNacitatelLabel = "N&ačitatel:"
    self.searchNacitatelEdit = sHelper.addLabeledControl(searchNacitatelLabel, wx.TextCtrl)

    searchVseLabel = "&Vše:"
    self.searchVseEdit = sHelper.addLabeledControl(searchVseLabel, wx.TextCtrl)

    #button row
    sHelper_Buttons = guiHelper.BoxSizerHelper(self, orientation=wx.VERTICAL)

    # The label of an OK button 
    self.okButton = wx.Button(self, id=wx.ID_OK, label="&OK")
    self.okButton.Bind(wx.EVT_BUTTON, self.onOk)
    self.AffirmativeId = self.okButton.Id
    self.okButton.SetDefault()
    sHelper_Buttons.addItem(self.okButton)

    # The label of a Cancel button 
    self.cancelButton = wx.Button(self, wx.ID_CANCEL, label="&Zrušit")
    self.cancelButton.Bind(wx.EVT_BUTTON, self.onCancel)
    sHelper_Buttons.addDialogDismissButtons(self.cancelButton)

    sHelper.addItem(sHelper_Buttons, flag = wx.CENTER)

    mainSizer.Add(sHelper.sizer, flag=wx.ALL, border=guiHelper.BORDER_FOR_DIALOGS)
    mainSizer.Fit(self)
    self.Sizer = mainSizer
    self.searchNazevEdit.SetFocus()
    self.CentreOnScreen()

  def onOk(self, evt):
    KddMainDialog.sNazev = self.searchNazevEdit.Value
    KddMainDialog.sAutor = self.searchAutorEdit.Value
    KddMainDialog.sAnotace = self.searchAnotaceEdit.Value
    KddMainDialog.sRokVydani = self.searchRokVydaniEdit.Value
    KddMainDialog.sNakladatelstvi = self.searchNakladatelstviEdit.Value
    KddMainDialog.sMistoVydani = self.searchMistoVydaniEdit.Value
    KddMainDialog.sIsbn = self.searchIsbnEdit.Value
    KddMainDialog.sNacitatel = self.searchNacitatelEdit.Value
    KddMainDialog.sVse = self.searchVseEdit.Value
    if self.searchNazevEdit.Value == "" and self.searchAutorEdit.Value == "" and self.searchAnotaceEdit.Value == "" and self.searchRokVydaniEdit.Value == "" and self.searchNakladatelstviEdit.Value == "" and self.searchMistoVydaniEdit.Value == "" and self.searchIsbnEdit.Value == "" and self.searchNacitatelEdit.Value == "" and self.searchVseEdit.Value == "":
      self.EndModal(False)
    else:
      self.EndModal(True)
    self.Destroy()

  def onCancel(self, evt):
    self.EndModal(False)
    self.Destroy()


def readcfg():
  global kddConfig
  global kddUserLogin
  global kddUserPassword
  global kddUpdate
  global kddUpdateBefore
  global kddCatSize
  if not os.path.isfile(os.path.join(libraryDataFolder, "configkdd.json")):
    kddConfig = {"KddUserLogin": "", "KddUserPassword": "", "KddUpdate": "", "KddUpdateBefore": "", "KddCatalogSize": 0}
    SetFile = os.path.join(libraryDataFolder, "configkdd.json")
    with open(SetFile, 'w') as f:
      json.dump(kddConfig, f)    
  SetFile = os.path.join(libraryDataFolder, "configkdd.json")
  with open(SetFile, 'r') as f:
    kddConfig = json.load(f)
  kddUserLogin = kddConfig['KddUserLogin']
  kddUserPassword = base64.b64decode(kddConfig['KddUserPassword']).decode("utf-8")
  kddUpdate = kddConfig['KddUpdate']
  kddUpdateBefore = kddConfig['KddUpdateBefore']
  # kddCatSize was added in oko 1.20 so it can be missing in config
  try:
    kddCatSize = kddConfig['KddCatalogSize']
  except:
    kddCatSize = 0

def readcfgZora():
  global zoraConfig
  global zoraUserLogin
  global zoraUserPassword
  if not os.path.isfile(os.path.join(libraryDataFolder, "configzora.json")):
    zoraConfig = {"ZoraUserLogin": "", "ZoraUserPassword": ""}
    SetFile = os.path.join(libraryDataFolder, "configzora.json")
    with open(SetFile, 'w') as f:
      json.dump(zoraConfig, f)    
  SetFile = os.path.join(libraryDataFolder, "configzora.json")
  with open(SetFile, 'r') as f:
    zoraConfig = json.load(f)
  zoraUserLogin = zoraConfig['ZoraUserLogin']
  zoraUserPassword = base64.b64decode(zoraConfig['ZoraUserPassword']).decode("utf-8")

def readcfgKtn():
  global ktnConfig
  global ktnUserLogin
  global ktnUserPassword
  global ktnCatalogIndex
  if not os.path.isfile(os.path.join(libraryDataFolder, "configktn.json")):
    ktnConfig = {"KtnUserLogin": "", "KtnUserPassword": "", "KtnCatalogIndex": "0"}
    SetFile = os.path.join(libraryDataFolder, "configktn.json")
    with open(SetFile, 'w') as f:
      json.dump(ktnConfig, f)    
  SetFile = os.path.join(libraryDataFolder, "configktn.json")
  with open(SetFile, 'r') as f:
    ktnConfig = json.load(f)
  ktnUserLogin = ktnConfig['KtnUserLogin']
  ktnUserPassword = base64.b64decode(ktnConfig['KtnUserPassword']).decode("utf-8")
  ktnCatalogIndex = ktnConfig['KtnCatalogIndex']

def readcfgSpvk():
  global spvkConfig
  global spvkUserLogin
  global spvkUserPassword
  if not os.path.isfile(os.path.join(libraryDataFolder, "configspvk.json")):
    spvkConfig = {"SpvkUserLogin": "", "SpvkUserPassword": ""}
    SetFile = os.path.join(libraryDataFolder, "configspvk.json")
    with open(SetFile, 'w') as f:
      json.dump(spvkConfig, f)    
  SetFile = os.path.join(libraryDataFolder, "configspvk.json")
  with open(SetFile, 'r') as f:
    spvkConfig = json.load(f)
  spvkUserLogin = spvkConfig['SpvkUserLogin']
  spvkUserPassword = base64.b64decode(spvkConfig['SpvkUserPassword']).decode("utf-8")

def readcfgMathilda():
  global mathildaConfig
  global mathildaUserLogin
  global mathildaUserPassword
  if not os.path.isfile(os.path.join(libraryDataFolder, "configmathilda.json")):
    mathildaConfig = {"MathildaUserLogin": "", "MathildaUserPassword": ""}
    SetFile = os.path.join(libraryDataFolder, "configmathilda.json")
    with open(SetFile, 'w') as f:
      json.dump(mathildaConfig, f)    
  SetFile = os.path.join(libraryDataFolder, "configmathilda.json")
  with open(SetFile, 'r') as f:
    mathildaConfig = json.load(f)
  mathildaUserLogin = mathildaConfig['MathildaUserLogin']
  mathildaUserPassword = base64.b64decode(mathildaConfig['MathildaUserPassword']).decode("utf-8")

def GetSubstrings(stringSubject,string1,string2):
  MyList = []
  intstart=0
  strlength=len(stringSubject)
  continueloop = 1

  while(intstart < strlength and continueloop == 1):
    intindex1=stringSubject.find(string1,intstart)
    if(intindex1 != -1): #The substring was found, lets proceed
      intindex1 = intindex1+len(string1)
      intindex2 = stringSubject.find(string2,intindex1)
      if(intindex2 != -1):
        subsequence=stringSubject[intindex1:intindex2]
        MyList.append(subsequence)
        intstart=intindex2+len(string2)
      else:
        continueloop=0
    else:
      continueloop=0
  return MyList

def cleanhtml(raw_html):
  cleanr = re.compile('<.*?>|&([a-z0-9]+|#[0-9]{1,6}|#x[0-9a-f]{1,6});')
  cleantext = re.sub(cleanr, '', raw_html)
  return cleantext

def strtoDate(s):
  import datetime
  sList = s.split(".")
  d = datetime.datetime(int(sList[2]), int(sList[1]), int(sList[0]))
  return d

def timestamptoDateTime(ts):
  import datetime
  dt = datetime.datetime(int(ts[0:4]), int(ts[4:6]), int(ts[6:8]), int(ts[8:10]), int(ts[10:12]), int(ts[12:14]))
  return dt

def removeNamespaceKtn():
  origXmlFile = os.path.join(libraryDataFolder, "MarcXML.xml")
  editXmlFile = os.path.join(libraryDataFolder, "ktncat.xml")
  nsStr = ' xmlns="http://www.loc.gov/MARC21/slim"'
  with open(origXmlFile, 'r', encoding="utf8") as f:
    with open(editXmlFile, 'w', encoding="utf8") as fw:
      while True:
        c = f.read(8192)
        if c == '':
          break
        if nsStr in c:
          c = c.replace(nsStr, "")
        fw.write(c)
      fw.close()
  os.remove(origXmlFile)

def GetUrlData(address):
  """Returns the contents of a web page"""
  e = data = ""
  try:
    with closing(urlopen(address)) as response:
      data = response.read()
  except Exception as e:
    if "CERTIFICATE_VERIFY_FAILED" in repr(e):
  #retry using ssl
      import ssl
      data = "no connect"
      gcontext = ssl.SSLContext(ssl.PROTOCOL_TLSv1)
      try:
        e, data = "", None
        with contextlib.closing(urllib2.urlopen(address, context=gcontext)) as response:
          data = response.read()
      except Exception as e: pass
    elif "failed" in repr(e): data = "no connect"

    if e:
      e = str(e)
  return data

def ktnCatDownload():
  isDownloaded = False
  global ktnCatalogIndex
  # reads the version from the server
  data = GetUrlData('https://suscik.cz/nvda/oko/ktncat.txt')
  try:
    data = data.decode()
  except: pass
  if not data or data == "no connect":
    return
  try:
    newVersion = data
  except: newVersion = ""
  newVersionValue = False
  if int(newVersion) > int(ktnCatalogIndex):
    newVersionValue = True
  # checks to see if a new version is available
  if newVersion and newVersionValue:
    # go ahead with catalog downloading
    url = "https://suscik.cz/nvda/oko/ktncat.zip"
    target = os.path.join(libraryDataFolder, "ktncat.zip")
    result = KddMainDialog.Download_file(KddMainDialog, url, target)
    ktnCatalogIndex = newVersion

    # save new catalog index
    ktnConfig['KtnCatalogIndex'] = ktnCatalogIndex
    SetFile = os.path.join(libraryDataFolder, "configktn.json")
    with open(SetFile, 'w') as f:
      json.dump(ktnConfig, f)    

    # unzip catalog
    unzipDone = True
    target = os.path.join(libraryDataFolder, "ktncat.zip")
    unzipFolder = libraryDataFolder
    import zipfile
    try:
      with zipfile.ZipFile(target, "r") as z:
        z.extractall(unzipFolder)
    except:
      unzipDone = False
    if unzipDone:
      os.remove(target)
      # remove namespace from KTN xml file
      removeNamespaceKtn()
      isDownloaded = True
  return isDownloaded

def ktnCatalog():
  global ktnConfig
  global newKtnCatalogDownloaded
  # download new catalog
  newKtnCatalogDownloaded = ktnCatDownload()
  return

def spvkCatalog():
  global spvkConfig
  # login to FTP
  try:
    server = FTP('ftp.spvk.cz', timeout=120)
    #server.set_pasv(False)
    server.login(user=base64.b64decode("bnZkYV9va28uc3B2ay5jeg==").decode("utf-8"), passwd = base64.b64decode("QVJTT05BTkljTG9QMUVudElPTjFpVGlwRlVSNA==").decode("utf-8"))
  except:
    return False
  catFileName = "catalog.zip"
  catFile = os.path.join(libraryDataFolder, "catalog.xml")
  if os.path.isfile(catFile):
    os.remove(catFile)
  localfile = open(os.path.join(libraryDataFolder, catFileName), 'wb')
  try:
    server.retrbinary('RETR ' + catFileName, localfile.write, 1024)
  except:
    return False
  localfile.close()
  # unzip catalog
  target = os.path.join(libraryDataFolder, catFileName)
  unzipFolder = os.path.join(libraryDataFolder)
  unzipDone = True
  import zipfile
  try:
    with zipfile.ZipFile(target, "r") as z:
      z.extractall(unzipFolder)
  except:
    unzipDone = False
  if unzipDone:
    os.remove(os.path.join(libraryDataFolder, catFileName))
  server.quit()
  return True

def kddCatalog():
  global kddConfig
  global kddUpdate
  global kddUpdateBefore
  global newKddCatalogDownloaded
  global kddCatSize
  newKddCatalogDownloaded = False
  # login to FTP to find out the date of KDD catalog
  server = FTP('kdd.cz')
  server.login(user=base64.b64decode("eG1sZGF0YV9rZGRfZG93bmxvYWQ=").decode("utf-8"), passwd = base64.b64decode("QWxTa0RqMjQ2X29ka2Rk").decode("utf-8"))
  catFileName = "dokumenty.zip"
  catFile = os.path.join(libraryDataFolder, "dokumenty.xml")
  catDownload = False
  timestamp = server.voidcmd("MDTM " + catFileName)[4:].strip()
  filesize = server.size(catFileName)
  # if catalog doesn't exist or catalog date in config is missing, it is necessary to download catalog
  if not os.path.isfile(catFile) or not kddUpdate:
    catDownload = True
  else:
    import datetime
    dtcatalog = timestamptoDateTime(timestamp)
    dtcatalogCurrent = timestamptoDateTime(kddUpdate)
    if dtcatalogCurrent < dtcatalog:
    # the catalog on FTP is newer, we check the size
      if filesize != kddCatSize:
      # the size of catalog on FTP is different, download is necessary
        catDownload = True
  if catDownload:
    localfile = open(os.path.join(libraryDataFolder, catFileName), 'wb')
    server.retrbinary('RETR ' + catFileName, localfile.write, 1024)
    localfile.close()
    # unzip catalog
    target = os.path.join(libraryDataFolder, catFileName)
    unzipFolder = os.path.join(libraryDataFolder)
    unzipDone = True
    import zipfile
    try:
      with zipfile.ZipFile(target, "r") as z:
        z.extractall(unzipFolder)
    except:
      unzipDone = False
    if unzipDone:
      os.remove(os.path.join(libraryDataFolder, catFileName))
    if kddUpdate:
      kddUpdateBefore = kddUpdate
    kddUpdate = timestamp
    kddCatSize = filesize
    # save catalog date and size to config
    kddConfig['KddUpdate'] = kddUpdate 
    kddConfig['KddUpdateBefore'] = kddUpdateBefore
    kddConfig['KddCatalogSize'] = kddCatSize
    SetFile = os.path.join(libraryDataFolder, "configkdd.json")
    with open(SetFile, 'w') as f:
      json.dump(kddConfig, f)    
    newKddCatalogDownloaded = True
  server.quit()

def kddLogin():
  global kddLogged
  global kddUser
  global kddCredits
  global kddBonusCredits
  global kddLastLogin
  if not kddUserLogin:
    kddLogged = False
    return kddLogged
  base_url = 'www.kdd.cz'
  https_base_url = 'https://' + base_url

  # here goes URL that's found inside form action='.....'
  authentication_url = https_base_url

  # we will use this string to confirm a login at end
  check_string = 'Odhlášení'

  # correct header
  headers={"Content-Type":"application/x-www-form-urlencoded",
  "User-agent":"Mozilla/5.0 Chrome/81.0.4044.92",    # Chrome 80+ as per web search
  "Host":base_url,
  "Origin":https_base_url,
  "Referer":https_base_url}

  # first a simple request, just to get login page
  request = urllib.request.Request(https_base_url)
  response = urllib.request.urlopen(request)
  contents = response.read()

  # here we craft our payload, it's all the form fields
  payload = {
      'login[username]':kddUserLogin,
      'login[password]':kddUserPassword
  }

  # now we prepare all we need for login
  #   data
  data = urllib.parse.urlencode(payload)
  binary_data = data.encode('UTF-8')
  # and put the URL + encoded data + correct headers into our POST request
  request = urllib.request.Request(authentication_url, binary_data, headers)
  response = urllib.request.urlopen(request, timeout=30)
  contents = response.read()

  # just for kicks, we confirm some element in the page that's secure behind the login
  #   we use a particular string we know only occurs after login
  contents = contents.decode("utf-8")
  index = contents.find(check_string)
  # if we find it
  if index != -1:
    kddLogged=True
  else:
    kddLogged=False
  if kddLogged:
    plainPage = cleanhtml(contents)

    # we find out user name
    searchStr = "Přihlášen: "
    startIndex = plainPage.find(searchStr) + len(searchStr)
    endIndex = plainPage.find("\n", startIndex)
    kddUser = plainPage[startIndex:endIndex]

    # we find out number of credits
    searchStr = "Stav účtu: "
    startIndex = plainPage.find(searchStr) + len(searchStr)
    endIndex = plainPage.find("\n", startIndex)
    x = plainPage[startIndex:endIndex].split()
    kddCredits = x[0]

    # we find out number of credits
    searchStr = "Stav bonusového účtu: "
    startIndex = plainPage.find(searchStr) + len(searchStr)
    endIndex = plainPage.find("\n", startIndex)
    x = plainPage[startIndex:endIndex].split()
    kddBonusCredits = x[0]

    # we find out last login date
    searchStr = "Datum posledního přihlášení: "
    startIndex = plainPage.find(searchStr) + len(searchStr)
    endIndex = plainPage.find("\n", startIndex)
    kddLastLogin = plainPage[startIndex:endIndex]
  else:
    kddUser = ""
    kddCredits = 0
    kddBonusCredits= 0
    kddLastLogin = ""
  return kddLogged

